/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * AuraXHR: struct used to represent a connection.
 *
 * @private
 */
Aura.Services.AuraClientServiceMarker = 0;

Aura.Services.AuraClientService$AuraXHR = function AuraXHR() {};

/**
 * Reset the xhr.
 */
Aura.Services.AuraClientService$AuraXHR.prototype.reset = function() {};

/**
 * add an action.
 */

Aura.Services.AuraClientService$AuraXHR.prototype.addAction = function(action) {};

/**
 * get an action for a response.
 */
Aura.Services.AuraClientService$AuraXHR.prototype.getAction = function(id) {};

/**
 * A handy structure to hold data.
 *
 * @private
 */
Aura.Services.AuraClientService$AuraActionCollector = function AuraActionCollector() {};


/**
 * @description The Aura Client Service, accessible using <code>$A.clientService</code>.
 *
 * There be dragons here.
 *
 * Manage the queue of actions sent to the server.
 *
 * Queue Processing Notes:
 *  * The queue is processed synchronously, but almost all of the interesting functionality occurs asynchronously.
 *  * client actions are run in a single flow of setTimeout calls.
 *  * storable server actions get processed in the "then" clause of the promise
 *  * non-storable server actions are processed synchronously.
 *
 * Input conditions:
 * We have a queue of actions that are inserted vie enqueueAction.
 *
 * Dangers:
 *  * Race conditions.
 *  * caboose actions.
 *  * problems with storage.
 *  * complexity of code due to asynchronicity.
 *
 * Tradeoffs:
 *  * number of actions boxcarred.
 *  * actions in flight
 *  * sending XHR early vs. late.
 *
 * Processing Loop:
 *  * gather actions into a collector.
 *  * walk each action, queueing up server actions to retrieve from storage and client actions to execute.
 *  * execute client actions with setTimeout(0) to allow server actions to complete.
 *  * As server actions come back from storage, queue up for execution, and queue refresh actions to refresh.
 *  * After all actions collect, check for further actions in the queue, restart loop if there are some.
 *  * Once we have finished processing all actions, check for actions to be put in an XHR.
 *    + All foreground actions go in a single XHR, and are de-duped on send.
 *    + background actions are sent one per XHR, with a de-dupe step during the queue walk.
 *    + deferred actions are sent if we are idle, with a de-dupe step during the queue walk.
 *
 * Queues:
 *  * actionsQueued - queue of actions that have yet to be processed.
 *  * actionsDeferred - actions that have been processed through storage, but need to go to the server.
 *
 * @constructor
 * @export
 */
function AuraClientService (util) {
    this._host = "";
    this._token = null;
    this._isDisconnected = false;
    this._parallelBootstrapLoad = true;
    this.authorizationToken = undefined;
    this.auraStack = [];
    this.actionStorage = new Aura.Controller.ActionStorage();
    this.appcacheDownloadingEventFired = false;
    this.isOutdated = false;
    this.finishedInitDefs = false;
    this.protocols={"layout":true};
    this.lastSendTime = Date.now();

    this.moduleServices = {};
    this.moduleSchemas = { "label" : this.labelSchemaResolver };
    this.moduleSchemasCache = {};

    // TODO: @dval We should send this from the server, but for LightningOut apps is a non-trivial change,
    // so for the time being I hard-coded the resource path here to ensure we can lazy fetch them.
    this.clientLibraries = {
        "ckeditor": { resourceUrl : "/auraFW/resources/{fwuid}/ckeditor/ckeditor-4.x/rel/ckeditor.js" },
        "quill": { resourceUrl : "/auraFW/resources/{fwuid}/quill.js" }
    };

    // Access Control
    this.accessStack=[];
    this.registeredNamespaces={internal:{},privileged:{}};
    this.currentAccess=null;
    this.enableAccessChecks=true;
    this.logAccessFailures= true
                            // Logging off by default in PROD mode
                            // #if {"modes" : ["PRODUCTION"]}
                            && false
                            // #end
                            ;

    // whether an appcache error event has been received
    this.appCacheError = false;

    // This will be only changed after the unload event
    this._appNotTearingDown = true;

    // XHR timeout (milliseconds)
    this.xhrTimeout = undefined;

    // bootstrap.js value used to boot the app. populated after a bootstrap version
    // is successfully loaded, processed, and merged.
    this.appBootstrap;

    // cookie name to force boostrap.js to the server (aka skip cache). done as a cookie so the server
    // can set this flag if ever required.
    this._disableBootstrapCacheCookie = "auraDisableBootstrapCache";

    // appcache progress.
    // = -1 on app cache error state.
    // = 0 on initial page load state or if appcache is not in use.
    // otherwise ranges from 0 to 100.
    this.appCacheProgress = 0;

    this.NOOP = function() {};

    var auraXHR = new Aura.Services.AuraClientService$AuraXHR();
    this.availableXHRs = [ auraXHR ];
    this.allXHRs = [ auraXHR ];
    this.actionStoreMap = {};
    this.collector = undefined;
    // if true will use one XHR to send each action (to be used with HTTP/2)
    this.xhrExclusivity = false;

    this.actionsQueued = [];
    this.actionsDeferred = [];

    // guess at xhr type.
    if (window.XMLHttpRequest) {
        this.httpType = 'generic';
    } else {
        this.httpType = undefined;
    }

    this._disconnected = undefined;

    // queue of functions to run when no XHRs in flight
    this.xhrIdleQueue = [];

    //
    // Run client actions synchronously. This is the previous behaviour.
    //
    this.optionClientSynchronous = true;

    this.reloadFunction = undefined;
    this.reloadPointPassed = false;

    // Shares token data across tabs to prevent unneeded page reloads.
    this.tokenSharing = util && util.isLocalStorageEnabled();
    this.maxActionRetries = 4;

    this.handleAppCache();
    this.setupBootstrapErrorReloadButton();
    this.setupTokenListener();
}

/**
 * Storage key for CSRF token.
 * TODO W-2531907 - Get/set done directly against the adapter because pre-200 non-Aura clients modified this value
 * direclty in SSA prior to Aura booting. Instead Aura should handle CSRF/invalid session issues, then this key/value
 * can operate against AuraStorage.
 * TODO W-2481519 - use a dedicated storage (not actions) for this framework-internal data to avoid eviction blacklists
 * in every adapter.
 */
AuraClientService.TOKEN_KEY = "$AuraClientService.token$";

/**
 * Storage key for bootstrap.js
 * TODO W-2481519 - use a dedicated storage (not actions) for this framework-internal data to avoid eviction blacklists
 * in every adapter.
 */
AuraClientService.BOOTSTRAP_KEY = "$AuraClientService.bootstrap$";

/**
 * Duration (in milliseconds) to check for boot progress. If progress is not seen after
 * this duration then reload the page.
 */
AuraClientService.BOOT_TIMER_DURATION = 30000;

/**
 * Number of consecutive reloads without fwk + app finishing the boot sequence. When this is exceeded
 * the user is notified of an error and prompted to reload.
 */
AuraClientService.INCOMPLETE_BOOT_THRESHOLD = 5;

/**
 * Query parameter to ensure a server trip even with appcache enabled. Servlet performs
 * prerequisite check (eg valid sid) then redirects back to app on success or elsewhere
 * on failure.
 */
AuraClientService.CACHE_BUST_QUERY_PARAM = "nocache";

/**
 * The status to return to action postprocess when receiving a response with system exception event
 */
AuraClientService.SYSTEM_EXCEPTION_EVENT_RETURN_STATUS = "SYSTEMERROR";

/**
 * The status to return to action postprocess when receiving a response with invalid session
 */
AuraClientService.INVALID_SESSION_RETURN_STATUS = "INVALIDSESSION";

/**
 * Reserved token that the server responds with in the case that a new TOKEN
 * cannot be issued.
 */
AuraClientService.INVALID_CSRF = "invalid_csrf";

/**
 * Framework + app reload counter to detect and prevent infinite reloads.
 *
 * Counter is cleared when Aura Framework finishes app instantiation. Counter is
 * tracked in sessionStorage so it's per-tab.
 */
AuraClientService.CONSECUTIVE_RELOAD_COUNTER_KEY = "__RELOAD_COUNT";

/**
 * fwuid used when logging a bootstrap error and context was not initialized
 */
AuraClientService.UNKNOWN_FRAMEWORK_UID = "UNKNOWN";

/**
 * Maximum length for action request query strings.
 */
AuraClientService.MAX_ACTION_QUERY_LENGTH = 1000;

/**
 * set the XHR queue size.
 *
 * This is a one time set for the queue size. Any further attempts will be ignored.
 * This should become a configuration parameter at some point.
 *
 * @private
 */
AuraClientService.prototype.setQueueSize = function(queueSize) {};

/**
 * set xhrExclusivity, if true will use one XHR to send each action (to be used with HTTP/2)
 *
 * @private
 */
AuraClientService.prototype.setXHRExclusivity = function(xhrExclusivity) {};

/**
 * Mark all currently queued (but not sent) actions as 'deferred'.
 *
 * This is intended for use when components are kept 'alive' after they are no longer on the screen for better
 * performance going back and forth between various displays.
 */
AuraClientService.prototype.deferPendingActions = function() {};

/**
 * @private
 */
AuraClientService.prototype.getSourceMapsUrl = function (descriptor, type) {};

/**
 * @private
 */
AuraClientService.prototype.uncommentExporter = function (exporter) {};

/**
 * @private
 */
AuraClientService.prototype.evalExporter = function(script, descriptor, type) {};

/**
 * Take a json (hopefully) response and decode it. If the input is invalid JSON, we try to handle it gracefully.
 *
 * @param {XmlHttpRequest} response the XHR object.
 * @param {Boolean} [noStrip] true to not strip off the JSON hijacking prevention (while(1) prefix).
 * @param {Boolean} [timedOut] true if the XHR timed out; false otherwise.
 * @returns {Object} An object with properties 'status', which represents the status of the response, and potentially
 *          'message', which contains the decoded server response or an error message.
 */
AuraClientService.prototype.decode = function(response, noStrip, timedOut) {};

/**
 * Fire an event exception from the wire.
 *
 * This is published, but only for use in the case of an event exception serialized as JS,
 * not sure if this is important.
 *
 * Exported to be used in ClientSideEventExceptionJSFormatAdapter to serialize JS exception handling code.
 *
 * @param {Object} resp The data for the exception event
 * @memberOf AuraClientService
 * @export
 */
AuraClientService.prototype.throwExceptionEvent = function(resp) {};

/**
 * Handler for remote NoAccessException
 */
AuraClientService.prototype.handleNoAccessException = function(values) {};

/**
 * Handler for remote ClientOutOfSyncException
 */
AuraClientService.prototype.handleClientOutOfSyncException = function() {};

/**
 * Handler for remote InvalidSessionException
 */
AuraClientService.prototype.handleInvalidSessionException = function(values) {};

/**
 * Handler for remote SystemErrorException
 */
AuraClientService.prototype.handleSystemErrorException = function() {};

/**
 * Handler for remote GenericEventException
 */
 AuraClientService.prototype.handleGenericEventException = function() {};

AuraClientService.prototype.fireDoneWaiting = function() {};

/**
 * This will be called by the unload event
 *
 * @private
 */
AuraClientService.prototype.tearDown = function() {};


/**
 * Initializes the clientLibraries sent as part of the lazy HTML scripts
 * This will be called before instanciating the app
 * @private
*/
AuraClientService.prototype.initializeClientLibraries = function () {};

/**
 * Reference an external JavaScript library which is registered on component by aura:clientLibrary.
 * It loads the required client library from the server if needed.
 *
 * @param {String} name - client library name
 * @param {Function} callback - a callback function that is executed if library is loaded
 *
 * @export
 */
AuraClientService.prototype.loadClientLibrary = function(name, callback) {};


/**
 * make the current thread be 'in aura collections'
 *
 * @private
 */
AuraClientService.prototype.setInCollection = function() {};

/**
 * release the current thread from 'in aura collections'
 *
 * @private
 */
AuraClientService.prototype.clearInCollection = function() {};

AuraClientService.prototype.isDisconnectedOrCancelled = function(response) {};

/**
 * Process a single action/response.
 *
 * Note that it does this inside an $A.run to provide protection against error returns, and to notify the user if an
 * error occurs.
 *
 * @param {Action} action the action.
 * @param {Object} actionResponse the server response.
 * @private
 */
AuraClientService.prototype.singleAction = function(action, actionResponse) {};

AuraClientService.prototype.isBB10 = function() {};

AuraClientService.prototype.getManifestURL = function() {};

AuraClientService.prototype.isManifestPresent = function() {};

/**
 * Count the available XHRs.
 */
AuraClientService.prototype.countAvailableXHRs = function(/*isBackground*/) {};

/**
 * Get an available XHR.
 *
 * Used for instrumentation
 *
 * @param {Boolean} isBackground is the XHR for a background action.
 */
AuraClientService.prototype.getAvailableXHR = function(isBackground) {};

/**
 * Release an xhr back in to the pool.
 *
 * @export
 */
AuraClientService.prototype.releaseXHR = function(auraXHR) {};

/**
 * Sets up the bootstrap error reload button.
 * @private
 */
AuraClientService.prototype.setupBootstrapErrorReloadButton = function() {};

/**
 * Gets a URL that will trigger a server trip, even with appcache enabled.
 * See important appcahce note on AuraClientService#hardRefresh.
 * @return {String} URL that will trigger a server trip.
 * @private
 */
AuraClientService.prototype.getHardRefreshURL  = function() {};

/**
 * Perform "hard refresh" by forcing a request for the .app to the server.
 *
 * IMPORTANT APPCACHE NOTE: if the .app is requested it'll be served from appcache. To
 * force a trip to the server a cache-busting URL parameter is used. This server trip
 * allows for an HTTP 302 redirect to:
 * 1. An alternate endpoint, like an auth page if the session is invalid.
 * 2. Back to the .app if the session, etc is valid.
 * Redirecting back to the .app will cause the browser to load the .app from appcache.
 * The browser will then start the appcache refresh cycle (by fetching the manifest; see
 * https://www.w3.org/TR/2011/WD-html5-20110525/offline.html#appcacheevents), and we rely
 * on the appcache event handlers (AuraClientService#handleAppCache) to respond.
 *
 * Other reload techniques, like window.location.reload(true), do not force a request to
 * the server.
 *
 * @memberOf AuraClientService
 * @export
 */
AuraClientService.prototype.hardRefresh = function() {};

AuraClientService.prototype.isDevMode = function() {};

/**
 * Clears caches (actions/GVP, ComponentDefStorage) then requests the .app
 * from the server.
 * @param {Object} [metricsPayload] optional payload to send to metrics service.
 * @private
 */
AuraClientService.prototype.actualDumpCachesAndReload = function(metricsPayload) {};

/**
 * Queues a request to clear caches (actions/GVP, ComponentDefStorage) then
 * reload the .app from the server.
 *
 * @param {Boolean} force True to force an immediate cache dump and reload. By default the
 * request is enqueued until framework has finished initialization. This should only be
 * used if $A.initAsync() will not be invoked (eg severe bootstrap error).
 * @param {Object} [metricsPayload] optional payload to send to metrics service.
 */
AuraClientService.prototype.dumpCachesAndReload = function(force, metricsPayload) {};

/**
 * Tracks consecutive reloads that do not reach $A.finishInit to prevent infinite reloads.
 *
 * AuraClientService#hardRefresh() relies on this function to indicate when too many
 * consecutive reloads have occurred. In that case the caches are cleared and the user
 * is given a prompt indicating the app can't boot.
 *
 * @returns {boolean} true if too many consecutive reloads without the fwk + app booting.
 */
AuraClientService.prototype.shouldPreventReload = function() {};

/**
 * Clears the counter tracking consecutive reloads.
 */
AuraClientService.prototype.clearReloadCount = function() {};

/**
 * Shows error dialog with reload button
 *
 * @param {AuraError} e error object
 * @param {String} [additionalLoggedMessage] additional text to log to the server, without being displayed to the user.
 * @private
 */
AuraClientService.prototype.showErrorDialogWithReload = function(e, additionalLoggedMessage) {};

/**
 * Sets up listeners for appcache.
 */
AuraClientService.prototype.handleAppCache = function() {};

/**
 * Gets the framework and app bootstrap state, which indicates execution state of each file.
 * @return {Object} loading state for all bootstrap files.
 * @private
 */
AuraClientService.prototype.getBootstrapState = function() {};

/**
 * Start or extend the boot timers that monitor for progress of the bootup sequence.
 * This is exclusively used in appcache scenarios: when an appcache error is triggered
 * the timer is started.
 */
AuraClientService.prototype.startBootTimers = function() {};

/**
 * Marks the application as outdated.
 *
 * Applications should call this when they require appcache to be refreshed. This
 * will result in the page being reloaded.
 *
 * @memberOf AuraClientService
 * @export
 */
AuraClientService.prototype.setOutdated = function() {};

/**
 * Updates appCache if online and status is IDLE
 * @returns {boolean} whether appcache update was performed
 */
AuraClientService.prototype.updateAppCacheIfOnlineAndIdle = function() {};

/**
 * Inform Aura that the environment is either online or offline.
 *
 * @param {Boolean} isConnected Set to true to run Aura in online mode,
 * or false to run Aura in offline mode.
 * @memberOf AuraClientService
 * @export
 */
AuraClientService.prototype.setConnected = function(isConnected) {};

/**
 * Init host is used to set the host name for communications.
 *
 * It should only be called once during the application life cycle, since it
 * will be deleted in production mode.
 *
 * Note that in testing, this can be used to make the host appear unreachable.
 *
 * @param {string} host the host name of the server.
 * @export
 */
AuraClientService.prototype.initHost = function(host) {};

/**
 * Initialize aura.
 *
 * FIXME: why is this exported
 *
 * This should never be called by client code.
 *
 * @param {Object} config the configuration for aura.
 * @param {string} token the XSS token.
 * @param {function} callback the callback when init is complete.
 * @param {object} container the place to install aura (defaults to document.body).
 * @export
 */
AuraClientService.prototype.init = function(config, token, container) {};

AuraClientService.prototype.getCurrentAccessName = function() {};

/**
 * Return the number of inFlightXHRs
 *
 * @export
 */
AuraClientService.prototype.inFlightXHRs = function(excludeBackground) {};

/**
 * This function is used by the test service to determine if there are outstanding actions.
 *
 * @private
 */
AuraClientService.prototype.idle = function() {};

/**
 * Enqueues a function to run when no XHRs are in-flight.
 * @param {Function} f the function to execute.
 */
AuraClientService.prototype.runWhenXHRIdle = function(f) {};

/**
 * Executes the queue of functions to run when no XHRs are in-flight.
 */
AuraClientService.prototype.processXHRIdleQueue = function() {};

/**
 * This function is used by the test service to determine if there are outstanding actions queued.
 *
 * @private
 */
AuraClientService.prototype.areActionsWaiting = function() {};

/**
 * Add privileged namespaces.
 *
 * @param {Array} a list of namespaces to mark privileged
 * @private
 */
AuraClientService.prototype.setNamespacePrivileges = function(sentNs) {};

/**
 * Initialize definitions.
 *
 * FIXME: why is this exported
 *
 * This should never be called by client code. It is exposed, but deleted after
 * first use.
 *
 * @param {Object} config the set of definitions to initialize
 * @export
 */

AuraClientService.prototype.initDefs = function() {};

/**
 * Gets the bootstrap.js payload, from network or cache.
 * @return {Object} the bootstrap payload and its source, or undefined if it hasn't yet loaded.
 */
AuraClientService.prototype.getAppBootstrap = function() {};

/**
 * Sets network bootstrap.js load status. To be invoked only after bootstrap.js &lt;script&gt;
 * has been processed (eg loaded successfully or not).
 *
 * This was historically triggered via the appcache fallback mechanism but with the reduced use
 * of appcache and the absence of a fallback for bootstrap.js, this is now triggered after all
 * bootstrap &lt;script&gt; tags are processed.
 *
 * @private
 */
AuraClientService.prototype.setAppBootstrapStatus = function() {};

AuraClientService.prototype.runAfterBootstrapReady = function (callback) {};

/**
 * Checks whether framework or app UID of cached and network has changed.
 * If either differs, dump caches and reload because we should not use stale defs in storage.
 * UIDs check is different from bootstrap md5 check. bootstrap md5 is from application bootstrap and models
 * whereas UIDs are framework code and application defs.
 *
 * @param {Object} boot bootstrap config
 */
AuraClientService.prototype.checkBootstrapUIDs = function(boot) {};

/**
 * Fire an aura:applicationRefreshed application level event if bootstrap returned from network differs from bootstrap
 * loaded from cache.
 * @private
 */
AuraClientService.prototype.checkBootstrapUpgrade = function() {};

/**
 * Run a callback after defs are initialized.
 *
 * This is for internal use only. The function is called synchronously if definitions have
 * already been initialized.
 *
 * @param {function} callback the callback that should be invoked after defs are initialized
 * @private
 */
AuraClientService.prototype.runAfterInitDefs = function(callback) {};

/**
 * Run a callback after the application is ready (rendered)
 *
 * This is for internal use only. The function is called synchronously if definitions have
 * already been initialized.
 *
 * @param {function} callback the callback that should be invoked after defs are initialized
 * @private
 */
AuraClientService.prototype.runAfterAppReady = function(callback) {};

/**
 * Loads bootstrap.js from storage, if it exists, and populates several
 * global variables consumed by runAfterBootstrapReady().
 *
 * Aura["appBootstrapCacheStatus"] must be set at the end of all code paths so
 * bootstrap robustness logic can determine when loading from storage is complete.
 *
 * @return {Promise} a promise that always resolves. Errors are logged
 *  and the promise resolves.
 */
AuraClientService.prototype.loadBootstrapFromStorage = function() {};

/**
 * Save bootstrap.js to storage.
 *
 * @param {Object} boot The bootstrap.js payload
 * @return {Promise} promise that resolves when bootstrap is saved to storage.
 *  If a storage error occurs then a flag is set to force bootstrap.js from
 *  network on next app load.
 */
AuraClientService.prototype.saveBootstrapToStorage = function(boot) {};

/**
 * Initializes app
 *
 * Waits for all the pieces (app.js, bootststrap.js) to init the application
 * @memberOf AuraClientService
 * @private
 */
AuraClientService.prototype.initializeApplication = function() {};

/**
 * Initializes injected services defined in the application.
 * @param {String[]} services Service descriptors to initialize.
 * @memberOf AuraClientService
 * @private
 */
AuraClientService.prototype.initializeInjectedServices = function(services) {};

/**
 * Adds a schema resolver so modules can statically import from it
 * Example: ` import url from "resource://myCustomResource" `
 *
 * This method needs to be exported (not platform)
 * so it can be invoked before framework initialization
 * @export
 */

AuraClientService.prototype.addModuleSchemaResolver = function (schema, resolver) {};

/**
 * Resolve a given schema `schema://resourceuri` from a module static import
 * Schemas can be defined:
 *   - By default in framework (like label://)
 *   - $A.clientService.addModuleSchemaResolver
 *   - Service injection at the app level.
 * This method just guards and invokes a resolver for a given schema if registered.
*/
AuraClientService.prototype.resolveSchemaDependency = function (schema, resourceUri, fullImport) {};


/**
 * Default resolver for label:// schema for module static imports
 * @memberOf AuraClientService
 * @private
 */
AuraClientService.prototype.labelSchemaResolver = function(resourceUri) {};

/**
 * Check to see if we are inside the aura processing 'loop'.
 *
 * @private
 */
AuraClientService.prototype.inAuraLoop = function() {};

/**
 * Push a new name on the stack.
 *
 * @param {string} name the name of the item to push.
 * @private
 */
AuraClientService.prototype.pushStack = function(name) {};

/**
 * Pop an item off the stack.
 *
 * The name of the item must match the previously pushed. If this is the last
 * item on the stack we do post processing, which involves sending actions to
 * the server.
 *
 * @param name the name of the last item pushed.
 * @private
 */
AuraClientService.prototype.popStack = function(name) {};

/*
 * @private
 */
AuraClientService.prototype.postProcess = function() {};

/**
 * Run the collection of actions.
 *
 * Entry point for processing actions. This creates a collector, and parcels out the action handling.
 * After this, server actions will be either getting values from storage, or will be executed, and the
 * client actions will all be queued up to be executed in order via setTimeout, giving server actions
 * entry points to collect.
 */
AuraClientService.prototype.process = function() {};

/**
 * continuation.
 *
 * Setp 2: walk actions setting up collections.
 *
 * We divide into client actions and server actions. Server actions are further
 * divided into stored vs. non-stored.
 */
AuraClientService.prototype.continueProcessing = function() {};

/**
 * Collect a storable action for subsequent bulk processing.
 * @param {Action} action The action to collect.
 * @param {Number} index The index of the array in the queue.
 */
AuraClientService.prototype.collectStorableAction = function(action, index) {};

/**
 * @param actionItem
 * @param response
 * @param callback who's first parameter is a boolean
 * @private
 */
AuraClientService.prototype.allDefsExistOnClient = function(actionItem, response, callback) {};

/**
 * Bulk process the storable actions collected by AuraClientService#collectStorableAction.
 * - Action storage is bulk queried for cached action results.
 * - If a cached result is found for an action then the action is processed immediately. Otherwise
 *   the action is enqueued for the server.
 * - If an error occurs querying storage then all actions are sent to the server.
 */
AuraClientService.prototype.processStorableActions = function() {};

/**
 * Bulk persist storable actions to storage.
 * @param {Array} actions An array of storable actions to persist.
 */
AuraClientService.prototype.persistStorableActions = function(actions) {};

/**
 * Enqueue a stored action for execution after the XHR send.
 */
AuraClientService.prototype.enqueueStoredAction = function(action, response) {};

/**
 * Execute a single stored action.
 *
 * This is done in situations when we get a result from the storage service. We also queue up a refresh
 * action if we are due a refresh or we have trouble running the action.
 */
AuraClientService.prototype.executeStoredAction = function(action, response, collected, index) {};

/**
 * Collect a single action into our list.
 */
AuraClientService.prototype.collectServerAction = function(action, index) {};

/**
 * Run client actions asynchronously.
 */
AuraClientService.prototype.continueClientActions = function() {};

/**
 * Run client actions synchronously.
 */
AuraClientService.prototype.runClientActions = function() {};

/**
 * Execute a client action.
 */
AuraClientService.prototype.executeClientAction = function(action) {};

/**
 * Finish the collection process and send XHRs.
 */
AuraClientService.prototype.finishCollection = function() {};

/**
 * @private
 */
AuraClientService.prototype.shouldSendOutForegroundActions = function( foregroundActions, cabooseCount ) {};

/**
 * Send actions.
 */
AuraClientService.prototype.sendActionXHRs = function() {};

/**
 * Send a group of actions as single action XHRs or re-enqueue them.
 *
 * All actions in the group will either be sent, marked as dupes, or put back in
 * the deferred queue.
 *
 * @private
 * @param {Array} actions the set of actions to send.
 * @param {int} count the number of actions to send.
 * @param {Options} options extra options for the send, allows callers to set headers and background option
 */
AuraClientService.prototype.sendAsSingle = function(actions, count, options) {};

/**
 * Continue with completions, running all action callbacks.
 *
 * This is used when the actions are stored, and we wish to run them after the XHRs
 * might have been sent.
 */
AuraClientService.prototype.continueCompletions = function() {};

/**
 * finish up processing, force a rerender.
 */
AuraClientService.prototype.finishProcessing = function() {};

/**
 * Check, and then dedupe actions that are duplicates.
 *
 * @param {Action} action the action to dedupe.
 * @param {Boolean} sending true if we are sending and should create an entry.
 * @return true if the action has been deduped.
 */
AuraClientService.prototype.deDupe = function(action, sending) {};

AuraClientService.prototype.getAndClearDupes = function(key) {};

/**
 * Send an xhr with a set of actions.
 *
 * The only note here is that if we fail to serialize the actions for any reason, we will log an
 * error and error out the actions. This is because we don't have a way of determining which of the
 * actions errored out.
 *
 * Used for instrumentation.
 *
 * @param auraXHR the wrapped XHR.
 * @param actions the set of actions to send.
 * @param method GET or POST. GET method is for publicly cacheable actions.
 * @param options extra options for the send, allows callers to set headers and background option
 * @return true if the XHR was sent, otherwise false.
 */
AuraClientService.prototype.send = function(auraXHR, actions, method, options) {};

/**
* Prepares the request
*/
AuraClientService.prototype.prepareRequest = function (actions) {};

/**
 * Send beacon
 *
 * @returns true if payload was successfully sent to the server.
 * sendBeacon will return false if the payload is too large for the browser to queue
 * At the time of this comment, the limit was 64kB, but that can change
 * @export
 */
AuraClientService.prototype.sendBeacon = function(action) {};

/**
 * Sets a timeout for use by the XHR timeout mechanism. Hook for testing.
 * @private
 */
AuraClientService.prototype.xhrSetTimeout = function(f) {};

/**
 * Clears a timeout used by the XHR timeout mechanism. Hook for testing.
 * @private
 */
AuraClientService.prototype.xhrClearTimeout = function(id) {};



/**
 * @returns {Object} An XHR based on what is available on the current browser.
 * @private
 */
AuraClientService.prototype.createXHR = function() {};

/**
 * Create an encoded string of parameters.
 *
 * @param {Map} map A map of parameter names and values
 * @returns {String} The encoded parameters
 * @private
 */
AuraClientService.prototype.buildParams = function(map) {};

/**
 * Create an encoded query string with action names and their occurrence count.
 *
 * @param {Action[]} actions  The list of actions.
 * @returns {String}          The encoded query string.
 * @private
 */
AuraClientService.prototype.buildActionNameList = function(actions, actionDefs) {};

/**
 * This function is only meant to be used for the corner case of preloading actions before Aura is available
 * It gets the actions that were preloaded ahead of time, a map with actionIds, and the prelaod XHR response object
 * Basically is like a regular server action but re-wiring the server results manually
 * @export
 */
AuraClientService.prototype.hydrateActions = function(actions, preloadMapId, response) {};

/**
 * This function is only meant to be used for the corner case of preloading actions before Aura is available
 * It gets the actions that were preloaded ahead of time, a map with actionIds, and the prelaod XHR response object
 * Basically is like a regular server action but re-wiring the server results manually
 *
 * @param {Object[]} rawResponses - collection of rawResponse objects with shape {{Number}status, {String}responseText}
 * @return {Promise<ReifyResult>} a promise that resolves to the actions contained in the rawResponse
 */
AuraClientService.prototype.reifyActions = function(rawResponses) {};

/**
 * Callback for an XHR for a set of actions.
 *
 * This function does all of the processing for a set of actions that come back from the server. It correctly deals
 * with the case of interrupted communications, and handles aborts.
 *
 * @param {AuraXHR} auraXHR the xhr container.
 * @param {Boolean} timedOut true if the XHR timed out, false otherwise.
 * @private
 */
AuraClientService.prototype.receive = function(auraXHR, timedOut) {};

/**
 * Retries in-flight actions on the given XHR due to a failed server response (invalidSession)
 * @param auraXHR originating auraXHR
 * @param event parsed server event in the originating response
 */
AuraClientService.prototype.retryActions = function(auraXHR, event) {};

/**
 * Returns true if the token is of valid format
 */
AuraClientService.prototype.isValidToken = function(token) {};

/**
 * Mark actions from an XHR response as being in the error state and set the error on the actions.
 *
 * @param {AuraXHR} auraXHR The xhr container.
 * @param {String} errorMessage The error message to associate with the actions.
 * @private
 */
AuraClientService.prototype.processErrors = function(auraXHR, errorMessage) {};

/**
 * Remove actions from storeMap when the response is SYSTEMERROR
 *
 * @private
 */
AuraClientService.prototype.processSystemError = function(auraXHR) {};

AuraClientService.prototype.addAllDefsToMap = function(defs, map) {};

AuraClientService.prototype.extractAllDefs = function(config) {};

AuraClientService.prototype.processResponses = function(auraXHR, responseMessage) {};

AuraClientService.prototype.buildStorableServerAction = function(response) {};

AuraClientService.prototype.processIncompletes = function(auraXHR) {};

/**
 * A utility to handle events passed back from the server.
 */
AuraClientService.prototype.parseAndFireEvent = function(evtObj) {};

/**
 * Set the token.
 *
 * @param {String} token The new token.
 * @param {Boolean} saveToStorage True to save the token to storage, false to not save.
 * @param {Boolean} broadcast True to broadcast token to existing windows/tabs
 * @memberOf AuraClientService
 * @private
 */
AuraClientService.prototype.setToken = function(newToken, broadcast) {};

/**
 * Broadcasts token to other open tabs to prevent stale token usage after a re-issue from the server.
 *
 * @param {String} newToken The new token to broadcast.
 * @private
 */
AuraClientService.prototype.broadcastToken = function(newToken) {};

/**
 * Establish event listener for receiving broadcasted tokens
 * @private
 */
AuraClientService.prototype.setupTokenListener = function() {};

/**
 * Set the token.
 *
 * @param {String} token the new token.
 * @memberOf AuraClientService
 * @export
 */
AuraClientService.prototype.resetToken = function(newToken) {};


/**
 * Inject a component and set up its event handlers. For Integration
 * Service.
 *
 * FIXME: this should be private.
 *
 * @param {Object} rawConfig the config for the component to be injected
 * @param {String} locatorDomId the DOM id where we should place our element.
 * @param {String} localId the local id for the component to be created.
 * @memberOf AuraClientService
 * @export
 */
AuraClientService.prototype.injectComponent = function(config, locatorDomId, localId) {};

/**
 * Create error component config to display integration service errors
 *
 * @param {(String|String[])} errorText
 * @returns {Object} error config for ui:message
 */
AuraClientService.prototype.createIntegrationErrorConfig = function(errorText) {};

/**
 * Used within async callback for AIS.
 *
 * @param {Component} component - component
 * @param {String} locator - parent element or the id of the parent element where to inject component
 * @param {Object} [actionEventHandlers] - event handlers
 */
AuraClientService.prototype.renderInjection = function(component, locator, actionEventHandlers) {};

/**
 * Use async created component for integration service
 *
 * @param {Object} config - component def config
 * @param {String} locator - parent element or the id of the parent element where to inject component
 * @param {Object} [eventHandlers] - handlers of registered event
 * @param {Function} callback The callback to use once the component is successfully created
 * @export
 */
AuraClientService.prototype.injectComponentAsync = function(config, locator, eventHandlers, callback) {};

/**
 * Add handlers of registered events for AIS
 *
 * @param {Component} component - component
 * @param {Object} [actionEventHandlers] - handlers of registered events
 */
AuraClientService.prototype.addComponentHandlers = function(component, actionEventHandlers) {};

/**
 * Return whether Aura believes it is online.
 * Immediate and future communication with the server may fail.
 * @memberOf AuraClientService
 * @return {Boolean} Returns true if Aura believes it is online; false otherwise.
 * @export
 */
AuraClientService.prototype.isConnected = function() {};

/**
 * This function must be called from within an event loop.
 *
 * @param {Action} action the action to enqueue
 * @param {Boolean} background Set to true to run the action in the background, otherwise the value of action.isBackground() is used.
 * @export
 * @public
 * @platform
 */
AuraClientService.prototype.enqueueAction = function(action, background) {};

/**
 * This function is used in enqueueAction when hotspot flow is set.
 *
 * @param {Action} action the action
 * @returns {Boolean} true if the action storage key is guaranteed to not be accessible from storage; false if accessible or unknown.
 * @private
 */
AuraClientService.prototype.isActionAbsentFromStorage = function(action) {};

/**
 * Determines whether an action is stored.
 *
 * @param {String} descriptor - action descriptor.
 * @param {Object} params - map of keys to parameter values.
 * @param {Function} callback - called asynchronously after the action was looked up in the cache. Fired with a
 * single parameter, isInStorge {Boolean} - representing whether the action was found in the cache.
 * @export
 */
AuraClientService.prototype.isActionInStorage = function(descriptor, params, callback) {};

/**
 * Resets the cache cleanup timer for an action.
 *
 * @param {String} descriptor - action descriptor.
 * @param {Object} params - map of keys to parameter values.
 * @param {Function} callback - called asynchronously after the action was revalidated. Called with a single
 * parameter, wasRevalidated {Boolean} - representing whether the action was found in the cache and
 * successfully revalidated.
 * @export
 */
AuraClientService.prototype.revalidateAction = function(descriptor, params, callback) {};

/**
 * Clears an action out of the action cache.
 *
 * @param {String} descriptor action descriptor.
 * @param {Object} params map of keys to parameter values.
 * @param {Function} successCallback called after the action was invalidated. Called with true if the action was
 * successfully invalidated and false if the action was invalid or was not found in the cache.
 * @param {Function} errorCallback called if an error occurred during execution
 * @export
 */
AuraClientService.prototype.invalidateAction = function(descriptor, params, successCallback, errorCallback) {};

// ACCESS CONTROL
AuraClientService.prototype.isInternalNamespace = function(namespace) {};

AuraClientService.prototype.isPrivilegedNamespace = function(namespace) {};

AuraClientService.prototype.getAccessStackHierarchy=function(){};

AuraClientService.prototype.setCurrentAccess=function(component){};

AuraClientService.prototype.releaseCurrentAccess=function(){};

AuraClientService.prototype.getAccessVersion = function(name) {};

AuraClientService.prototype.allowAccess = function(definition, component) {};

/**
 * Handles invalidSession exception from the server when the CSRF token is invalid.
 * Saves the new token to storage then refreshes page.
 * @export
 */
AuraClientService.prototype.invalidSession = function(newToken) {};

/**
 * Sets how Aura loads bootstrap.js: in parallel from network and cache,
 * or serially from network then cache. This must be called from a template's
 * auraPreInitBlock. By default this is enabled.
 * @param {Boolean} parallel if true parallelly load bootstrap.js from
 *  network and cache. If false load from network first (if it fails then
 *  load from cache).
 * @export
 */
AuraClientService.prototype.setParallelBootstrapLoad = function(parallel) {};

/**
 * On next load, serially load bootstrap.js from network then cache.
 *
 * If a valid CSRF token is not available then on next load bootstrap.js must
 * go to the server to fetch a new and valid CSRF token.
 * @private
 */
AuraClientService.prototype.disableParallelBootstrapLoadOnNextLoad = function() {};

/**
 * Clears disabling parallel load of bootstrap.js. See disableParallelBootstrapLoadOnNextLoad.
 * @private
 */
AuraClientService.prototype.clearDisableParallelBootstrapLoadOnNextLoad = function() {};

/**
 * Gets whether to check action cache for bootstrap.js.
 * @return {Boolean} true if the cache should be checked; false to skip the cache.
 */
AuraClientService.prototype.getParallelBootstrapLoad = function() {};

/**
 * This is a temporary API to workaround a broken network stack found on Samsung
 * Galaxy S5/S6 devices on Android 5.x.
 *
 * Sets the timeout for all Aura-initiated XHRs.
 *
 * The timeout applies to each XHR. The timer starts when XHR.send() is invoked
 * and ends when XHR.onreadystatechange (readyState = 4) is fired. If the timeout
 * expires before XHR.onreadystatechange then the actions in the XHR are moved to
 * INCOMPLETE state.
 *
 * @param {Number} timeout The XHR timeout in milliseconds.
 * @memberOf AuraClientService
 * @export
 */
AuraClientService.prototype.setXHRTimeout = function(timeout) {};

/**
 * Populates the persisted actions filter if applicable.
 * @return {Promise} a promise that resolves when the action keys are loaded.
 */
AuraClientService.prototype.populateActionsFilter = function() {};

/**
 * Enables or disables the persisted actions filter, required for multi-tab
 * environments that use storable actions.
 *
 * Actions can depend on defs. Defs are loaded from persistent storage at
 * framework init and so the same must be done for actions: loaded at framework
 * init. Otherwise storable actions may get cache hits that reference defs the
 * current tab does not have.
 *
 * @param {Boolean} enable true to enable the filter, false to disable the filter.
 * @export
 */
AuraClientService.prototype.setPersistedActionsFilter = function(enable) {};

AuraClientService.prototype.clearActionsFilter = function () {};

/**
 * Returns Action storage
 * @returns {ActionStorage}
 */
AuraClientService.prototype.getActionStorage = function() {};

/**
 * Returns name of Action storage
 * @returns {String} name of Action storage
 */
AuraClientService.prototype.getActionStorageName = function() {};

/**
 * Returns the globalId for the owner component.
 * Used on component instantiation.
 * @return {String} GlobalId of parent component
 *
 * @private
 */
AuraClientService.prototype.getCurrentAccessGlobalId = function () {};

/**
 * If set, an Authorization header will be set with the value of the given token for every request.
 *
 * @param {String} token value to be set for every request
 * @export
 */
AuraClientService.prototype.setAuthorizationToken = function (token) {};

Aura.Services.AuraClientService = AuraClientService;
