/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @description The Aura Component Service, accessible using <code>$A.componentService</code>.
 * Creates and manages components.
 * @constructor
 * @export
 */
function AuraComponentService() {}

/**
 * Gets an instance of a component.
 * @param {String} globalId The generated globally unique Id of the component that changes across pageloads.
 *
 * @public
 * @deprecated use getComponent instead
 * @export
 */
AuraComponentService.prototype.get = function(globalId) {};

AuraComponentService.prototype.initCoreModules = function () {};

/**
 * Adds to the mapping of case mismatched descriptors
 * @export
 */
AuraComponentService.prototype.addDescriptorCaseMapping = function(descriptorFrom, descriptorTo) {};

/**
 * Gets an instance of a component from either a GlobalId or a DOM element that was created via a Component Render.
 * @param {Object} identifier that is either a globalId or an element.
 *
 * @public
 * @platform
 * @export
 */
AuraComponentService.prototype.getComponent = function(identifier) {};

/**
 *  Validates as best it can that the config bag you specified is a ComponentDefRef that
 *  we can use to create instances of components from.
 */
AuraComponentService.prototype.isComponentDefRef = function(config) {};

/**
 * Gets descriptor from the config object (for normalization)
 * @param {Object} Controller descriptor config
 * @returns {String} Descriptor
 * @private
 */
AuraComponentService.prototype.getDescriptorFromConfig = function(descriptorConfig) {};

/**
 * Gets descriptor from the config object (for normalization)
 * @param {Object} Controller descriptor config
 * @returns {String} Descriptor
 * @private
 */
AuraComponentService.prototype.createDescriptorConfig = function(descriptorConfig) {};


/**
 * Counts all the components currently created in the application.
 * @example
 * var count = $A.componentService.countComponents();
 *
 * @public
 * @platform
 * @export
 */
AuraComponentService.prototype.countComponents = function() {};

/**
 * Calculates the locator for a given component and target
 *
 * @returns A fully resolved locator rooted at targetCmp
 * @param {Component} cmp The component that contains a locator targeting targetCmp
 * @param {String} targetCmp The component inside cmp that is targeted by a locator in cmp
 * @public
 * @export
 */
AuraComponentService.prototype.getComponentLocator = function(cmp, targetCmp, includeMetadata) {};

/**
 * Gets the rendering component for the provided element recursively.
 * @param {Object} element The element that is used to find the rendering component
 * @memberOf AuraComponentService
 * @public
 * @export
 */
AuraComponentService.prototype.getRenderingComponentForElement = function(element) {};

/**
 * Gets the attribute provider for the provided element.
 * @param {Object} element The element whose attribute provider is to be returned
 * @memberOf AuraComponentService
 * @public
 * @export
 */
AuraComponentService.prototype.getAttributeProviderForElement = function(element) {};

/**
 * Determines if the container contains cmp. The return value is an object
 * with two properties.
 *      - {Boolean} "result" true if container contains cmp; false otherwise
 *      - {Boolean} "isOwner" true if the containment is within the owner hierarchy;
 *          false if the containment is only by transclusion
 * @param {Component} container the container
 * @param {Component} cmp cmp to check
 * @return {Object}
 * @private
 */
AuraComponentService.prototype.containsComponent = (function() {})();


/**
 * Create a new component array.
 * @private
 */
AuraComponentService.prototype.newComponentArray = function(config, attributeValueProvider, localCreation, doForce){};

/**
 * Create a component from a type and a set of attributes.
 * It accepts the name of a type of component, a map of attributes,
 * and a callback to notify callers.
 *
 * @param {String} type The type of component to create, e.g. "ui:button".
 * @param {Object} attributes A map of attributes to send to the component. These take the same form as on the markup,
 * including events <code>{"press":component.getReference("c.handlePress")}</code>, and id <code>{"aura:id":"myComponentId"}</code>.
 * @param {Function} callback The method to call, to which it returns the newly created component.
 *
 * @example
 * $A.createComponent("aura:text",{value:'Hello World'}, function(auraTextComponent, status, statusMessagesList){
 *      // auraTextComponent is an instance of aura:text containing the value Hello World
 * });
 *
 * @public
 * @platform
 * @export
 */
AuraComponentService.prototype.createComponent = function(type, attributes, callback) {};


/*
 * Creates an internal object config for a component from a public config.
 *
 * This is only used by createComponentFromConfig to separate the internal
 * representation from the external representation.
 * If we change the component format, we could change this method
 * without breaking anyone's code.
 *
 * @function
 */
AuraComponentService.prototype.createInternalConfig = function (config) {};

/**
 * Creates a component from a config.
 *
 * It accepts a config Object generated directly by the framework
 * or a custom manually created config (see notes for details).
 *
 * IMPORTANT NOTES:
 *
 * - It's key that we separate the internal representation of a component
 * from the external one (publicly available), so we can always improve
 * and change the framework implementation without breaking anything.
 *
 * - Passing a user generated config is discouraged (instead createComponent
 * should be used). This method will only work for clientCreatable components
 * and for very simple use cases.
 *
 * @param {Object} config A map with the component tree configuration,
 * the configuration can be external (publicly exposed) or internal.
 *
 * {
        descriptor    : "markup://ns:cmpName",
        localId       : "localId",
        flavor        : "flavor",
        attributes    : { attr1: value1, ... },
        valueProvider : myValueProviderComponent
 * }
 *
 * @public
 * @function
 * @export
 */
AuraComponentService.prototype.createComponentFromConfig = function(config) {};

/**
 * Create an array of components from a list of types and attributes.
 * It accepts a list of component names and attribute maps, and a callback
 * to notify callers.
 *
 * @param {Array} components The list of components to create, e.g. <code>["ui:button",{"press":component.getReference("c.handlePress")}]</code>
 * @param {Function} callback The method to call, to which it returns the newly created components.
 *
 * @example $A.createComponents([
 *      ["aura:text",{value:'Hello'}],
 *      ["ui:button",{label:'Button'}],
 *      ["aura:text",{value:'World'}]
 *  ],function(components,status,statusMessagesList){
 *      // Components is an array of 3 components
 *      // 0 - Text Component containing Hello
 *      // 1 - Button Component with label Button
 *      // 2 - Text component containing World
 *  });
 *
 * @public
 * @platform
 * @function
 * @export
 */
AuraComponentService.prototype.createComponents = function(components, callback) {};

/**
 * newComponent() calls newComponentDeprecated().
 * @param {Object} config Use config to pass in your component definition and attributes. Supports lazy or exclusive loading by passing in "load": "LAZY" or "load": "EXCLUSIVE"
 * @param {Object} attributeValueProvider The value provider for the attributes
 *
 * @public
 * @deprecated use createComponent instead
 * @export
 */
AuraComponentService.prototype.newComponent = function(config, attributeValueProvider, localCreation, doForce){};


/**
 * Creates a new component on the client or server and initializes it. For example <code>$A.services.component.newComponentDeprecated("ui:inputText")</code>
 * creates a <code>ui:inputText</code> component.
 * @param {Object} config Use config to pass in your component definition and attributes. Supports lazy or exclusive loading by passing in "load": "LAZY" or "load": "EXCLUSIVE"
 * @param {Object} attributeValueProvider The value provider for the attributes
 *
 * @platform
 * @function
 * @deprecated use createComponent instead
 * @export
 */
AuraComponentService.prototype.newComponentDeprecated = function(config, attributeValueProvider, localCreation, doForce){};

/**
 * Takes a config for a component, and creates an instance of the component using the component class of that component.
 * @param {Object} config Config is the same object you would pass to the constructor $A.Component to create a component. This method will use that information to further configure the component class that is created.
 * @param {Boolean} localCreation See documentation on Component.js constructor for documentation on the localCreation property.
 */
AuraComponentService.prototype.createComponentInstance = function(config, localCreation, def) {};

/**
 * Use the specified constructor as the definition of the class descriptor.
 * We store them for execution later so we do not load definitions into memory unless they are utilized in getComponentClass.
 * @param {String} descriptor Uses the pattern of namespace:componentName.
 * @param {Function} exporter A function that when executed will return the component object litteral.
 * @export
 */
AuraComponentService.prototype.addComponentClass = function(descriptor, exporter){};

/**
 * Initialize modules
 * @param {String} descriptor Uses the pattern of namespace:componentName.
 * @param {Function} exporter A function that when executed will return the component object litteral.
 * @export
 */
AuraComponentService.prototype.initModuleDefs = function(modules) {};

/**
 * Use the specified constructor as the definition of the Module descriptor.
 * @param {String} descriptor Uses the pattern of namespace:componentName.
 * @param {String} name module custom element name
 * @param {Array} list of dependencies
 * @param {Function} exporter A function that when executed will return the component object literal
 * @export
 */
AuraComponentService.prototype.addModule = function(descriptor, name, dependencies, exporterClass, nsexports) {};

AuraComponentService.prototype.evaluateModuleDef = function (descriptor) {};

AuraComponentService.prototype.collectModuleStyles = function (Ctor) {};


AuraComponentService.prototype.flushModuleStyles = function () {};

AuraComponentService.prototype.createInteropComponentDef = function (descriptor) {};

/**
 * Get the class constructor for the specified component.
 * @param {String} descriptor use either the fqn markup://prefix:name or just prefix:name of the component to get a constructor for.
 * @returns Either the class that defines the component you are requesting, or undefined if not found.
 * @export
 */
AuraComponentService.prototype.getComponentClass = function(descriptor, def) {};

/**
 * Initializes event configs
 * @export
*/
AuraComponentService.prototype.initEventDefs = function(evtConfigs) {};

/**
 * Initializes library configs
 * @export
*/
AuraComponentService.prototype.initLibraryDefs = function(libraryConfigs) {};

/**
 * Init controller configs
 * @export
*/
AuraComponentService.prototype.initControllerDefs = function(controllerConfigs) {};

/**
 * Detects of the component class has been already defined without actually defining it.
 * hasComponentClass is more performant that running getComponentClass() since if the class
 * hasn't been built yet, we don't want it to be forcably built if not requested.
 *
 * @param {String} descriptor The qualified name of the component to check in the form prefix:componentname or protocol://prefix:componentname
 */
AuraComponentService.prototype.hasComponentClass = function(descriptor) {};

/**
 * Asynchronous version of newComponent(). Creates a new component and
 * calls your provided callback with the completed component regardless of any server-side dependencies.
 *
 * @param {Object} callbackScope The "this" context for the callback (null for global)
 * @param {Function} callback The callback to use once the component is successfully created
 * @param {Object} config The componentDef descriptor and attributes for the new component
 * @param {Object} attributeValueProvider The value provider for the attributes
 * @param {Boolean} [localCreation] Whether created client side (passed to Component)
 * @param {Boolean} [doForce] Whether to force client side creation
 * @param {Boolean} [forceServer] Whether to force server side creation
 *
 * @deprecated Use <code>$A.createComponent(String type, Object attributes, function callback)</code> instead.
 * @platform
 * @export
 */
AuraComponentService.prototype.newComponentAsync = function(callbackScope, callback, config, attributeValueProvider, localCreation, doForce, forceServer) {};

AuraComponentService.prototype.hasCacheableDefinitionOfAnyType = function(descriptor) {};

AuraComponentService.prototype.loadComponentDefs = function(descriptorMap, callback) {};

/**
 * Request component from server.
 *
 * @param config
 * @param callback
 * @private
 */
AuraComponentService.prototype.requestComponent = function(callback, config, avp, index, returnNullOnError) {};

/**
 * Evaluates value object into their literal values. Typically used to pass configs to server.
 *
 * @param {Object} valueObj Value Object
 * @param {Object} valueProvider value provider
 *
 * @returns {*}
 * @export
 */
AuraComponentService.prototype.computeValue = function(value, valueProvider) {};

/**
 * Provides processed component config, definition, and descriptor.
 *
 * @param {Object} config
 * @param {Object} attributeValueProvider
 * @return {Object} {{configuration: {}, definition: ComponentDef, descriptor: String}}
 */
AuraComponentService.prototype.getComponentConfigs = function(config, attributeValueProvider) {};

/**
 * Indexes the component using its global Id, which is uniquely generated across pageloads.
 * @public
 */
AuraComponentService.prototype.indexComponent = function(component){};

/**
 * Checks to see if the definition for the component currently reside on the client and the context has access to it.
 * Could still exist on the server, we won't know that till we use a getDefinition call to try to retrieve it.
 *
 * This method is private, to use it, use $A.hasDefinition("prefix:name");
 *
 * @private
 * @param  {String}  descriptor Component descriptor in the pattern prefix:name or markup://prefix:name.
 * @return {Boolean}            True if the definition is present on the client.
 */
AuraComponentService.prototype.hasDefinition = function(descriptor) {};


/**
 * Return the definition of the components that were not used yet (we have the def config but we haven't build the def instance)
 * @export
 */
AuraComponentService.prototype.getUnusedDefinitions = function () {};

/**
 * Get the component definition. If it is not available will go to the server to retrieve it.
 *
 * This method is private, to utilize it, you should use $A.getDefinition("prefix:markup");
 *
 * @private
 *
 * @param  {String}   descriptor Component descriptor in the pattern prefix:name or markup://prefix:name.
 * @param  {Function} callback   Function that is passed the definition. The definition may be NULL if either the definition does not exist, or you do not have access to it.
 * @return undefined             Always use the callback to access the returned definition.
 */
AuraComponentService.prototype.getDefinition = function(descriptor, callback) {};

/**
 * Gets the component definition from the registry for internal use, without access checks.
 *
 * @param {Object} descriptor The descriptor object.
 * @returns {ComponentDef} The metadata of the component
 *
 * @private
 */
AuraComponentService.prototype.getComponentDef = function(config) {};

AuraComponentService.prototype.hasDefinition = function(descriptor) {};

/**
 * Whether module definition exist in registry
 *
 * moduleName may be in module format ie custom-web-component or
 * Aura descriptor format ie markup://custom:webComponent
 *
 * @param moduleName name in module format or Aura descriptor
 * @returns {boolean} whether module definition exists
 * @private
 */
AuraComponentService.prototype.hasModuleDefinition = function(moduleName) {};

/**
 * Gets the component definition from the registry.
 * Does not go to the server if the definition is not available.
 *
 * @param {String|Object} descriptor The descriptor (<code>markup://ui:scroller</code>) or other component attributes that are provided during its initialization.
 * @returns {ComponentDef} The metadata of the component
 *
 * @public
 * @export
 * @deprecated use getDefinition(descriptor, callback) instead, it will go to the server if the definition is not present on the client.
 */
AuraComponentService.prototype.getDef = function(descriptor) {};

/**
 * Add a component to the registry
 * We store them for execution later so we do not load definitions into memory unless they are utilized in getComponent.
 * @param {String} descriptor Uses the pattern of namespace:componentName.
 * @param {Function} exporter A function that when executed will return the component object litteral.
 * @export
 */
AuraComponentService.prototype.addComponent = function(descriptor, exporter) {};


AuraComponentService.prototype.hydrateComponent = function(descriptor, exporter) {};

/**
 * Checks for saved component config, creates if available, and deletes the config
 *
 * @param {String} descriptor component descriptor to check and create
 * @return {ComponentDef} component definition if config available
 * @private
 */
AuraComponentService.prototype.createFromSavedComponentConfigs = function(config) {};

/**
 * Creates ComponentDef from provided config
 * @param {Object} config component definition config
 * @return {ComponentDef}
 * @private
 */
AuraComponentService.prototype.createComponentDef = function(config) {};

/**
 * Gets the component's controller definition from the registry.
 * @param {String} descriptor controller descriptor
 * @returns {ControllerDef} ControllerDef from registry
 * @private
 */
AuraComponentService.prototype.getControllerDef = function(descriptor) {};

/**
 * Creates and returns ControllerDef
 * @param {Object} config Configuration for ControllerDef
 * @returns {ControllerDef} ControllerDef from registry
 * @private
 */
AuraComponentService.prototype.createControllerDef = function(config) {};

/**
 * Gets the action definition from the registry.
 * @param {String} descriptor actionDef descriptor
 * @returns {ActionDef} ActionDef from registry
 * @private
 */
AuraComponentService.prototype.getActionDef = function(descriptor) {};

/**
 * Creates and returns ActionDef
 * @param {Object} config Configuration for ActionDef
 * @returns {ActionDef} ControllerDef from registry
 * @private
 */
AuraComponentService.prototype.createActionDef = function(config) {};

/**
 * Gets the model definition from the registry.
 * @param {String} descriptor ModelDef descriptor
 * @returns {ModelDef} ModelDef from registry
 * @private
 */
AuraComponentService.prototype.getModelDef = function(descriptor) {};

/**
 * Creates and returns ModelDef
 * @param {Object} config Configuration for ModelDef
 * @returns {ModelDef} ModelDef from registry
 * @private
 */
AuraComponentService.prototype.createModelDef = function(config) {};

/**
 * Detects if the library exists without actually defining it.
 * @param {String} descriptor The qualified name of the library in the form markup://namespace:library
 */
AuraComponentService.prototype.hasLibrary = function(descriptor) {};

/**
 * Stores a library definition.
 * @param {Object} config component definition config
 * @export
 */
AuraComponentService.prototype.saveLibraryConfig = function(config) {};

/**
 * Store a library exporter.
 * @param {String} descriptor name of the include.
 * @param {Function} exporter A function that when executed will return the include object.
 * @export
 */
AuraComponentService.prototype.addLibraryExporter = function(descriptor, exporter) {};

/**
 * Get a library from the registry.
 * @param {String} descriptor library descriptor.
 * @returns {Object} library from registry.
 * @private
 */
AuraComponentService.prototype.getLibrary = function(descriptor) {};

/**
 * Get a library as a module from the registry.
 * @param {String} descriptor library descriptor.
 * @returns {Object} library from registry.
 * @private
 */
AuraComponentService.prototype.getLibraryAsModule = function(descriptor) {};

/**
 * Store a library include.
 * @param {String} descriptor name of the include.
 * @param {Array} dependencies The list of dependencies (other includes).
 * @param {Function} exporter A function that when executed will return the include object.
 * @export
 */
AuraComponentService.prototype.addLibraryInclude = function(descriptor, dependencies, exporter) {};

/**
 * Get a library include from the registry.
 * @param {String} descriptor in the form markup://namespace:include.
 * @returns Either the instance of the include you are requesting, or undefined if not found.
 * @private
 */
AuraComponentService.prototype.getLibraryInclude = function(descriptor) {};

/**
 * Destroys the components.
 * @private
 */
AuraComponentService.prototype.destroy = function(components){};

/**
 * Removes the index of the component.
 * @private
 */
AuraComponentService.prototype.deIndex = function(globalId){};

/**
 * Returns the descriptors of all components known to the registry.
 * @memberOf AuraComponentService
 * @public
 * @export
 */
AuraComponentService.prototype.getRegisteredComponentDescriptors = function(){};

/**
 * Get the dynamic namespaces defined by 'layout://name'
 */
AuraComponentService.prototype.getDynamicNamespaces = function(){};

/**
 * @memberOf AuraComponentService
 * @export
 */
AuraComponentService.prototype.getIndex = function(){};

/**
 * @memberOf AuraComponentService
 * @private
 */
AuraComponentService.prototype.isConfigDescriptor = function(config) {};

/**
 * Saves component config so it can be use later when component def is actually used.
 * Allows Aura to only create ComponentDef when needed
 *
 * Also save reference to componentDef for its ControllerDef and ActionDefs in cases
 * where direct access to the defs are needed
 *
 * @param {Object} config component definition config
 */
AuraComponentService.prototype.saveComponentConfig = function(config) {};

/**
 * Asynchronously retrieves all definitions from storage and adds to saved component config or library registry.
 * @return {Promise} a promise that resolves when definitions are restored.
 */
AuraComponentService.prototype.restoreDefsFromStorage = function (context) {};

/**
 * Clears persisted definitions and all dependent stores and context.
 * @param {Object} [metricsPayload] optional payload to send to metrics service.
 * @return {Promise} Promise when storage is cleared
 */
AuraComponentService.prototype.clearDefsFromStorage = function (metricsPayload) {};

/**
 * Returns name of ComponentDefStorage
 * @returns {String} name of ComponentDefStorage
 */
AuraComponentService.prototype.getComponentDefStorageName = function() {};

/**
 * Saves component and library defs to persistent storage.
 * @param {Object} config the config bag from which defs are to be stored.
 * @param {Object} context the context (already merged)
 * @return {Promise} promise which resolves when storing is complete. If errors occur during
 *  the process error recovery occurs. If error recovery fails then the promise rejects.
 */
AuraComponentService.prototype.saveDefsToStorage = function (config, context) {};


AuraComponentService.prototype.createComponentPrivAsync = function (config, callback, infiniteLoopProtection) {};

AuraComponentService.prototype.createComponentPriv = function (config, callback) {};

/*
 * ====================================================================
 * PERSISTENT CACHE EVICTION LOGIC
 * ====================================================================
 */

/**
 * Find dependencies of a component def or action.
 * @param {String} key the component def or action id.
 * @param {Object} defConfig
 * @param {Array} storedDeps
 * @return {Array} the list of dependencies
 */
AuraComponentService.prototype.findDependencies = function (key, defConfig, storedDeps) {};

/**
 * Builds the dependency graph for for all persisted component definitions and stored actions (persistent or not).
 *
 * A component definition depends on components in its superDef chain and its facets.
 * A stored action depends on components specified in its return value.
 * This definition is recursive.
 *
 * For example:
 * - Three components: plant, tree, leaf.
 * - Tree's superDef is plant.
 * - Tree has leaf in its facet.
 * - An action was used to retrieve an instance of tree.
 *
 * Tree's graph node looks like this:
 * { action: false, id: "markup://ns:tree", dependencies: ["markup://ns:plant", "markup://ns:leaf"] }
 *
 * The action's graph node looks like this:
 * { action: true, id: "java://.../ACTION$getTree:{}", dependencies: ["markup://ns:tree", "markup://ns:plant", "markup://ns:leaf"] }
 *
 * @return {Promise} promise that resolves to an object (keyed on descriptor) of dependency objects. A dependency object has these keys:
 * - action: true if this is an action, false if a component definition.
 * - id: the action or def descriptor.
 * - dependencies: array of action/def descriptors this item depends on.
 */
AuraComponentService.prototype.buildDependencyGraph = function() {};

/**
 * Sorts the dependency graph by topological order.
 * @param {Object} graph a graph of nodes. See #buildDependencyGraph().
 * @return {Array} a topologically sorted array of node ids.
 */
AuraComponentService.prototype.sortDependencyGraph = function(graph) {};


/**
 * Gets the "upstream" dependencies for a node in the graph. In other words, gets the nodes from the
 * graph that are dependent, directly or indirectly, on a given node.
 *
 * This provides the list of component definitions and actions which must be removed if a given
 * component definition is removed.
 *
 * For example:
 * - Three components: plant, tree, leaf.
 * - Tree's superDef is plant.
 * - Tree has leaf in its facet.
 * - An action was used to retrieve an instance of tree.
 *
 * If node is leaf then the upstream dependencies are:
 * - tree because it has leaf in a facet
 * - action because it depends on tree
 *
 * Notably plant is not an upstream dependency. It does not depend on leaf, tree or action.
 *
 * @param {String} rootKey key of the graph node whose upstream dependencies are desired.
 * @param {Object} graph a graph of nodes. See #buildDependencyGraph().
 * @param {Object} upstream map to populate with upstream dependencies. Key is the graph key; value is always true.
 */
AuraComponentService.prototype.getUpstreamDependencies = function(rootKey, graph, upstream) {};

/**
 * Separates the keys into actions and defs, pruning those that appear in exclude.
 * @param {Object} graph a graph of nodes. See #buildDependencyGraph().
 * @param {Object} keys a map whose keys are graph keys.
 * @param {Array} exclude the list of keys to exclude.
 * @param {Array} actions the array to populate with action keys.
 * @param {Array} defs the array to populate with component def keys.
 */
AuraComponentService.prototype.splitComponentsAndActions = function(graph, keys, exclude, actions, defs) {};

/**
 * Evicts component definitions and dependent actions from storage until the
 * component def storage is under a size threshold.
 *
 * @param {Array} sortedKeys the ordered list of graph keys to remove under the size threshold is met.
 * @param {Object} graph a graph of nodes. See #buildDependencyGraph().
 * @param {Number} requiredSpaceKb space required to store incoming defs.
 * @return {Promise} a promise that resolves with the list of evicted actions and component defs.
 */
AuraComponentService.prototype.evictDefsFromStorage = function(sortedKeys, graph, requiredSpaceKb) {};

/**
 * Prunes component definitions and dependent actions from persistent storage.
 *
 * This is the entry point for dependency graph generation, analysis, and
 * eviction. Eviction proceeds until the component def storage is under a threshold
 * size or all component defs are evicted from storage.
 *
 * @param {Number} requiredSpaceKb space (in KB) required by new configs to be stored.
 * @return {Promise} a promise that resolves when pruning is complete.
 */
AuraComponentService.prototype.pruneDefsFromStorage = function(requiredSpaceKb) {};

/**
 * Returns the number of batches fetching components from the server
 *
 * @return {Number} Number of component batch loads in progress
 * @export
 */
AuraComponentService.prototype.inFlightComponents = function() {};

Aura.Services.AuraComponentService = AuraComponentService;
