/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description The Aura Event Service, accessible using <code>$A.eventService</code>. Creates and manages events.
 * @constructor AuraEventService
 * @export
 */
function AuraEventService () {
    this.eventDispatcher   = {};
    this.eventDefRegistry  = {};
    this.savedEventConfigs = {};
    this.componentHandlers = {};
}

AuraEventService.Phase = {
    CAPTURE: "capture",
    BUBBLE: "bubble",
    DEFAULT: "default"
};

AuraEventService.validatePhase=function(phase, defaultPhase){};

/**
 * Creates a new application event. Set the event parameters using <code>event.setParams()</code> and fire
 * it using <code>event.fire()</code>. For example, <code>$A.eventService.newEvent("app:navError")</code>
 * fires the <code>app:navError</code> event. Set parameters on the new event
 * by using <code>event.setParams()</code>.
 *
 * @param {String} eventDef The event object in the format namespace:component
 * @param {String=} eventName The event name if the event is a "COMPONENT" type event
 * @param {sourceCmp=} eventName The component source if the event is a "COMPONENT" type event
 * @return {Event} new Event
 * @memberOf AuraEventService
 * @public
 * @export
 */
AuraEventService.prototype.newEvent = function(eventDef, eventName, sourceCmp) {};

/**
 * Get a new Event instance, but do not do an access check on the event definition. This method is private to
 * aura and should only be used within trusted portions of the framework.
 *
 * @param {String} eventDef The event object in the format namespace:component
 * @param {String=} eventName The event name if the event is a "COMPONENT" type event
 * @param {sourceCmp=} eventName The component source if the event is a "COMPONENT" type event
 * @return {Event} new Event
 */
AuraEventService.prototype.getNewEvent = function(eventDefinition, eventName, sourceCmp) {};

/**
 * Accumulates the ordered list of components from cmp to the root through
 * which an event should bubble. This includes all paths through both owners
 * and containerComponents.
 * @param {Component} cmp
 * @param {Array} queue ordered list of nodes in the path
 * @param {Object} visited map of components that have been visited already
 * @param {Boolean} isOwner indicates if this path is the value provider path
 *      of the cmp from the original call in the stack
 * @return {Array} queue of ordered list of nodes in the path; each element
 *      includes a "cmp" property and a "isOwner" property
 * @private
 */
AuraEventService.prototype.collectBubblePath = function(cmp, queue, visited, isOwner) {};


/**
 * Returns an iterator over the sequence of components for a given event
 * through the "capture" phase then the "bubble" phase.
 * @return {Object} An object with the form { cmp: Component, isOwner: Boolean, phase: String }
 * @private
 * @function
 */
AuraEventService.prototype.getEventPhaseCmpIterator = (function() {})();

/**
 * Function that supplies an array of handlers for a given cmp, evt, and phase.
 * @callback handlerSupplier
 * @param {Component} cmp
 * @param {Event} evt
 * @param {String} phase
 * @return {Function[]} array of handler functions whose only parameter is the event
 */

/**
 * A result from a handler iterator
 * @typedef {Object} HandlerIteratorResult
 * @property {Component} cmp - The component with the registered handler
 * @property {String} phase - The phase at which the handler is bound
 * @property {Function} handler - The event handler function
 */

/**
 * Returns an iterator over event handlers for a given event that
 * supports "capture" and "bubble" phases. The HandlerIteratorResult
 * object are returned in the correct order for those two respective phases.
 * If stopPropagation() is called on the event before this iterator
 * completes, it will terminate, returning a result with done=true and
 * whose value is the last HandlerIteratorResult that it previously returned
 * but with an undefined handler property. This facilitates consumers
 * determining which component is responsible for stopping the event's
 * propagation.
 *
 * @param {Event} evt The event
 * @param {handlerSupplier} The handler supplier function
 * @return {HandlerIteratorResult} A HandlerIteratorResult
 * @private
 * @function
 */
AuraEventService.prototype.getPhasedEventHandlerIterator = (function() {})();


/**
 * Returns an iterator over component event handlers for a given event.
 * @param {Event} evt The event
 * @return {HandlerIteratorResult} A HandlerIteratorResult
 * @private
 * @function
 */
AuraEventService.prototype.getComponentEventHandlerIterator = (function() {})();

/**
 * A faux handler that may be inserted into an iteration
 * to stop propagation of an event.
 * @private
 */
AuraEventService.prototype.eventStopPropagationHandler = function(evt) {};

/**
 * A handlerSupplier implementation for phased COMPONENT event handlers
 * that go through capture and bubble phases.
 * @private
 */
AuraEventService.prototype.getComponentEventHandlers = function(evt, cmp, phase, isOwner) {};


/**
 * A handlerSupplier implementation for COMPONENT event handlers to
 * use for COMPONENT events that do NOT go through capture or bubble
 * phases.
 * @private
 */
AuraEventService.prototype.getNonBubblingComponentEventHandlers = function(cmp, evt, phase/*, isOwner*/) {};

/**
 * Returns an iterator over event handler tuples for a COMPONENT event that
 * does not support "capture" or "bubble" phases. The iterator returns HandlerIteratorResult with
 * handlers to invoke in the correct order for those two respective phases.
 * @param {Event} evt The event
 * @param {handlerSupplier} The handler supplier function
 * @return {HandlerIteratorResult} A HandlerIteratorResult
 * @private
 * @function
 */
AuraEventService.prototype.getNonBubblingComponentEventHandlerIterator = (function() {})();

/**
 * Returns an iterator over all event handlers for a VALUE event or method call event.
 * The iterator emits the next handler function to invoke for the event.
 * @param {Event} evt The event
 * @return {HandlerIteratorResult} A HandlerIteratorResult
 * @private
 * @function
 */
AuraEventService.prototype.getValueHandlerIterator = (function() {})();

/**
 * A handlerSupplier implementation for APPLICATION event handlers
 * @private
 */
AuraEventService.prototype.getPhasedApplicationEventHandlers = function(evt, cmp, phase, isOwner) {};


/**
 * Convenience function that indicates if an event has any handlers registered
 * in the bubble or capture phases at all.
 * @return {Boolean} True if the event has any handlers in bubble or capture
 *      phases
 * @private
 */
AuraEventService.prototype.applicationEventHasPhasedHandlers = (function() {})();


/**
 * Returns an iterator over all event handlers for a given APPLICATION event.
 * The iterator emits the next handler function to invoke for the event.
 * @param {Event} evt The event
 * @return {HandlerIteratorResult} A HandlerIteratorResult
 * @private
 * @function
 */
AuraEventService.prototype.getAppEventHandlerIterator = (function() {})();


/**
 * Returns the new event.
 * @param {String} name The event object in the format namespace:component
 * @param {Function} [callback] The function that gets executed when the get has succeeded. Since this could download the event def if it is not present you need the callback to get the definition.
 * @memberOf AuraEventService
 */
AuraEventService.prototype.get = function(name, callback) {};

/* Evaluates an action expression and runs it
* @private
* */
AuraEventService.prototype.expressionHandler=function(expression,event){};

/**
 * Dynamically adds an application event handler for the specified event.
 *
 * @param {String} event The name of the event to handle, e.g. 'aura:applicationEvent'.
 * @param {Function} handler A reference to the function or action to invoke when the event is fired, e.g., 'cmp.getReference("c.myAction")'.
 * @param {String} phase The event bubbling phase for which to add the handler. Optional. If omitted, uses "default".
 * @param {Boolean} includeFacets If true, attempt to catch events generated by components transcluded by facets, e.g. v.body.
 *
  * @private
 */
AuraEventService.prototype.addEventHandler=function(component,eventDef,handler,phase,includeFacets){};

/**
 * Adds an event handler.
 * @param {Object} config The data for the event handler
 * @memberOf AuraEventService
 * @public
 * @export
 * @deprecated use <code>addEventHandler</code> instead.
 */
AuraEventService.prototype.addHandler = function(config) {};

/**
 * Dynamically removes an application event handler for the specified event.
 *
 * @param {String} event The name of the event to remove, e.g. 'aura:applicationEvent'.
 * @param {Function} handler A reference to the function or action to action to remove, e.g., 'cmp.getReference("c.handleApplicationEvent");'.
 * @param {String} phase The event bubbling phase for which to remove the handler. Optional. If omitted, uses "default".
 *
 * @export
 * @private
 */
AuraEventService.prototype.removeEventHandler=function(component,eventDef,handler,phase) {};

/**
 * Removes an event handler.
 * @param {Object} config The data for the event
 * @memberOf AuraEventService
 * @public
 * @export
 */
AuraEventService.prototype.removeHandler = function(config) {};

/**
 * Removes all handlers in all the phases for a component. Used by component.destroy()
 * @param {String} globalId the id of the component.
 */
AuraEventService.prototype.removeHandlersByComponentId = function(globalId) {};

/**
 * Adds an event handler that will trigger only once, and inmediately will be removed.
 * @param {Object} config The data for the event handler
 * @memberOf AuraEventService
 * @public
 * @export
 * @deprecated
 */
AuraEventService.prototype.addHandlerOnce = function(config) {};

/**
 * Returns the event definition.
 * Internal method to the framework. To get an event def from the API, use $A.get("e.prefix:name", function(def){});
 *
 * @param {String} descriptor name of EventDef
 * @return {EventDef} The event definition.
 * @memberOf AuraEventService
 * @private
 */
AuraEventService.prototype.getEventDef = function(config) {};

/**
 * Get the event definition.
 * Does access checks.
 * You cannot fire this though, use newEvent() for that.
 *
 * @param  {String}  descriptor Event descriptor in the pattern prefix:name or markup://prefix:name.
 */
AuraEventService.prototype.getDef = function(descriptor) {};


/**
 * Checks to see if the definition for the event currently reside on the client.
 * Could still exist on the server, we won't know that till we use a getDefinitiion call to try to retrieve it.
 * This method is private, to use it, use $A.hasDefinition("e.prefix:name");
 * @private
 * @param  {String}  descriptor Event descriptor in the pattern prefix:name or markup://prefix:name.
 * @return {Boolean}            True if the definition is present on the client.
 */
AuraEventService.prototype.hasDefinition = function(descriptor) {};

/**
 * Gets descriptor from the config object (for normalization)
 * @param {Object} Controller descriptor config
 * @returns {String} Descriptor
 * @private
 */
AuraEventService.prototype.createDescriptorConfig = function(descriptor) {};


/**
 * Get the event definition. If it is not available, contact the server to download it.
 *
 * This method is private, to utilize it's functionality you can use $A.getDefinition("e.prefix:name");
 *
 * @private
 *
 * @param  {String}  descriptor Event descriptor in the pattern prefix:name or markup://prefix:name.
 * @param  {Function} callback  The function callback that gets executed with the definition. May go to the server first.
 * @return undefined            Always use the callback to access the definition.
 */
AuraEventService.prototype.getDefinition = function(descriptor, callback) {};


/**
 * Gets descriptor from the config object (for normalization)
 * @param {Object} Controller descriptor config
 * @returns {String} Descriptor
 * @private
 */
AuraEventService.prototype.getDescriptorFromConfig = function(descriptorConfig) {};


/**
 * Creates and saves EventDef into registry
 * @param {Object} config config for EventDef
 * @returns {EventDef} instance from registry
 */
AuraEventService.prototype.createFromSavedConfigs = function(config) {};



/**
 * Creates and returns EventDef from config
 * @param {Object} config The parameters for the event
 * @return {EventDef} The event definition.
 * @memberOf AuraEventService
 * @private
 */
AuraEventService.prototype.createEventDef = function(config) {};

/**
 * Saves EventDef config so it can be used later when EventDef is actually used.
 * Allows Aura to only create EventDef when needed
 *
 * @param {Object} config event definition config
 */
AuraEventService.prototype.saveEventConfig = function(config) {};

/**
 * Returns true if the event has handlers.
 * @param {String} name The event name
 * @memberOf AuraEventService
 * @public
 * @export
 */
AuraEventService.prototype.hasHandlers = function(name) {};

/**
 * Returns the qualified name of all events known to the registry.
 * @export
 */
AuraEventService.prototype.getRegisteredEvents = function() {};

//#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
/**
 * Whether there are pending events
 * Available in DEV mode only.
 * @export
 */
AuraEventService.prototype.hasPendingEvents = function() {};
//#end

Aura.Services.AuraEventService = AuraEventService;
