/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @class Aura
 * @classdesc The Aura framework. Default global instance name is $A.
 * @constructor
 * @platform
 * @namespace
 * @alias $A
 *
 * @borrows AuraClientService#enqueueAction as enqueueAction
 * @borrows AuraComponentService#createComponent as createComponent
 * @borrows AuraComponentService#createComponents as createComponents
 * @borrows AuraComponentService#getComponent as getComponent
 */
function AuraInstance () {
    this.globalValueProviders = {};
    this.deprecationUsages    = {};
    this.displayErrors        = true;
    this.initializers         = {};

    this.logger               = new Aura.Utils.Logger();

    /**
     * Collection of basic utility methods to operate on the DOM and Aura Components. <br/>
     * See the documentation for <a href="#reference?topic=api:Util">Util</a> for the members.
     *
     * @type $A.ns.Util
     * @platform
     */
    this.util                 = new Aura.Utils.Util();
    this["util"]              = this.util; //Move this? (check prod mangling)

    this.auraError            = Aura.Errors.AuraError;
    this.auraFriendlyError    = Aura.Errors.AuraFriendlyError;

    /**
     * Error severity for categorizing errors
     *
     * ALERT [default error severity level if error thrower doesn’t explicitly specify a severity level] -
     * the current page has issues and we need to alert the user that an error has occurred.  The error(s) could potentially be corrected by a page reload
     *
     * FATAL - the user’s session is now completely broken and cannot continue being used.
     * The user should logout and contact Salesforce support or their administrator.
     *
     * QUIET - An error has occurred but it won’t affect the user/page.
     * This is likely something unexpected that a lower level component can just quietly log for later diagnostics by Salesforce (e.g. a perf issue or something else).
     *
     * @public
     */
    this.severity = {
        ALERT: "ALERT",
        FATAL: "FATAL",
        QUIET: "QUIET"
    };

    this.lastKnownError = null;

    /**
     * Instance of the AuraLocalizationService which provides utility methods for localizing data or getting formatters for numbers, currencies, dates, etc.<br/>
     * See the documentation for <a href="#reference?topic=api:AuraLocalizationService">AuraLocalizationService</a> for the members.
     *
     * @type AuraLocalizationService
     * @platform
     */
    this.localizationService  = new Aura.Services.AuraLocalizationService();
    this.clientService        = new Aura.Services.AuraClientService(this.util);
    this.componentService     = new Aura.Services.AuraComponentService();
    this.renderingService     = new Aura.Services.AuraRenderingService();
    this.expressionService    = new Aura.Services.AuraExpressionService();
    this.historyService       = new Aura.Services.AuraHistoryService();
    this.eventService         = new Aura.Services.AuraEventService();
    this.storageService       = new Aura.Services.AuraStorageService();
    this.styleService         = new Aura.Services.AuraStyleService();
    this.metricsService       = new Aura.Services.MetricsService();
    this.lockerService        = new Aura.Services.LockerService();

    //#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
    this.devToolService = new AuraDevToolService();
    //#end

    this.injectedServices={
        "localization":this.localizationService,
        "metrics":this.metricsService
    };

    /** @field */
    this.services = {
        /**
         * Rendering Service
         *
         * @public
         * @type AuraRenderingService
         * @memberOf Aura.Services
         */
        rendering : this.renderingService,
        /**
         * Event Service
         *
         * @public
         * @type AuraEventService
         * @memberOf Aura.Services
         */
        event : this.eventService,
        /**
         * Component Service
         *
         * @public
         * @type AuraComponentService
         * @memberOf Aura.Services
         */
        component : this.componentService,
        /**
         * Client Service
         *
         * @public
         * @type AuraClientService
         * @memberOf AuraInstance.prototype
         */
        client : this.clientService,

        /**
         * History Service
         *
         * @public
         * @type AuraHistoryService
         * @memberOf AuraInstance.prototype
         */
        history : this.historyService,

        /**
         * Storage Service
         *
         * @public
         * @type AuraStorageService
         * @memberOf AuraInstance.prototype
         */
        storage : this.storageService,

        /**
         * Alias of Component Service
         *
         * @public
         * @type AuraComponentService
         * @memberOf AuraInstance.prototype
         * @see Aura#services.component
         */
        cmp : this.componentService,

        /**
         * Alias of Event Service
         *
         * @public
         * @type AuraEventService
         * @memberOf AuraInstance.prototype
         * @see Aura#services.event
         */
        e : this.eventService,

        /**
         * Style Service
         *
         * @public
         * @type AuraStyleService
         * @memberOf AuraInstance.prototype
         */
        style: this.styleService,

        /**
         * Metrics Service
         *
         * @public
         * @type AuraMetricsService
         * @memberOf AuraInstance.prototype
         */
        metrics: this.metricsService,

        /**
         * Locker Service
         *
         * @public
         * @type AuraLockerService
         * @memberOf AuraInstance.prototype
         */
        locker: this.lockerService,

        get : function(key) {}
    };


    // Doced at the source by the @borrows statements on the Aura class
    this.Component                 = Component;

    this.enqueueAction             = this.clientService.enqueueAction.bind(this.clientService);
    this.deferPendingActions       = this.clientService.deferPendingActions.bind(this.clientService);
    this.runAfterInit              = this.clientService.runAfterInitDefs.bind(this.clientService);

    this.render                    = this.renderingService.render.bind(this.renderingService);
    this.rerender                  = this.renderingService.rerender.bind(this.renderingService);
    this.unrender                  = this.renderingService.unrender.bind(this.renderingService);
    this.afterRender               = this.renderingService.afterRender.bind(this.renderingService);

    this.getCmp                    = this.componentService.get.bind(this.componentService);
    this.getComponent              = this.componentService.getComponent.bind(this.componentService);
    this.createComponent           = this.componentService["createComponent"].bind(this.componentService);
    this.createComponents          = this.componentService["createComponents"].bind(this.componentService);
    this.createComponentFromConfig = this.componentService.createComponentFromConfig.bind(this.componentService);

    this.getEvt                    = this.eventService.newEvent.bind(this.eventService);

    // DEPRECATED
    this.newCmp                    = this.componentService["newComponentDeprecated"].bind(this.componentService);
    this.newCmpDeprecated          = this.componentService["newComponentDeprecated"].bind(this.componentService);
    this.newCmpAsync               = this.componentService["newComponentAsync"].bind(this.componentService);
    // END DEPRECATED

    /**
     * Pushes current portion of attribute's creationPath onto stack
     * @param {String} creationPath
     *
     * @public
     */
    this.pushCreationPath = function(creationPath) {};


    /**
     * pops current portion of attribute's creationPath from stack
     * @param {String} creationPath
     *
     * @public
     */
    this.popCreationPath = function(creationPath) {};

    /**
     * sets pathIndex for the current attribute on creationPath's stack
     * @param {String} creationPath
     *
     * @public
     */
    this.setCreationPathIndex = function(idx) {};

    //  Google Closure Compiler Symbol Exports
    this["runAfterInit"] = this.runAfterInit;
    this["clientService"] = this.clientService;
    this["componentService"] = this.componentService;
    this["renderingService"] = this.renderingService;
    this["expressionService"] = this.expressionService;
    this["historyService"] = this.historyService;
    this["localizationService"] = this.localizationService;
    this["eventService"] = this.eventService;
    this["layoutService"] = this.layoutService;
    this["metricsService"] = this.metricsService;
    this["lockerService"] = this.lockerService;
    this["storageService"] = this.storageService;
    this["styleService"] = this.styleService;
    this["services"] = this.services;
    this["enqueueAction"] = this.enqueueAction;
    this["deferPendingActions"] = this.deferPendingActions;
    this["render"] = this.render;
    this["rerender"] = this.rerender;
    this["unrender"] = this.unrender;
    this["afterRender"] = this.afterRender;
    this["logger"] = this.logger;
    this["getCmp"] = this.getCmp;
    this["getComponent"] = this.getComponent;
    this["pushCreationPath"] = this.pushCreationPath;
    this["popCreationPath"] = this.popCreationPath;
    this["setCreationPathIndex"] = this.setCreationPathIndex;

    //#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
    this["devToolService"] = this.devToolService;
    this["getQueryStatement"] = this.devToolService.newStatement;
    //#end

    this["createComponent"] = this.createComponent;
    this["createComponents"] = this.createComponents;
    this["createComponentFromConfig"] = this.createComponentFromConfig;
    this["getEvt"] = this.getEvt;
    this["Component"] = this.Component;

    this["auraFriendlyError"] = this.auraFriendlyError;
    this["severity"] = this.severity;
    this["severity"]["ALERT"] = this.severity.ALERT;
    this["severity"]["FATAL"] = this.severity.FATAL;
    this["severity"]["QUIET"] = this.severity.QUIET;

    // work around closure compiler
    this["severity"] = Object.freeze(this["severity"]);

    this["hasDefinition"] = this.hasDefinition;
    this["getDefinition"] = this.getDefinition;
    this["getDefinitions"] = this.getDefinitions;

    // DEPRECATED
    this["newCmp"] = this.newCmp;
    this["newCmpDeprecated"] = this.newCmpDeprecated;
    this["newCmpAsync"] = this.newCmpAsync;
    //#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
    this["qhelp"] = function() {};
    //#end

    // END DEPRECATED

    var services = this.services;

    // TODO: convert to //#exportSymbols when available

    services["rendering"] = services.rendering;
    services["event"] = services.event;
    services["component"] = services.component;
    services["client"] = services.client;
    services["history"] = services.history;
    services["storage"] = services.storage;
    services["metrics"] = services.metrics;
    services["cmp"] = services.cmp;
    services["e"] = services.e;
    services["c"] = {
        get: function(name) {}
    };

}

/**
 * Initializes Aura with context info about the app that should be loaded.
 * @param {Object} config
 *
 * {
 *      <code>config.descriptor</code> : The descriptor of the application or component that should be loaded as the root. For example, <code>"markup://foo:bar"</code><br />
 *      <code>config.attributes</code> : The attributes that should be passed into the root component when it is constructed. For example, <code>{at1 : 1, at2 : "asdf"}</code><br />
 *      <code>config.defType</code> : The defType of the descriptor.  For example, <code>"DEFINITION"</code> or <code>"APPLICATION"</code><br />
 *      <code>config.lastmod</code> : The timestamp, in millis of the latest changes to the preloaded metadata associated with this application.
 * }
 * @public
 */

/*
 * Execute all the functions that have been injected before the framework is initialized
 * An example of injection is the code that a user can write in the template as part of the preInitBlock
 */
AuraInstance.prototype.beforeInitHooks = function () {};

/*
 * Execute all the functions that have been injected after the framework is initialized
 * (the app component tree has been created and rendered)
 */
AuraInstance.prototype.afterInitHooks = function () {};

AuraInstance.prototype.initAsync = function(config) {};

/**
 * This function initializes external libraries that were appended on aura.js
 * An example of library is moment.js
 */
AuraInstance.prototype.executeExternalLibraries = function () {};

/**
 * Initializes Aura with context info but without retrieving component from server. Used for synchronous initialization.
 *
 * Whoever named this function should be shot, but I won't rename for now. Eventually we want to use
 * startApplication, and make it either auto-require app.js or have the caller load app.js and then invoke
 * startApplication with the data.
 *
 * @param {Object} config The configuration attributes
 * @param {Boolean} useExisting
 * @param {Boolean} doNotInitializeServices Set to true if the History service should not be initialized, or false if
 *   it should. Defaults to true for Aura Integration Service.
 * @public
 */
AuraInstance.prototype.initConfig = function(config, useExisting, doNotInitializeServices) {};

/**
 * Initializes Aura in debug environment.
 *
 * @param {Object} config The descriptor ("markup://foo:bar"), attributes, defType ("APPLICATION" or "COMPONENT"), and
 *        timestamp of last modified change
 * @param {String} token
 * @param {Object} container Sets the container for the component.
 * @param {Boolean=} doNotInitializeServices True if the History service should not be initialized, or false if
 *        it should. Defaults to true for Aura Integration Service.
 * @private
 */
AuraInstance.prototype.initPriv = function(config, token, container, doNotInitializeServices) {};

/**
 * Add default handler to aura:systemError event
 * @private
 */
AuraInstance.prototype.addTearDownHandler = function () {};

/**
 * Add default handler to aura:systemError event
 * @private
 */
AuraInstance.prototype.addDefaultEventHandlers = function (app) {};

/*
*  Default Error handler
*  @private
* */
AuraInstance.prototype.defaultErrorHandler=function(event) {};

/*
 * Default XSS controlled location redirect event handler
 * @private
 * */
AuraInstance.prototype.defaultRedirectHandler=function(evt) {};


/**
 * Signals that initialization has completed.
 * @private
 */
AuraInstance.prototype.finishInit = function(doNotInitializeServices) {};

/**
 * Deprecated. Use the standard JavaScript throw new Error(msg) instead for a serious error
 * that has no recovery path.
 *
 * If this occurs during a test, the test will be stopped unless you add calls to '$A.test.expectAuraError' for
 * each error that occurs. <code>auraErrorsExpectedDuringInit</code> allows server side errors to not stop the
 * test as well.
 *
 *
 * @public
 * @param {String} msg The error message to be displayed to the user.
 * @param {Error} [e] The error object to be displayed to the user.
 * @platform
 * @deprecated throw new Error(msg) instead
 */
AuraInstance.prototype.error = function(msg, e){};

/**
 * Optionally sets and returns whether to display error dialog
 *
 * @private
 * @param {Boolean} [toggle] toggles display of error dialog
 * @returns {Boolean} whether to display error dialog
 */
AuraInstance.prototype.showErrors = function(toggle){};

/**
 * @private
 */
AuraInstance.prototype.handleError = function(message, e) {};

/**
 * @private
 */
AuraInstance.prototype.isCustomerError = function(e) {};

/**
 * @private
 */
AuraInstance.prototype.isCustomerComponent = function(cmp) {};

/**
 * @private
 */
AuraInstance.prototype.isCustomerComponentStack = function(cmpStack) {};

/**
 * Report error to the server after handling it.
 * Note that the method should only be used if try-catch mechanism
 * of error handling is not desired or not functional (ex: in nested promises)
 * @public
 * @param {String} message - The message to display.
 * @param {Error} error - An error object to be included in handling and reporting.
 * @platform
 */
AuraInstance.prototype.reportError = function(message, error) {};

/**
 * <code>$A.warning()</code> should be used in the case where poor programming practices have been used.
 *
 * These warnings will not, in general, be displayed to the user, but they will appear in the console (if
 * available), and in the aura debug window.
 *
 * @public
 * @param {String} w - The message to display.
 * @param {Error} e - an error, if any.
 * @platform
 */
AuraInstance.prototype.warning = function(w, e) {};

/**
 * Displays an error message to the user. Currently used for displaying errors that do not cause the application to
 * stop completely.
 *
 * @public
 * @param {String} msg The message to display.
 * @param {Error} error object
 * @param {Boolean} showReload whether to show reload button in dialog
 */
AuraInstance.prototype.message = function(msg, error, showReload) {};

/**
 * Returns a callback which is safe to invoke from outside Aura, e.g. as an event handler or in a setTimeout.
 * The $A.getCallback() call ensures that the framework rerenders the modified component
 * and processes any enqueued actions.
 * @public
 * @function
 * @param {Function} callback - The method to call after reestablishing Aura context.
 * @platform
 */
AuraInstance.prototype.getCallback = function(callback) {};

/*
* Allows for non-blocking action dispatch while executing cpu-bound code.
* Synchronous code run in the callback passed to this method will allow server actions to
* be dispatched immediately, without waiting for a boxcar at the end of the thread.
*
* @function
* @param {Function} callback The method to invoke while allowing actions to flow out as they occur.
* @export
* @experimental
* */
AuraInstance.prototype.executeHotspot=function(callback){};

/**
 * Returns the application configuration token referenced by name.
 * A tokens file is configured with the tokens attribute in the aura:application tag.
 *
 * @function
 * @param {String} token - The name of the application configuration token to retrieve, for example, <code>$A.getToken("section.configuration")</code>.
 *
 * @platform
 */
AuraInstance.prototype.getToken = function(token){};

/**
 * Returns a service registered by the application.
 *
 * @function
 * @param {String} name - The name of the service registered during application startup, for example, <code>$A.getService("metrics")</code>.
 *
 * @export
 */
AuraInstance.prototype.getService = function(name) {};

/**
 * Returns the value referenced using property syntax. Gets the value from the specified global value provider.
 * @public
 * @function
 * @param {String} key - The data key to look up on element, for example, <code>$A.get("$Label.section.key")</code>.
 * @param {Function} [callback] - The method to call with the result if a server trip is expected.
 * @platform
 */
AuraInstance.prototype.get = function(key, callback) {};

/**
 * Sets the value referenced using property syntax on the specified global value provider.
 * @public
 * @function
 * @param {String} key - The data key we want to change on the global value provider, for example, <code>$A.set("$Custom.something","new value")</code>.
 * @param {Object} value - The value to set the key location to. If the global value provider does not implement .set(), this method will throw an exception.</code>.
 * @platform
 */
AuraInstance.prototype.set = function(key, value) {};


/**
 * Returns a live reference to the global value indicated using property syntax.
 *
 * @param {String} key - The data key for which to return a reference.
 * @return {PropertyReferenceValue}
 * @public
 * @platform
 * @export
 */
AuraInstance.prototype.getReference = function(key) {};

Aura.OverrideMap$Instance = undefined;

/**
 * Override a function in aura.
 *
 * This should only be available to plugins, and only works on functions designed for this purpose.
 *
 * @param {string} name the name of the override point
 * @param {Function} fn the function to insert in the chain.
 * @param {Object} scope a scope for invoking the function.
 * @param {number} priority a priority for the function (0 = highest -> first, 100 = lowest ->last, default 50)
 * @public
 */
AuraInstance.prototype.installOverride = function(name, fn, scope, priority) {};

/**
 * Remove an override in aura.
 *
 * @public
 */
AuraInstance.prototype.uninstallOverride = function(name, fn) {};

/**
 * Gets the root component or application. For example, <code>$A.getRoot().get("v.attrName");</code> returns the attribute from the root component.
 * @public
 * @function
 * @platform
 * @export
 */
AuraInstance.prototype.getRoot = function() {};

/**
 * @private
 */
AuraInstance.prototype.setRoot = function(root) {};

/**
 * Gets the current <code>AuraContext</code>. The context consists of the mode, descriptor, and namespaces to be loaded.
 *
 * @public
 * @function
 * @return {AuraContext} current context
 */
AuraInstance.prototype.getContext = function() {};

/**
 * Deprecated. Use <code>getCallback()</code> instead.
 * Runs a function within the standard Aura lifecycle.
 *
 * This ensures that <code>enqueueAction</code> methods and rerendering are handled properly
 * from JavaScript outside of controllers, renderers, providers.
 * @param {Function} func The function to run.
 * @param {String} name an optional name for the stack.
 * @public
 * @platform
 * @deprecated Use <code>getCallback()</code> instead.
 */
AuraInstance.prototype.run = function(func, name) {};

/**
 * Checks the condition and if the condition is false, displays an error message.
 *
 * Displays an error message if condition is false, runs <code>trace()</code> and stops JS execution. The
 * app will cease to function until reloaded if this is called, and errors are not caught.
 * Internal assertion, should never happen
 * <p>For example, <code>$A.assert(cmp.get("v.name") , "The name attribute is required.");</code> checks for the name attribute.
 *
 * This is protected as it is an internal assertion, should never happen.
 *
 * @param {Boolean} condition True prevents the error message from being displayed, or false otherwise.
 * @param {String} assertMessage A message to be displayed when condition is false
 */
AuraInstance.prototype.assert = function(condition, assertMessage) {};

/**
 * Checks for a specified user condition, only to be used for fatal errors!. Displays an error message if condition is
 * false, and stops JS execution. The app will cease to function until reloaded if this is called.
 *
 * @param {Boolean} condition The conditional expression to be evaluated.
 * @param {String} msg The message to be displayed when the condition is false.
 * @public
 */
AuraInstance.prototype.userAssert = function(condition, msg) {};

/**
 *  Logs to the browser's JavaScript console if it is available.
 *  This method doesn't log in PROD or PRODDEBUG modes.
 *  If both value and error are passed in, value shows up in the console as a group with value logged within the group.
 *  If only value is passed in, value is logged without grouping.
 *  <p>For example, <code>$A.log(action.getError());</code> logs the error from an action.</p>
 *
 * @public
 * @param {Object} value - The object to log.
 * @param {Object} error - The error messages to be logged in the stack trace.
 * @platform
 */
AuraInstance.prototype.log = function(value, error) {};

/**
 * Logs a stack trace. Trace calls using <code>console.trace()</code> if defined on the console implementation.
 * @public
 */
AuraInstance.prototype.trace = function() {};

/*
 * Called from methods that have entered or are entering the deprecation pipeline. Provides first warnings, then errors to developers.
 * When possible, includes a workaround for the behavior or method being deprecated.
 *
 * It reports the deprecation usages to server if reportSignature is provided.
 *
 * @param {String} message - The message to provide the developer indicating the method or behavior which has been or is being deprecated.
 * @param {String} workaround - Any known or suggested workaround to accomplish the behavior being deprecated.
 * @param {String} reportSignature - The deprecated function name if deprecating an API, or function signature if deprecating a function signature.
 *
 * @private
 * */
AuraInstance.prototype.deprecated = function(message, workaround, reportSignature) {};

/**
 * Sets mode to production (default), development, or testing.
 *
 * @param {String} mode Possible values are production "PROD", development "DEV", or testing "PTEST".
 * @private
 */
AuraInstance.prototype.setMode = function(mode) {};

/**
 * Get GVP directly.
 * @param {String} type The type of global value provider to retrieve.
 * @return {GlobalValueProvider} The global value provider, such as $Label, $Browser, $Locale, etc.
 *
 * @private
 */
AuraInstance.prototype.getValueProvider = function(type) {};

/**
 * Add a new Global Value Provider.
 * @param {String} type The type of global value provider to add. Types must start with a '$', and may not use reserved
 *                      types, such as '$Label', '$Browser', or '$Locale'
 * @param {ValueProvider} The global value provider. This can either implement a '.get(expression)' and
 *                               optional '.set(expression, value)' method, or simply be an instance of an Object.
 *
 * @public
 */
AuraInstance.prototype.addValueProvider=function(type,valueProvider){};

/**
 * Gets the event or component definition. If it is not currently on the client, we will access the server to attempt to retrieve it.
 *
 * @public
 *
 * @param  {String}   descriptor Descriptor in the pattern prefix:name or markup://prefix:name. Use e.prefix:name, or markup://e.prefix:name for an event definition.
 * @param  {Function} callback   Function whos first parameter is the requested definition if it exists. Otherwise the first parameter is null.
 * @return undefined
 */
AuraInstance.prototype.getDefinition = function(descriptor, callback) {};

/**
 * Similar to $A.getDefinition() will retrieve an array of definitions at one time. Optimal if you expect them not to be available on the client.
 * @public
 * @param  {Array}   descriptors An Array of Descriptors in the format expected by $A.getDefinition() can be a mix of events and component descriptors.
 * @param  {Function} callback   Function whos first parameter is an array of the definitions requested. Some definitions may be null, which are those definitions you don't have access to or did not exist.
 * @return undefined             Always use the callback to access the definitions you requested.
 */
AuraInstance.prototype.getDefinitions = function(descriptors, callback) {};

/**
 * Detect if a definition is present on the client. May still exist on the server.
 * @public
 * @param  {String}   descriptor Descriptor in the pattern prefix:name. Use e.prefix:name for an event definition.
 * @return {Boolean}             True if the definition is present on the client.
 */
AuraInstance.prototype.hasDefinition = function(descriptor) {};

// #include aura.util.PerfShim
AuraInstance.prototype.Perf = window['Perf'] || PerfShim;

// #include aura.Aura_export
