/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description The Aura Localization Service, accessible using <code>$A.localizationService</code>. Provides utility methods
 * for localizing data or getting formatters for numbers, currencies, dates, etc.
 * @constructor
 * @export
 * @platform
 */
function AuraLocalizationService() {}

/**
 * Formats a number with the default number format.
 * @param {Number} number - The number to be formatted.
 * @return {Number} The formatted number
 * @memberOf AuraLocalizationService
 * @example
 * var num = 10000;
 * // Returns 10,000
 * var formatted = $A.localizationService.formatNumber(num);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatNumber = function(number) {};

/**
 * Returns a formatted percentage number based on the default percentage format.
 * @param {Number} number - The number to be formatted.
 * @return {Number} The formatted percentage
 * @memberOf AuraLocalizationService
 * @example
 * var num = 0.54;
 * // Returns 54%
 * var formatted = $A.localizationService.formatPercent(num);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatPercent = function(number) {};

/**
 * Returns a currency number based on the default currency format.
 * @param {Number} number - The currency number to be formatted.
 * @return {Number} The formatted currency
 * @memberOf AuraLocalizationService
 * @example
 * var curr = 123.45;
 * // Returns $123.45
 * $A.localizationService.formatCurrency(curr);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatCurrency = function(number) {};


/**
 * Returns a NumberFormat object.
 * @param {String} format - The number format. <code>format=".00"</code> displays the number followed by two decimal places.
 * @param {String} symbols
 * @return {Number} The number format
 * @memberOf AuraLocalizationService
 * @example
 * var f = $A.get("$Locale.numberFormat");
 * var num = 10000
 * var nf = $A.localizationService.getNumberFormat(f);
 * var formatted = nf.format(num);
 * // Returns 10,000
 * var formatted = $A.localizationService.formatNumber(num);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getNumberFormat = function(format, symbols) {};

/**
 * Returns the default NumberFormat object.
 * @return {Number} The number format returned by <code>$Locale.numberFormat</code>.
 * @memberOf AuraLocalizationService
 * @example
 * // Returns 20,000.123
 * $A.localizationService.getDefaultNumberFormat().format(20000.123);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getDefaultNumberFormat = function() {};


/**
 * Returns the default percentage format.
 * @return {Number} The percentage format returned by <code>$Locale.percentFormat</code>.
 * @memberOf AuraLocalizationService
 * @example
 * // Returns 20%
 * $A.localizationService.getDefaultPercentFormat().format(0.20);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getDefaultPercentFormat = function() {};

/**
 * Returns the default currency format.
 * @return {Number} The currency format returned by <code>$Locale.currencyFormat</code>.
 * @memberOf AuraLocalizationService
 * @example
 * // Returns $20,000.00
 * $A.localizationService.getDefaultCurrencyFormat().format(20000);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getDefaultCurrencyFormat = function() {};

/**
 * Displays a length of time.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @param {Boolean} withSuffix - If true, returns value with the suffix
 * @return {String} a String of a length of time
 * @memberOf AuraLocalizationService
 * @public
 * @example
 * var dur = $A.localizationService.duration(1, 'day');
 * // Returns "a day"
 * var length = $A.localizationService.displayDuration(dur);
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDuration = function(duration, withSuffix) {};

/**
 * Displays a length of time in days.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The length of time in days.
 * @memberOf AuraLocalizationService
 * @public
 * @example
 * var dur = $A.localizationService.duration(24, 'hour');
 * // Returns 1
 * var length = $A.localizationService.displayDurationInDays(dur);
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDurationInDays = function(duration) {};

/**
 * Displays a length of time in hours.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The length of time in hours.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(2, 'day');
 * // Returns 48
 * var length = $A.localizationService.displayDurationInHours(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDurationInHours = function(duration) {};

/**
 * Displays a length of time in milliseconds.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The length of time in milliseconds.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(1, 'hour');
 * // Returns 3600000
 * var length = $A.localizationService.displayDurationInMilliseconds(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDurationInMilliseconds = function(duration) {};

/**
 * Displays a length of time in minutes.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The length of time in minutes.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(1, 'hour');
 * // Returns 60
 * var length = $A.localizationService.displayDurationInMinutes(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDurationInMinutes = function(duration) {};

/**
 * Displays a length of time in months.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The length of time in months.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(60, 'day');
 * // Returns 1.971293
 * var length = $A.localizationService.displayDurationInMonths(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDurationInMonths = function(duration) {};

/**
 * Displays a length of time in seconds.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The length of time in seconds.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(60, 'minutes');
 * // Returns 3600
 * var length = $A.localizationService.displayDurationInSeconds(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDurationInSeconds = function(duration) {};

/**
 * Displays a length of time in years.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The length of time in years.
 * @memberOf AuraLocalizationService
 * example
 * var dur = $A.localizationService.duration(6, 'month');
 * // Returns 0.5
 * var length = $A.localizationService.displayDurationInYears(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.displayDurationInYears = function(duration) {};

/**
 * Creates an object representing a length of time.
 * @param {Number} num - The length of time in a given unit
 * @param {String} unit - A datetime unit. The default is milliseconds. Options: years, months, weeks, days, hours, minutes, seconds, milliseconds
 * @return {Object} A duration object
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(1, 'day');
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.duration = function(num, unit) {};

/**
 * Formats a date.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 *   If you provide a String value, use ISO 8601 format to avoid parsing warnings.
 * @param {String} formatString - A string containing tokens to format a date and time. For example, "YYYY-MM-DD" formats 15th January, 2017 as "2017-01-15".
 * 	The default format string comes from the $Locale value provider.
 *  For details on available tokens, see https://developer.salesforce.com/docs/atlas.en-us.lightning.meta/lightning/js_cb_format_dates.htm.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @return {String} A formatted and localized date string
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * // Returns date in the format "Oct 9, 2015"
 * $A.localizationService.formatDate(date);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatDate = function(date, formatString, locale) {};

/**
 * Formats a date in UTC.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 *   If you provide a String value, use ISO 8601 format to avoid parsing warnings.
 * @param {String} formatString - A string containing tokens to format a date and time. For example, "YYYY-MM-DD" formats 15th January, 2017 as "2017-01-15".
 * 	The default format string comes from the $Locale value provider.
 *  For details on available tokens, see https://developer.salesforce.com/docs/atlas.en-us.lightning.meta/lightning/js_cb_format_dates.htm.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @return {String} A formatted and localized date string
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * // Returns date in UTC in the format "Oct 9, 2015"
 * $A.localizationService.formatDateUTC(date);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatDateUTC = function(date, formatString, locale) {};

/**
 * Formats a datetime.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 *   If you provide a String value, use ISO 8601 format to avoid parsing warnings.
 * @param {String} formatString - A string containing tokens to format a date and time.
 * 	The default format string comes from the $Locale value provider.
 *  For details on available tokens, see https://developer.salesforce.com/docs/atlas.en-us.lightning.meta/lightning/js_cb_format_dates.htm.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @return {String} A formatted and localized datetime string
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * // Returns datetime in the format "Oct 9, 2015 9:00:00 AM"
 * $A.localizationService.formatDateTime(date);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatDateTime = function(date, formatString, locale) {};

/**
 * Formats a datetime in UTC.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 *   If you provide a String value, use ISO 8601 format to avoid parsing warnings.
 * @param {String} formatString - A string containing tokens to format a date and time.
 * 	The default format string comes from the $Locale value provider.
 *  For details on available tokens, see https://developer.salesforce.com/docs/atlas.en-us.lightning.meta/lightning/js_cb_format_dates.htm.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @return {String} A formatted and localized datetime string
 * @example
 * var date = new Date();
 * // Returns datetime in UTC in the format "Oct 9, 2015 4:00:00 PM"
 * $A.localizationService.formatDateTimeUTC(date);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatDateTimeUTC = function(date, formatString, locale) {};

/**
 * Formats a time.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 *   If you provide a String value, use ISO 8601 format to avoid parsing warnings.
 * @param {String} formatString - A string containing tokens to format a time.
 * 	The default format string comes from the $Locale value provider.
 *  For details on available tokens, see https://developer.salesforce.com/docs/atlas.en-us.lightning.meta/lightning/js_cb_format_dates.htm.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @return {String} A formatted and localized time string
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * // Returns a date in the format "9:00:00 AM"
 * var now = $A.localizationService.formatTime(date);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatTime = function(date, formatString, locale) {};

/**
 * Formats a time in UTC.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 *   If you provide a String value, use ISO 8601 format to avoid parsing warnings.
 * @param {String} formatString - A string containing tokens to format a time.
 * 	The default format string comes from the $Locale value provider.
 *  For details on available tokens, see https://developer.salesforce.com/docs/atlas.en-us.lightning.meta/lightning/js_cb_format_dates.htm.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @return {String} a formatted and localized time string
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * // Returns time in UTC in the format "4:00:00 PM"
 * $A.localizationService.formatTimeUTC(date);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.formatTimeUTC = function(date, formatString, locale) {};

/**
 * Gets the number of days in a duration.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The number of days in duration.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(48, 'hour');
 * // Returns 2, the number of days for the given duration
 * $A.localizationService.getDaysInDuration(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getDaysInDuration = function(duration) {};

/**
 * Gets the number of hours in a duration.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The number of hours in duration.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(60, 'minute');
 * // Returns 1, the number of hours in the given duration
 * $A.localizationService.getHoursInDuration(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getHoursInDuration = function(duration) {};

/**
 * Get the date time related labels (month name, weekday name, am/pm etc.).
 * @return {Object} the localized label set.
 * @memberOf AuraLocalizationService
 * @deprecated
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getLocalizedDateTimeLabels = function() {};

/**
 * Get today's date based on a time zone.
 * @param {String} timezone - A time zone id based on the java.util.TimeZone class, for example, America/Los_Angeles
 * @param {Function} callback - A function to be called after the "today" value is obtained
 * @return {String} the ISO8601 date string (yyyy-MM-dd).
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getToday = function(timezone, callback) {};


/**
 * Get the date's date string based on a time zone.
 * @param {String} timeZone - A time zone id based on the java.util.TimeZone class, for example, America/Los_Angeles
 * @param {Date} date - A Date object
 * @param {Function} callback - A function to be called after the date string is obtained
 * @memberOf AuraLocalizationService
 * @example
 * var timezone = $A.get("$Locale.timezone");
 * var date = new Date();
 * // Returns the date string in the format "2015-10-9"
 * $A.localizationService.getDateStringBasedOnTimezone(timezone, date, function(today){
 *    console.log(today);
 * });
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getDateStringBasedOnTimezone = function(timeZone, date, callback) {};

/**
 * Gets the number of milliseconds in a duration.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The number of milliseconds in duration.
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getMillisecondsInDuration = function(duration) {};

/**
 * Gets the number of minutes in a duration.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The number of minutes in duration.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(60, 'second');
 * // Returns 1, the number of minutes in the given duration
 * $A.localizationService.getMinutesInDuration(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getMinutesInDuration = function(duration) {};

/**
 * Gets the number of months in a duration.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The number of months in duration.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(70, 'day');
 * // Returns 2, the number of months in the given duration
 * $A.localizationService.getMonthsInDuration(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getMonthsInDuration = function(duration) {};

/**
 * Gets the number of seconds in a duration.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The number of seconds in duration.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(3000, 'millisecond');
 * // Returns 3
 * $A.localizationService.getSecondsInDuration(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getSecondsInDuration = function(duration) {};

/**
 * Gets the number of years in a duration.
 * @param {Duration} duration - The duration object returned by $A.localizationService.duration
 * @return {Number} The number of years in duration.
 * @memberOf AuraLocalizationService
 * @example
 * var dur = $A.localizationService.duration(24, 'month');
 * // Returns 2
 * $A.localizationService.getYearsInDuration(dur);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.getYearsInDuration = function(duration) {};

/**
 * A utility function to check if a datetime pattern string uses a 24-hour or period (12 hour with am/pm) time view.
 * @param {String} pattern - datetime pattern string
 * @return {Boolean} Returns true if it uses period time view.
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.isPeriodTimeView = function(pattern) {};

/**
 * Checks if date1 is after date2.
 * @param {String|Number|Date} date1 - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String|Number|Date} date2 - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String} unit - A datetime unit. The default is millisecond. Options: year, month, week, day, hour, minute, second, millisecond.
 * @return {Boolean} Returns true if date1 is after date2, or false otherwise.
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * var day = $A.localizationService.endOf(date, 'day');
 * // Returns false, since date is before day
 * $A.localizationService.isAfter(date, day);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.isAfter = function(date1, date2, unit) {};

/**
 * Checks if date1 is before date2.
 * @param {String|Number|Date} date1 - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String|Number|Date} date2 - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String} unit - A datetime unit. The default is millisecond. Options: year, month, week, day, hour, minute, second, millisecond.
 * @return {Boolean} Returns true if date1 is before date2, or false otherwise.
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * var day = $A.localizationService.endOf(date, 'day');
 * // Returns true, since date is before day
 * $A.localizationService.isBefore(date, day);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.isBefore = function(date1, date2, unit) {};

/**
 * Checks if date1 is the same as date2.
 * @param {String|Number|Date} date1 - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String|Number|Date} date2 - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String} unit - A datetime unit. The default is millisecond. Options: year, month, week, day, hour, minute, second, millisecond.
 * @return {Boolean} Returns true if date1 is the same as date2, or false otherwise.
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * var day = $A.localizationService.endOf(date, 'day');
 * // Returns false
 * $A.localizationService.isSame(date, day, 'hour');
 * // Returns true
 * $A.localizationService.isSame(date, day, 'day');
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.isSame = function(date1, date2, unit) {};

/**
 * Checks if a date is between two other dates (fromDate and toDate), where the match is inclusive.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String|Number|Date} fromDate - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String|Number|Date} toDate - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String} unit - A datetime unit. The default is millisecond. Options: year, month, week, day, hour, minute, second, millisecond.
 * @return {Boolean} Returns true if date is between fromDate and toDate, or false otherwise.
 * @memberOf AuraLocalizationService
 * @example
 * $A.localizationService.isBetween("2017-03-07","March 7, 2017", "12/1/2017")
 * // Returns true
 * $A.localizationService.isBetween("2017-03-07 12:00", "March 7, 2017 15:00", "12/1/2017")
 * // Returns false
 * $A.localizationService.isBetween("2017-03-07 12:00", "March 7, 2017 15:00", "12/1/2017", "day")
 * // Returns true
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.isBetween = function(date, fromDate, toDate, unit) {};

/**
 * Parses a string to a JavaScript Date.
 * @param {String} dateTimeString - The datetime string to be parsed.
 * @param {String} parseFormat - A Java format string which is used to parse datetime. The default is from LocaleValueProvider.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @param {Boolean} [strictParsing] - (optional) Set to true to turn off forgiving parsing and use strict validation.
 * @return {Date} A JavaScript Date object, or null if dateTimeString is invalid
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.parseDateTime = function(dateTimeString, parseFormat, locale, strictParsing) {};

/**
 * Parses a date time string in an ISO-8601 format.
 * @param {String} dateTimeString - The datetime string in an ISO-8601 format.
 * @return {Date} A JavaScript Date object, or null if dateTimeString is invalid.
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.parseDateTimeISO8601 = function(dateTimeString) {};

/**
 * Parses a string to a JavaScript Date in UTC.
 * @param {String} dateTimeString - The datetime string to be parsed
 * @param {String} parseFormat - A Java format string which is used to parse datetime. The default is from LocaleValueProvider.
 * @param {String} [locale] - [Deprecated] (optional) Locale value from Locale Value Provider. It falls back to the value in $Locale.langLocale if using unavailable locale. The default value is from $Locale.langLocale.
 * @param {Boolean} [strictParsing] - (optional) Set to true to turn off forgiving parsing and use strict validation.
 * @return {Date} A JavaScript Date object, or null if dateTimeString is invalid
 * @memberOf AuraLocalizationService
 * @example
 * var date = "2015-10-9";
 * // Returns "Thu Oct 08 2015 17:00:00 GMT-0700 (PDT)"
 * $A.localizationService.parseDateTimeUTC(date);
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.parseDateTimeUTC = function(dateTimeString, parseFormat, locale, strictParsing) {};

/**
 * Get a date which is the start of a unit of time for the given date.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String} unit - A datetime unit. Options: year, month, week, day, hour, minute or second.
 * @return {Date} A JavaScript Date object. It returns a parsed Date if unit is not provided.
 * @memberOf AuraLocalizationService
 * @example
 * var date = "2015-10-9";
 * // Returns "Thu Oct 01 2015 00:00:00 GMT-0700 (PDT)"
 * $A.localizationService.startOf(date, 'month');
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.startOf = function(date, unit) {};

/**
 * Get a date which is the end of a unit of time for the given date.
 * @param {String|Number|Date} date - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object.
 * @param {String} unit - A datetime unit. Options: year, month, week, day, hour, minute or second.
 * @return {Date} A JavaScript Date object. It returns a parsed Date if unit is not provided.
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * // Returns the time at the end of the day
 * // in the format "Fri Oct 09 2015 23:59:59 GMT-0700 (PDT)"
 * var day = $A.localizationService.endOf(date, 'day')
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.endOf = function(date, unit) {};

/**
 * Get a date time string in simplified extended ISO format.
 * @param {Date} date - a Date object
 * @return {String} An ISO8601 string to represent passed in Date object.
 * @memberOf AuraLocalizationService
 * @example
 * var date = new Date();
 * // Returns "2015-10-09T20:47:17.590Z"
 * $A.localizationService.toISOString(date);
 * @deprecated
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.toISOString = function(date) {};

/**
 * Translate the localized digit string to a string with Arabic digits if there is any.
 * @param {String} input - a string with localized digits.
 * @return {String} a string with Arabic digits.
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.translateFromLocalizedDigits = function(input) {};

/**
 * Translate the input date from other calendar system (for example, Buddhist calendar) to Gregorian calendar
 * based on the locale.
 * @param {Date} date - a Date Object.
 * @return {Date} an updated Date object.
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.translateFromOtherCalendar = function(date) {};

/**
 * Translate the input string to a string with localized digits (different from Arabic) if there is any.
 * @param {String} input - a string with Arabic digits.
 * @return {String} a string with localized digits.
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.translateToLocalizedDigits = function(input) {};

/**
 * Translate the input date to a date in other calendar system, for example, Buddhist calendar based on the locale.
 * @param {Date} date - a Date Object.
 * @return {Date} an updated Date object.
 * @memberOf AuraLocalizationService
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.translateToOtherCalendar = function(date) {};

/**
 * Converts a datetime from UTC to a specified timezone.
 * @param {Date} date - A JavaScript Date object
 * @param {String} timezone - A time zone id based on the java.util.TimeZone class, for example, America/Los_Angeles
 * @param {Function} callback - A function to be called after the conversion is done
 * @memberOf AuraLocalizationService
 * @example
 * // Provides locale information
 * var format = $A.get("$Locale.timeFormat");
 * format = format.replace(":ss", "");
 * var langLocale = $A.get("$Locale.langLocale");
 * var timezone = $A.get("$Locale.timezone");
 * var date = new Date();
 * $A.localizationService.UTCToWallTime(date, timezone, function(walltime) {
 *    // Returns the local time without the seconds, for example, 9:00 PM
 *    displayValue = $A.localizationService.formatDateTimeUTC(walltime, format, langLocale);
 * })
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.UTCToWallTime = function(date, timezone, callback) {};

/**
 * Converts a datetime from a specified timezone to UTC.
 * @param {Date} date - A JavaScript Date object
 * @param {String} timezone - A time zone id based on the java.util.TimeZone class, for example, America/Los_Angeles
 * @param {Function} callback - A function to be called after the conversion is done
 * @memberOf AuraLocalizationService
 * @example
 * $A.localizationService.WallTimeToUTC(date, timezone, function(utc) {
 *     displayDate = $A.localizationService.formatDateTime(utc, format, langLocale);
 * })
 * @public
 * @export
 * @platform
 */
AuraLocalizationService.prototype.WallTimeToUTC = function(date, timezone, callback) {};

/**---------- Private functions ----------*/

/**
 * Initialize localization service.
 * @private
 */
AuraLocalizationService.prototype.init = function() {};

AuraLocalizationService.prototype.normalizeTimeZone = function(timeZone) {};

AuraLocalizationService.prototype.createDateTimeData = function(date, timeZone) {};

/**
 * Convert datatime data created by createDateTimeData() to different time zone.
 *
 * @returns {Object} datetime data for the given zone
 *
 * @private
 */
AuraLocalizationService.prototype.setDataToZone = function(data, timeZone) {};

/**
 * @returns {Array} a tuple which contains timestamp and offset
 */
AuraLocalizationService.prototype.getZoneInfo = function(config, timeZone) {};

/**
 * Get the time zone offset during the given timestamp.
 *
 * @returns {Number} offset in minute
 */
AuraLocalizationService.prototype.zoneOffset = function(timestamp, timeZone) {};

/**
 * Formats a Date object to the ISO8601 date string.
 *
 * This method assumes the browser supports Intl API with time zone data.
 * @private
 */
AuraLocalizationService.prototype.formatDateWithTimeZone = function(date, timeZone) {};

/**
 * Parse a datetime string in en-US format, "month/day/year, hour:minute", to a timestamp in millisecond in UTC
 *
 * @param {String} dateTimeString - a datetime string in en-US format, "month/day/year, hour:minute"
 * @returns {Number} timestamp in millisecond
 *
 * @private
 */
AuraLocalizationService.prototype.parseEnUSDateTimeString = function(dateTimeString) {};

AuraLocalizationService.prototype.formatDateTimeToString = function(dateTimeFormat, date) {};

AuraLocalizationService.prototype.createDateTimeFormatByTimeZone = function(timeZone) {};

/**
 * Normalize the specified Java locale string to moment-js compatible locale which
 * has available data on the client. If the given locale doesn't have any available
 * corresponding locale, it falls back to 'en'.
 *
 * @private
 */
AuraLocalizationService.prototype.normalizeToMomentLocale = function(locale) {};

/**
 * Convert locale string into moment-js locale format.
 *
 * @private
 */
AuraLocalizationService.prototype.normalizeLocale = function(locale) {};

/**
 * Get available moment locale from cache based on specified Java locale.
 *
 * This function assumes all available locales are added to cache during init().
 *
 * @param {String} locale - a Java locale
 * @return {String} corresponding momnet locale string, or 'en' if Java locale doesn't exists in cache.
 *
 * @private
 */
AuraLocalizationService.prototype.getAvailableMomentLocale = function(locale) {};

/**
 * Check if a Java locale is available in localization service.
 *
 * This function assumes all available locales are added to cache during init().
 *
 * @param {String} locale - a Java locale
 * @return {Boolean} true if locale is available, false otherwise
 *
 * @private
 */
AuraLocalizationService.prototype.isAvailableLocale = function(locale) {};

/**
 * Display date, datetime or time based on the format string.
 *
 * @private
 */
AuraLocalizationService.prototype.displayDateTime = function(mDate, format, locale) {};

/**
 * Normalize a Java format string to make it compatible with moment.js
 *
 * @private
 */
AuraLocalizationService.prototype.getNormalizedFormat = function(format) {};

/**
 * Modifying the format so that moment's strict parsing doesn't break on minor deviations
 *
 * @private
 */
AuraLocalizationService.prototype.getStrictModeFormat = function(format) {};

/**
 * Modifying the date time string so that moment's strict parsing doesn't break on minor deviations
 *
 * @private
 */
AuraLocalizationService.prototype.getStrictModeDateTimeString = function(dateTimeString) {};

/**
 * Mutates the original date object by adding or subtract time.
 *
 * @param {Date} date - The Date object to mutated
 * @param {Number} num - The number of unit to add or to subtract
 * @param {String} unit - A normalized datetime unit, options: year, month, week, day, hour, minute or second
 * @param {Boolean} isSubtract - Set true if it is a subtract
 *
 * @private
 */
AuraLocalizationService.prototype.addSubtract = function(date, num, unit, isSubtract) {};

/**
 * Converts datetime input into a Date object. If datetime is a Date object, it returns the original input.
 * @param {String|Number|Date} datetime - A datetime string in ISO8601 format, or a timestamp in milliseconds, or a Date object
 * @returns {Date} A Date object which represents the provided datetime, an invalid Date if the given datetime is not a supported type
 *
 * @private
 */
AuraLocalizationService.prototype.normalizeDateTimeInput = function(datetime) {};

/**
 * Parses a datetime string into a Date object.
 * @param {String} dateTimeString - A datetime string in ISO8601 format. If the string is not in ISO8601 format, it uses native Date() to parse, which may have different results across browsers and versions.
 *
 * @private
 */
AuraLocalizationService.prototype.parseDateTimeString = function(dateTimeString) {};

AuraLocalizationService.prototype.isISO8601DateTimeString = function(dateTimeString) {};

AuraLocalizationService.prototype.normalizeDateTimeUnit = function(unit) {};

/**
 * Adds a datetime unit's aliases (lowercase, lowercase plural, shorthand) to unit alias map.
 */
AuraLocalizationService.prototype.addDateTimeUnitAlias = function(unit, short) {};

AuraLocalizationService.prototype.setupDateTimeUnitAlias = function() {};

AuraLocalizationService.prototype.isValidDate = function(date) {};

Aura.Services.AuraLocalizationService = AuraLocalizationService;
