/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @class The Aura Rendering Service, accessible using <code>$A.renderingService</code>.
 *        Renders components. The default behaviors can be customized in a
 *        client-side renderer.
 * @constructor
 * @export
 */
function AuraRenderingService() {}

/**
 * Renders a component by calling its renderer.
 *
 * @param {Component} components - A component or a component array to be rendered.
 * @param {HTMLElement} parent - Optional. The element into which rendered elements are inserted.
 *
 * @memberOf AuraRenderingService
 * @public
 * @export
 */
AuraRenderingService.prototype.render = function(components, parent) {};

/**
 * The default rerenderer for components affected by an event. Call
 * superRerender() from your customized function to chain the
 * rerendering to the components in the body attribute.
 *
 * @param {Component} components - The component or component array to be rerendered.
 * @returns {Array} An array of rerendered elements.
 *
 * @memberOf AuraRenderingService
 * @public
 * @export
 */
AuraRenderingService.prototype.rerender = function(components) {};

/**
 * The default behavior after a component is rendered.
 *
 * @param {Component|Array} components - The component or component array that has finished rendering
 * @memberOf AuraRenderingService
 * @public
 * @export
 */
AuraRenderingService.prototype.afterRender = function(components) {};

/**
 * The default unrenderer that deletes all the DOM nodes rendered by a
 * component's render() function. Call superUnrender() from your
 * customized function to modify the default behavior.
 *
 * @param {Component|Array} components - The component or component array to be unrendered.
 * @memberOf AuraRenderingService
 * @public
 * @export
 */
AuraRenderingService.prototype.unrender = function(components) {};

/**
 * Store current facet to component.
 * This method does not verify component and facet. Facet MUST be an array.
 *
 * @param {Component} component - The component which owns the facet.
 * @param {Array} facet - A component array to store.
 * @private
 */
AuraRenderingService.prototype.storeFacetInfo = function(component, facet) {};

/**
 *
 * @param {Component} component - The component which owns the facet.
 * @param {Component|Array} facet - The component or the component array to update.
 * @returns {Object} The updated facet configs
 * @private
 */
AuraRenderingService.prototype.getUpdatedFacetInfo = function(component, facet) {};

/**
 * Normailize facet to an array of components.
 * This method instantiates ComponentDefRef on facet to Component.
 * It also updates the container of all components on facet to the facet owner.
 *
 * @param {Component} component - The component whose facet is being rendered.
 * @param {Component|Array} facet - A component or a component array to be rendered.
 * @returns {Array} An array of components.
 *
 * @private
 */
AuraRenderingService.prototype.getFacetInfo = function(component, facet) {};

/**
 * @public
 * @param {Component} component - The component whose facet is being rendered.
 * @param {Component|Array} facet - The facet to be rendered.
 * @param {HTMLElement} parent - Optional. The element into which rendered elements are inserted.
 * @export
 */
AuraRenderingService.prototype.renderFacet = function(component, facet, parent) {};

/**
 * @public
 * @param {Component} component - The component whose facet is being rerendered.
 * @param {Component} facet - The facet to be rerendered.
 * @param {HTMLElement} referenceNode - The element into which rerendered elements are inserted.
 * @export
 */
AuraRenderingService.prototype.rerenderFacet = function(component, facet, referenceNode) {};


/**
 * Get the insert postion where the elements from component will be inserted on targetNode.
 *
 * Counting the dom elements, so that if an unknown element gets in the collection we won't blow up.
 * But this means we need to count everything for the off chance this condition happens.
 *
 * @param {Component} component - the component whose elements will be inserted
 * @param {HTMLElement} targetNode - the target node which the elements from component will be inserted into
 *
 * @private
 */
AuraRenderingService.prototype.getInsertPosition = function(component, targetNode) {};

/**
 * Update component's marker to componentOnFacet's sibling element if component shares same marker with componentOnFacet.
 * It also updates component's container chain if the containers share the marker as well.
 *
 * This function is used in rerenderFacet() when unrendering component on facet.
 *
 * @private
 */
AuraRenderingService.prototype.moveSharedMarkerToSibling = function(component, componentOnFacet) {};

/**
 * Used in AuraRenderingService.unrender to update the new marker element in the container chain.
 *
 * Do not use this method in render or re-render. It doesn't apply aura-class to the new element.
 *
 * @private
 */
AuraRenderingService.prototype.replaceElementOnContainers = function(component, oldElement, newElement) {};

/**
 * Find the element which is the last element in the DOM from component elements collection.
 * @private
 */
AuraRenderingService.prototype.getLastSharedElementInCollection = function(cmpElements, domElements) {};

/**
 * @param {Component} component - the component to be found from containers
 * @returns {Array} an arry of indexes that indicate where the component's elements are on the containers, from the bottom (direct) container to the top one
 *
 * @private
 */
AuraRenderingService.prototype.findElementsPositionFromContainers = function(component) {};

/**
 * Update elements through container chain.
 * When a dirty component gets rerendered, all containers of the component need to update their elements set accordingly.
 *
 * @param {Component} component - the component whose elements are used for updating containers
 * @param {Array} insertPositions - the old elements positions in the container's elements set, from the bottom (direct) container to the top one
 * @param {Array} oldElements - the elements before rerendering
 *
 * @private
 */
AuraRenderingService.prototype.updateElementsOnContainers = function(component, insertPositions, oldElements) {};

/**
 * @public
 * @param {Component} cmp the component for which we are unrendering the facet.
 * @param {Component} facet the facet to unrender.
 * @export
 */
AuraRenderingService.prototype.unrenderFacet = function(cmp,facet) {};

/**
 * Get a marker for a component.
 *
 * @public
 * @param {Component} cmp the component for which we want a marker.
 * @return the marker.
 * @export
 */
AuraRenderingService.prototype.getMarker = function(cmp){};

AuraRenderingService.prototype.setMarker = function(cmp, newMarker) {};

/**
 * @protected
 * @param expression the expression to mark as dirty.
 * @param cmp the owning component.
 */
AuraRenderingService.prototype.addDirtyValue = function(expression, cmp) {};

/**
 * Check if a component has any dirty value in dirty component set.
 * It's used in an exported method, Component.isDirty().
 * TODO: this method should be private.
 *
 * @protected
 * @param {Component} cmp - The component to check.
 */
AuraRenderingService.prototype.hasDirtyValue = function(cmp){};

/**
 * Check if an expression on a component is dirty.
 * It's used in an exported method, Component.isDirty().
 * TODO: this method should be private.
 *
 * @protected
 */
AuraRenderingService.prototype.isDirtyValue = function(expression, cmp) {};

/**
 * Rerender all dirty components.
 *
 * Called from ClientService when we reach the top of stack.
 *
 * @protected
 * @export
 */
AuraRenderingService.prototype.rerenderDirty = function(stackName) {};

/**
 * This method is not used by framework anymore, but it's exposed via Component.markClean().
 * TODO: Clean up the references and remove the both methods.
 *
 * @deprecated
 * @protected
 */
AuraRenderingService.prototype.removeDirtyValue = function(value, cmp) {};

//#if {"modes" : ["PTEST","STATS"]}
AuraRenderingService.prototype.statsIndex = {
    "afterRender": [],
    "render": [],
    "rerender": [],
    "rerenderDirty": [],
    "unrender": []
};
//#end

/**
 * Remove component's concrete global id from dirty components set
 * @private
 */
AuraRenderingService.prototype.cleanComponent = function(id) {};

/**
 * This method ensures to get an array.
 * If input is a falsy, it returns an empty array.
 * If input is an array, it returns the original array.
 * Otherwise, it returns an array which contains the input value.
 * @private
 */
AuraRenderingService.prototype.getArray = function(things) {};

/**
 * If a renderer returned a string, create html elements from that string.
 *
 * Returns an elements array, either the original one passed in or a new one
 * if "elements" passed in was a string, not an array.
 *
 * @private
 */
AuraRenderingService.prototype.evalStrings = function(elements) {};

AuraRenderingService.prototype.finishRender = function(cmp, elements) {};

/**
 * Insert elements to the DOM, relative to a reference node,
 * by default as its last child.
 *
 * @private
 */
AuraRenderingService.prototype.insertElements = function(elements, refNode, asSibling, asFirst) {};

/**
 * Calculates the flavor css class name for a component instance and element.
 * @private
 */
AuraRenderingService.prototype.getFlavorClass = function(cmp) {};

AuraRenderingService.prototype.addAuraClass = function(cmp, element){};

/**
 * Associate all of the elements with the component, and return a list of
 * pure elements - with no association objects wrapped around them.
 *
 * @private
 */
AuraRenderingService.prototype.associateElements = function(cmp, elements) {};

/**
 * Create a new Comment marker optinally before the specified target and for the specified reason.
 * Often the reason is something relating to what was unrendered or rendered such as a globalId.
 *
 * @param {HTMLElement} target - the target element where the created marker is placed before
 * @param {string} reason - the text content of the marker, to help understand when or why this marker get created
 *
 * @private
 */
AuraRenderingService.prototype.createMarker = function(target, reason) {};

/**
 * Basically was this node created by the createMarker() function above.
 * Since we use comment markers as placement in the dom for non-rendered components and expressions
 * We often branch logic on wether the element is a comment marker or not.
 * @private
 */
AuraRenderingService.prototype.isCommentMarker = function(node){};

/**
 * If you use component.getElements() it will normalize the array, but also slice it to give you a copy.
 * When in trusted code that is aware of this situation, we can avoid the cost of slicing the array.
 *
 * @private
 */
AuraRenderingService.prototype.getElements = function(component) {};

/**
 * Includes all the DOM elements the component output as part of its rendering cycle.
 * This method also returns the comment markers output as part of the component rendering cycle.
 * If you do not want the comment nodes returned to you (your known set of dom nodes), use cmp.getElements() or renderingService.getElements(component)
 */
AuraRenderingService.prototype.getAllElements = function(component) {};

/**
 * Similar to getAllElements, but this method will copy the allElements collection and return it. This allows you to modify the collection for processing
 * during the renderingService without worring about mutating the component elements collection.
 */
AuraRenderingService.prototype.getAllElementsCopy = function(component) {};

/**
 * Get a uniqueID to identify different HTML elements by.
 * This method tries to use the data-rendered-by attribute first if possible.
 * If not (such as comment nodes) then we'll just append our own data attribute.
 * We need this so we can maintain a map of references to a component without a reference to the component.
 *
 * @private
 */
AuraRenderingService.prototype.getUid = function(element) {};

/**
 * Assign a new unique id to the specified element.
 * The unique ID is just an incrementing number on the service.
 *
 * @private
 */
AuraRenderingService.prototype.newUid = function(element) {};

/**
 * Get the unique id for an element. If it does not have one, generate one and return that.
 * Uses a combination of getUid() and newUid().
 *
 * @private
 */
AuraRenderingService.prototype.resolveUid = function(element) {};

/**
 * The marker can be any dom node. This method tracks that that dom node is being
 * referenced by the component with the specified globalid
 *
 * @private
 */
AuraRenderingService.prototype.addMarkerReference = function(marker, globalId) {};

/**
 * The specified dom node (marker) is no longer being used by the component with the specified global id.
 *
 * @private
 */
AuraRenderingService.prototype.removeMarkerReference = function(marker, globalId) {};

/**
 * Remove the reference marker from the markerToReferencesMap object.
 *
 * @private
 */
AuraRenderingService.prototype.removeMarkerFromReferenceMap = function(resolvedMarker, refs) {};

/**
 * Get a collection of IDs who are using the specified element as a marker.
 * If the element is being removed, we'll want to move those references to another element or a comment marker.
 *
 * @private
 */
AuraRenderingService.prototype.getMarkerReferences = function(marker) {};

/**
 * Carefully remove the marker from the container.
 * If we're trying to remove a shared comment marker, we do nothing since others are using it.
 * If the parent isn't unrendering or is being destroyed, we won't do anything either since the container will have its dom node removed and that will remove all its children from the dom.
 *
 * @private
 */
AuraRenderingService.prototype.removeElement = function(marker, container) {};

/**
 * All the components who are using the specified dom node as a marker need to now be moved to a comment marker.
 * This method doesn't check if you're moving from one comment node to another. That would be a waste of time, so
 * be aware you should verify that first.
 *
 * @private
 */
AuraRenderingService.prototype.moveReferencesToMarker = function(marker, newMarker) {};

/**
 * Are multiple components using this as a marker?
 * Shared markers need to have their other references moved before being able to remove the marker node from the dom.
 *
 * @private
 */
AuraRenderingService.prototype.isSharedMarker = function(marker) {};

/**
 * The ReferenceCollection is a data structure for tracking references from components to dom nodes.
 * Since it is a many (components() to one element mapping, this is necessary to track whos using what.
 * The collection will optimize to not create an array at first. Only after you've added more than one reference
 * will the array be created.
 *
 * Lastly, this collecton acts just like a Set(). You cannot add the same value twice, which helps with our reference counting logic.
 *
 * @private
 */
AuraRenderingService.prototype.ReferenceCollection = function() {};

AuraRenderingService.prototype.ReferenceCollection.prototype.isCollection = false;

AuraRenderingService.prototype.ReferenceCollection.prototype.add = function(value){};

AuraRenderingService.prototype.ReferenceCollection.prototype.delete = function(value, callback){};

AuraRenderingService.prototype.ReferenceCollection.prototype.has = function(value){};

AuraRenderingService.prototype.ReferenceCollection.prototype.size = function(){};

AuraRenderingService.prototype.ReferenceCollection.prototype.get = function(index) {};

Aura.Services.AuraRenderingService = AuraRenderingService;
