/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Construct a new Component.
 *
 * @public
 * @class
 * @constructor
 *
 * @param {Object}
 *            config - component configuration
 * @param {Boolean}
 *            [localCreation] - local creation
 * @platform
 * @export
 */
function Component(config, localCreation) {}

/**
 * Sets this component's containerComponentId, which is the global id of
 * the component that owns the facet attribute for which this
 * component is a value.
 * @private
 */
Component.prototype.setContainerComponentId = function(containerComponentId) {};

Component.currentComponentId = 0;

Component.prototype.nextGlobalId = function(localCreation) {};

/**
 * The globally unique id of this component
 */
Component.prototype.setupGlobalId = function(globalId, localCreation) {};


/**
 * Returns the component's code-compatible camelCase name, e.g. 'uiButton'.
 *
 * @export
 * @platform
 */
Component.prototype.getName = function() {};

/**
 * Returns the component's canonical type, e.g. 'ui:button'.
 *
 * @export
 * @platform
 */
Component.prototype.getType = function() {};

/**
 * Gets the ComponentDef Shorthand: <code>get("def")</code>
 *
 * @public
 * @export
 */
Component.prototype.getDef = function() {};

/**
 * Indexes the given <code>globalId</code> based on the given
 * <code>localId</code>. Allows <code>cmp.find(localId)</code> to look up
 * the given <code>globalId</code>, look up the component, and return it.
 *
 * @param {String}
 *            localId The id set using the aura:id attribute.
 * @param {String}
 *            globalId The globally unique id which is generated on pageload.
 * @protected
 * @export
 */
Component.prototype.index = function(localId, globalId) {};

/**
 * Removes data from the index. If both <code>globalId</code> and
 * <code>localId</code> are provided, only the given pair is removed from the
 * index. If only <code>localId</code> is provided, every mapping for that
 * <code>localId</code> is removed from the index.
 *
 * This might be called after component destroy in some corner cases, be careful
 * to check for destroy before accessing.
 *
 * @param {String}
 *            localId The id set using the aura:id attribute.
 * @param {String}
 *            globalId The globally unique id which is generated on pageload.
 * @protected
 * @export
 */
Component.prototype.deIndex = function(localId, globalId) {};

/**
 * Locates a component using the localId. Shorthand: <code>get("asdf")</code>,
 * where "asdf" is the <code>aura:id</code> of the component to look for.
 * Returns different types depending on the result.
 * 	If the local ID is unique, find() returns the component.
 *	If there are multiple components with the same local ID, find() returns an array of the components.
 *  If there is no matching local ID, find() returns undefined.
 * Returns instances of a component using the format
 * <code>cmp.find({ instancesOf : "auradocs:sampleComponent" })</code>.
 *
 * @param {String|Object}
 *            name - If name is an object, return instances of it. Otherwise,
 *            finds a component using its index.
 * @public
 * @platform
 * @export
 */
Component.prototype.find = function(name) {};

/**
 * Find instances of a Component type, in this component's hierarchy, and in
 * its body, recursively.
 *
 * @param {Object}
 *            type The object type.
 * @param {Array}
 *            ret The array of instances to add the located Components to.
 * @param {Object}
 *            cmp The component to search for.
 * @private
 */
Component.prototype.findInstancesOf = function(type, ret, cmp) {};

/**
 * @private
 */
Component.prototype.getSuperest = function() {};

/**
 *
 * @private
 */
Component.prototype.findInstanceOf = function(type) {};

/**
 * Checks whether the component is an instance of the given component name (or
 * interface name).
 *
 * @param {String}
 *            name - The name of the component (or interface), with a format of
 *            <code>namespace:componentName</code>.
 * @returns {Boolean} true if the component is an instance, or false otherwise.
 * @platform
 * @export
 */
Component.prototype.isInstanceOf = function(name) {};

/**
 * @param {Object}
 *            type Applies the type to its definition.
 * @private
 */
Component.prototype.implementsDirectly = function(type) {};

/**
 * Dynamically adds an event handler for a component or application event.
 *
 * @example
 * // For component event, first param matches name attribute in <aura:registerEvent> tag
 * cmp.addEventHandler("compEvent", cmp.getReference("c.handleEvent"));
 *
 * // For application event, first param is event descriptor, "c:appEvent"
 * cmp.addEventHandler("c:appEvent", cmp.getReference("c.handleAppEvent"));
 *
 * // Anonymous function handler for component event
 * cmp.addEventHandler("compEvent", function(auraEvent) {
 *     // add handler logic here
 *     console.log("Handled the component event in anonymous function");
 * });
 *
 * @param {String} event The name of the event to handle.
 *   For a component event, set this argument to match the name attribute of the <code>aura:registerEvent</code> tag.
 *   For an application event, set this argument to match the event descriptor, <code>namespace:eventName</code>.
 * @param {Function} handler The handler for the event. There are two format options for this argument.
 *   To use a controller action, use the format: <code>cmp.getReference("c.actionName")</code>.
 *   To use an anonymous function, use the format: <code>function(auraEvent) { // handling logic here }</code>
 * @param {String} phase The event bubbling phase for which to add the handler. Optional. If omitted, uses "bubble".
 * @param {Boolean} includeFacets If true, attempt to catch events generated by components transcluded by facets, e.g. v.body.

 * @export
 * @platform
 */
Component.prototype.addEventHandler=function(event,handler,phase,includeFacets){};

/**
 * Adds an event handler. Resolving the handler Action happens at Event-handling
 * time, so the Action reference may be altered at runtime, and that change is
 * reflected in the handler.
 *
 * @param {String}
 *            eventName - The event name
 * @param {Object}
 *            valueProvider - The value provider to use for resolving the
 *            actionExpression.
 * @param {Object}
 *            actionExpression - The expression to use for resolving the handler
 *            Action against the given valueProvider.
 * @param {boolean}
 *            insert - The flag to indicate if we should put the handler at the
 *            beginning instead of the end of handlers array.
 * @param {String}
 *            phase - The target event phase; defaults to "bubble"
 * @param {Boolean}
 *            includeFacets If true, this handler will also be invoked when events
 *            flow from facet values whose value provider hierarchy does not
 *            include this component.
 * @public
 * @export
 * @platform
 * @deprecated use <code>addEventHandler</code> instead.
 */
Component.prototype.addHandler = function(eventName, valueProvider, actionExpression, insert, phase, includeFacets) {};

/**
 * Dynamically removes a component event handler for the specified event.
 *
 * @param {String} event The name of the event to remove, e.g. 'c:myEvent'.
 * @param {Function} handler A reference to the function or action to action to remove, e.g., 'cmp.getReference("c.handleMyEvent");'.
 * @param {String} phase The event bubbling phase for which to remove the handler. Optional. If omitted, uses "default".
 *
 * @export
 * @platform
 */
Component.prototype.removeEventHandler=function(event,handler,phase) {};

/**
 * Adds handlers to Values owned by the Component.
 *
 * @param {Object}
 *            config - Passes in the value, event (e.g. "change"), and action
 *            (e.g. "c.myAction").
 * @public
 * @export
 * @platform
 */
Component.prototype.addValueHandler = function(config) {};

// PRIVATE
Component.prototype.addChangeHandler = function(config){};

/**
 * @export
 */
Component.prototype.removeValueHandler = function(config) {};

/**
 * Add a document level event handler that auto-cleans.
 *
 * When called, this will create and return a handler that can be enabled and
 * disabled at will, and will be cleaned up on destroy.
 *
 * @public
 * @param {String}
 *            eventName the event name to attach.
 * @param {Function}
 *            callback the callback (only called when enabled, and component is
 *            valid & rendered)
 * @param {Boolean}
 *            autoEnable (truthy) enable the handler when created.
 * @return {Object} an object with a single visible call of setEnabled(Boolean)
 * @export
 */
Component.prototype.addDocumentLevelHandler = function(eventName, callback, autoEnable) {};

/**
 * Remove a document level handler.
 *
 * You need only call this if the document level handler should be destroyed, it
 * is not generally needed.
 *
 * @public
 * @param {Object}
 *            the object returned by addDocumentHandler.
 * @export
 */
Component.prototype.removeDocumentLevelHandler = function(dlh) {};

/**
 * Destroys the component and cleans up memory.
 * After a component that is declared in markup is no longer in use, the framework automatically destroys it
 * and frees up its memory.
 * If you create a component dynamically in JavaScript and that component isn't added to a facet (v.body or another
 * attribute of type Aura.Component[]), you have to destroy it manually using destroy() to avoid memory leaks.
 *
 * <code>destroy()</code> destroys the component.
 *
 * @platform
 * @export
 */
Component.prototype.destroy = function() {};

/**
 * Execute the super component's render method.
 * @protected
 * @export
 * @platform
 */
Component.prototype.superRender = function() {};

/**
 * Execute the super component's afterRender method.
 * @protected
 * @export
 * @platform
 */
Component.prototype.superAfterRender = function() {};

/**
 * Execute the super component's rerender method.
 * @protected
 * @export
 * @platform
 */
Component.prototype.superRerender = function() {};

/**
 * Execute the super component's superUnrender method.
 * @protected
 * @export
 * @platform
 */
Component.prototype.superUnrender = function() {};

/**
 * Returns true if this component has been rendered but not unrendered (does not
 * necessarily mean component is in the dom tree).
 *
 * @protected
 * @platform
 * @export
 */
Component.prototype.isRendered = function() {};

/**
 * Returns true if this component has been rendered but not unrendered (does not
 * necessarily mean component is in the dom tree).
 *
 * @private
 */
Component.prototype.setUnrendering = function(unrendering) {};

/**
 * Returns true if this component has been rendered but not unrendered (does not
 * necessarily mean component is in the dom tree).
 *
 * @private
 */
Component.prototype.isUnrendering = function() {};

/**
 * Sets the rendered flag.
 *
 * @param {Boolean}
 *            rendered Set to true if component is rendered, or false otherwise.
 * @protected
 */
Component.prototype.setRendered = function(rendered) {};

/**
 * Returns the renderer instance for this component.
 *
 * @protected
 * @export
 */
Component.prototype.getRenderer = function() {};

/**
 * Returns the renderable instance for this component.
 * @protected
 * @export
 */
Component.prototype.getRenderable = function() {};

/**
 * Gets the globalId. This is the generated globally unique id of the component.
 * It can be used to locate the instance later, but will change across
 * page loads.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.getGlobalId = function() {};

/**
 * Gets the id set using the <code>aura:id</code> attribute. Can be passed into
 * <code>find()</code> on the parent to locate this child.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.getLocalId = function() {};

/**
 * If the server provided a rendering of this component, return it.
 *
 * @public
 * @export
 */
Component.prototype.getRendering = function() {};

/**
 * Returns the super component.
 *
 * @platform
 * @export
 */
Component.prototype.getSuper = function() {};

/**
 * Associates a rendered element with the component that rendered it for later
 * lookup. Also adds the rendering component's global Id as an attribute to the
 * rendered element. Primarily called by RenderingService.
 *
 * @param {HTMLElement} element - the element to associate with the component
 *
 * @protected
 * @export
 */
Component.prototype.associateElement = function(element) {};

/**
 * Disassociates a rendered element with the component that rendered it for later
 * lookup.
 *
 * @protected
 * @export
 */
Component.prototype.disassociateElements = function() {};

/**
 * Returns a map of the elements previously rendered by this component.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.getElements = function() {};

/**
 * If the component only rendered a single element, return it. Otherwise, you
 * should use <code>getElements()</code>.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.getElement = function() {};
/**
 * Returns a live reference to the value indicated using property syntax.
 * This is useful when you dynamically create a component.
 *
 * @example
 * $A.createComponent(
 *     "ui:button",
 *     {
 *         "aura:id": "findableAuraId",
 *         "label": "Press Me",
 *         "press": cmp.getReference("c.handlePress")
 *     },
 *     function(newButton){
 *         // Do something with the new button
 *     }
 * );
 *
 * @param {String}
 *            key - The data key for which to return a reference.
 * @return {PropertyReferenceValue}
 * @public
 * @platform
 * @export
 */
Component.prototype.getReference = function(key) {};

/**
 * Clears a live reference for the value indicated using property syntax.
 * For example, if you use aura:set to set a value and later want to reset the value using <code>component.set()</code>,
 * clear the reference before resetting the value.
 *
 * @param {String}
 *            key The data key for which to clear the reference. For example, <code>"v.attributeName"</code>.
 * @public
 * @platform
 * @export
 */
Component.prototype.clearReference = function(key) {};

/**
 * Returns the value referenced using property syntax.
 * For example, <code>cmp.get("v.attr")</code> returns the value of the attr aura:attribute.
 *
 * @param {String}
 *            key - The data key to look up on the Component.
 * @public
 * @platform
 * @export
 */
Component.prototype.get = function(key) {};

/**
 * Returns a shadow value. Used for programmatically adding values after FCVs.
 * THIS IS NOT FOR YOU. DO NOT USE.
 *
 * @param {String}
 *            key The data key to look up on the Component.
 * @private
 */
Component.prototype.getShadowAttribute = function(key) {};

/**
 * Sets the value referenced using property syntax.
 *
 * @param {String}
 *            key - The data key to set on the Component. E.g.
 *            <code>cmp.set("v.key","value")</code>
 * @param {Object}
 *            value - The value to set
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.set = function(key, value, ignoreChanges) {};

//#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
/**
 * Add a warning in the console if a list of components is not rendered by the end
 * of the current event loop. This pattern usually lead to memory leaks.
 *
 * @param {Component[]} prevComp The list of component that has been replaced
 * @param {string} key The attribute key
 */
Component.prototype.trackComponentReplacement = function(prevCmps, key) {};
//#end

/**
 * Sets a shadow attribute. Used for programmatically adding values after FCVs.
 * THIS IS NOT FOR YOU. DO NOT USE.
 *
 * @param {String}
 *            key The data key to set on the Component.
 * @private
 */
Component.prototype.setShadowAttribute = function(key,value) {};


/**
 * @export
 */
Component.prototype.markDirty=function(reason){};

/**
 * @export
 */
Component.prototype.markClean=function(value) {};

Component.prototype.fireChangeEvent=function(key,oldValue,newValue,index){};

/**
 * Sets a flag to tell the rendering service whether or not to destroy this component when it is removed
 * from its rendering facet. Set to false if you plan to keep a reference to a component after it has
 * been unrendered or removed from a parent facet. Default is true: destroy once orphaned.
 * @param {Boolean} destroy - The flag to specify whether or not to destroy this component automatically.
 *   We don't recommend setting the value to false. If you do, be careful to avoid memory leaks.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.autoDestroy = function(destroy) {};

/**
 * Gets the concrete implementation of a component. If the component is
 * concrete, the method returns the component itself. For example, call this
 * method to get the concrete component of a super component.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.getConcreteComponent = function() {};

/**
 * Returns true if the component is concrete, or false otherwise.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.isConcrete = function() {};

/**
 * Returns the value provider.
 *
 * @return {Object} value provider
 * @public
 * @export
 */
Component.prototype.getAttributeValueProvider = function() {};

/**
 * @export
 */
Component.prototype.setAttributeValueProvider = function (avp) {};

/**
 * Returns the value provider of the component.
 *
 * @return {Object} component or value provider
 * @public
 * @export
 */
Component.prototype.getComponentValueProvider = function() {};

/**
 * Returns the owner of the component. This should represent the lexical scope for markup components, and the
 * component calling the create method for dynamically instantiated components, i.e. $A.createComponent().
 *
 * @return {Object} Owning component
 * @public
 * @export
 */
Component.prototype.getOwner = function() {};

/**
 * Returns the container component of the component.
 * The container is the component that owns the facet attribute for
 * which this component is a value. This may be different than the
 * result from getComponentValueProvider(), particularly if this
 * component was transcluded into its container and not created
 * directly by its container.
 *
 * @return {Object} Containing component
 * @private
 */
Component.prototype.getContainer = function() {};

/**
 * Adds Custom ValueProviders to a component
 * @param {String} key - string by which to identify the valueProvider. Used in expressions in markup, etc.
 * @param {Object} valueProvider - the object to request data from. Must implement .get(expression), can implement .set(key,value).
 * @public
 * @platform
 * @export
 */
Component.prototype.addValueProvider=function(key,valueProvider){};

/**
 * Removes a custom value provider from a component
 * @param {String} key string by which to identify the valueProvider to remove.
 * @public
 */
Component.prototype.removeValueProvider=function(key){};

/**
 * Returns the model for this instance, if one exists. Shorthand :
 * <code>get("m")</code>
 *
 * @public
 * @export
 */
Component.prototype.getModel = function() {};

/**
 * Returns a new event instance of the named component event.
 * @example
 * // evtName matches the name attribute in aura:registerEvent
 * cmp.getEvent("evtName")
 *
 * @param {String}
 *            name - The name of the Event.
 * @public
 * @platform
 * @export
 */
Component.prototype.getEvent = function(name) {};

/**
 * Get an event by descriptor qualified name.
 *
 * This is only used by action for firing of component events. It is a bit of a
 * hack (reversing the map).
 *
 * @param {String}
 *            descriptor a descriptor qualified name.
 * @return {String} null, or the component event.
 * @protected
 */
Component.prototype.getEventByDescriptor = function(descriptor) {};

/**
 * @private
 */
Component.prototype.fire = function(name) {};

/**
 * Looks up the specified value and checks if it is currently dirty.
 *
 * @returns true if the value is dirty, and false if it is clean or does not
 *          exist.
 * @public
 * @deprecated TEMPORARY WORKAROUND
 * @export
 */
Component.prototype.isDirty = function(expression) {};

/**
 * Returns true if the component has not been destroyed.
 *
 * @public
 * @platform
 * @export
 */
Component.prototype.isValid = function() {};

/**
 * Returns a string representation of the component for logging.
 *
 * @public
 * @export
 */
Component.prototype.toString = function() {};

/**
 * Returns component serialized as Json string
 *
 * @private
 */
Component.prototype.toJSON = function() {};

/**
 * Returns an object whose keys are the lower-case names of Aura events for
 * which this component currently has handlers.
 * @export
 */
Component.prototype.getHandledEvents = function() {};

/**
 * Check if we have an event handler attached.
 *
 * @param {String}
 *            eventName The event name associated with this component.
 * @export
 */
Component.prototype.hasEventHandler = function(eventName) {};

/**
 * Returns an array of this component's facets, i.e., attributes of type
 * <code>aura://Aura.Component[]</code>
 * @export
 */
Component.prototype.getFacets = function() {};


/**
 * Returns true if this is a flavorable html element.
 * @export
 */
Component.prototype.isFlavorable = function() {};

/**
 * Gets the flavor reference. This is either the flavor explicitly set on the
 * component instance (component def ref) or it is the default flavor of the
 * component, if a default (or app override) exists.
 *
 * @returns {String} The flavor, e.g., "default" or "xyz.flavors.default", etc...
 * @export
 */
Component.prototype.getFlavor = function() {};

/**
 * Invoke the render method defined on the component.
 * @export
 */
Component.prototype.render = function() {};

/**
 * Invoke the afterRender method defined on the component.
 * @export
 */
Component.prototype.afterRender = function() {};


/**
 * Invoke the rerender method defined on the component.
 * @export
 */
Component.prototype.rerender = function() {};

/**
 * Invoke the unrender method defined on the component.
 * @export
 */
Component.prototype.unrender = function() {};

/**
 * Get the expected version number of a component based on its caller's requiredVersionDefs
 * Note that for various rendering methods, we cannot rely on access stack.
 * We use creation version instead.
 * @platform
 * @export
 */
Component.prototype.getVersion = function() {};

/**
 * get version from context access stack
 *
 * @private
 */
Component.prototype.getVersionInternal = function() {};

Component.prototype.getValueProvider = function(key) {};

/**
 * Create the value providers
 */
Component.prototype.setupValueProviders = function(customValueProviders) {};

Component.prototype.createActionValueProvider = function() {};

Component.prototype.createStyleValueProvider = function() {};

/**
 * A reference to the ComponentDefinition for this instance
 */
Component.prototype.setupComponentDef = function(config) {};

Component.prototype.createComponentStack = function(facets, valueProvider){};


Component.prototype.setupSuper = function(configAttributes) {};

Component.prototype.setSuperComponent = function(component) {};

Component.prototype.isCollectionOfAuraComponentDefs = function (facetValueConfig) {};

Component.prototype.setupAttributes = function(config, localCreation) {};


Component.prototype.validatePartialConfig=function(config, partialConfig){};

Component.prototype.getMethodHandler = function(methodDef,methodEventDef){};

Component.prototype.getActionCaller = function(valueProvider, actionExpression) {};

/**
 * Creates the e.* value provider
 */
Component.prototype.createEventValueProvider = function() {};

/**
 * Gets the event dispatcher.
 *
 * @public
 * @export
 */
Component.prototype.getEventDispatcher = function() {};

Component.prototype.setupComponentEvents = function(cmp, config) {};

Component.prototype.setupApplicationEventHandlers = function() {};

Component.prototype.setupValueEventHandlers = function(cmp) {};

Component.prototype.setupMethods = function() {};

Component.prototype.setupModel = function(config) {};

Component.prototype.setupFlavors = function(config, configAttributes) {};

Component.prototype.doIndex = function(cmp) {};

Component.prototype.doDeIndex = function() {};

Component.prototype.replaceComponentClass = function(descriptor) {};

Component.prototype.injectComponent = function(config, localCreation) {};

/**
 * Runs the provide method and returns the component definition.
 * Throws an error if the provide method is not found.
 * @param {Boolean} localCreation
 */
Component.prototype.provide = function(localCreation) {};

Component.prototype.setProvided = function(realComponentDef, attributes) {};

Component.prototype.associateRenderedBy = function(cmp, element) {};

/**
 * Resolves a locator that targets targetCmp from within this component
 * @param targetCmp
 * @param includeMetadata
 *
 * @returns The locator object which contains the target & scope IDs and locator context resolved
 */
Component.prototype.getLocator = function(targetCmp, includeMetadata) {};

Aura.Component.Component = Component;
