/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * A registry of Component classes.
 * @constructor
 */
function ComponentClassRegistry () {
    // We delay the creation of the definition of a class till it's requested.
    // The function that creates the component class is a classExporter.
    this.classExporter = {};

    // Collection of all the component classes we generate for
    // proper stack traces and proper use of prototypical inheritance
    this.classConstructors = {};
}

/**
 * By default all components will use Aura.Component.Component as the constructor.
 * This wires up all the features a component might need.
 * Some rootComponents are moving into the framework with custom Component extensions.
 * This map defines the constructor they use in buildConstructor
 */
ComponentClassRegistry.prototype.customConstructorMap = {
    /*eslint-disable no-undef*/
    "aura$text":TextComponent,
    "aura$html":HtmlComponent,
    "aura$expression": ExpressionComponent,
    "aura$if":IfComponent,
    "aura$iteration":IterationComponent,
    "aura$component":BaseComponent
};

/**
 * Detects if the component class exists without actually defining it.
 * @param {String} descriptor The qualified name of the component in the form markup://namespace:component
 */
ComponentClassRegistry.prototype.hasComponentClass = function(descriptor) {};

/**
 * The function that handles definitions of component classes.
 * @param {String} descriptor in the form markup://namespace:component
 * @param {Function} exporter A function that when executed will return the component object litteral.
 * @export
 */
ComponentClassRegistry.prototype.addComponentClass = function(descriptor, exporter){};

/**
 * Get or build the class constructor for the specified component.
 * @param {String} descriptor in the form markup://namespace:component
 * @returns Either the class that defines the component you are requesting, or undefined if not found.
 * @export
 */
ComponentClassRegistry.prototype.getComponentClass = function(descriptor, def) {};

ComponentClassRegistry.prototype.buildInteropComponentClass = function(descriptor, def) {};

/**
 * Build the class for the specified component.
 * This process is broken into subroutines for clarity and maintainabiity,
 * and those are all combined into one single scope by the compiler.
 * @param {Object} componentProperties The pre-built component properties.
 * @returns {Function} The component class.
 */
ComponentClassRegistry.prototype.buildComponentClass = function(componentProperties) {};


/**
 * Augment the component class properties with their respective inheritance. The
 * inner classes are "static" classes, and currenltly, only the helper is inherited.
 * @param {Object} componentProperties The pre-built component properties.
 */
ComponentClassRegistry.prototype.buildInheritance = function(componentProperties) {};

/**
 * Augment the component class properties with the component libraries. This method
 * attached the component imports (a.k.a. "libraries") on the properties.
 * @param {Object} componentProperties The pre-built component properties.
 */
ComponentClassRegistry.prototype.buildLibraries = function(componentProperties) {};

/**
 * Build the class constructor for the specified component.
 * @param {Object} componentProperties The pre-built component properties.
 * @returns {Function} The component class.
 */

ComponentClassRegistry.prototype.buildConstructor = function(componentProperties, name, Ctor) {};

Aura.Component.ComponentClassRegistry = ComponentClassRegistry;
