/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @class ComponentDef
 *
 * Constructs a new ComponentDef object, which is a component definition.
 * A ComponentDef instance is created as part of Aura initialization.
 *
 * @constructor
 * @protected
 * @export
 */
function ComponentDef(config) {}

ComponentDef.prototype.hasInit = function() {};

/**
 * Returns a DefDescriptor object.
 *
 * @returns {DefDescriptor} A DefDescriptor object contains a prefix, namespace,
 *          and name.
 * @export
 */
ComponentDef.prototype.getDescriptor = function() {};

/**
 * Checks whether the Component is abstract. Returns true if the component is
 * abstract.
 *
 * @returns {Boolean} True if component is abstract, or false otherwise.
 * @export
 */
ComponentDef.prototype.isAbstract = function() {};

/**
 * Returns the component definition for the immediate super type or null if none
 * exists (should only be null for aura:component).
 *
 * @return {ComponentDef} The ComponentDef for the immediate super type
 * @export
 */
ComponentDef.prototype.getSuperDef = function() {};

/**
 * Gets the Helper instance. This method is for backward compatibility,
 * the helper is now an integral part of the component class.
 *
 * @returns {Helper}
 * @export
 */
ComponentDef.prototype.getHelper = function() {};

/**
 * Returns RequiredVersionDef objects.
 *
 * @returns {RequiredVersionDefs}
 */
ComponentDef.prototype.getRequiredVersionDefs = function() {};

/**
 * Checks whether the component has remote dependencies. Returns true if remote
 * dependencies are found.
 *
 * @returns {Boolean} True if remote dependencies exist, or false otherwise.
 */
ComponentDef.prototype.hasRemoteDependencies = function() {};

/**
 * Gets all the StyleDef objects, including inherited ones, for this
 * ComponentDef.
 *
 * @returns {StyleDef}
 */
ComponentDef.prototype.getAllStyleDefs = function() {};

/**
 * Gets all the FlavoredStyleDef objects, including inherited ones, for this
 * ComponentDef.
 *
 * @returns {StyleDef}
 */
ComponentDef.prototype.getAllFlavoredStyleDefs = function() {};

/**
 * Gets the CSS class name to use for Components of this type. Includes the
 * class names from all StyleDefs, including inherited ones, associated with
 * this ComponentDef. If multiple class names are found, the return value is a
 * space-separated list of class names. This string can be applied directly to
 * DOM elements rendered by Components of this type.
 *
 * @returns {String} The style class name
 * @export
 */
ComponentDef.prototype.getStyleClassName = function() {};

/**
 * Gets the style definition. Returns a StyleDef object.
 *
 * @returns {StyleDef}
 * @export
 */
ComponentDef.prototype.getStyleDef = function() {};

/**
 * Gets the default flavor name, either from app-specified overrides or the
 * default specified on the component def.
 *
 * @returns {String} The flavor, e.g., "default" or "xyz.flavors.default", etc...
 * @export
 */
ComponentDef.prototype.getDefaultFlavor = function() {};

/**
 * Gets the default flavor explicitly set on the component def (or one of its supers).
 *
 * @returns {String}
 */
ComponentDef.prototype.getExplicitDefaultFlavor = function() {};

/**
 * Gets whether this def has at least one flavorable child element.
 *
 * @returns {Boolean}
 * @export
 */
ComponentDef.prototype.hasFlavorableChild = function() {};

/**
 * Gets the set of default flavor overrides.
 *
 * @returns {FlavorsDef}
 * @export
 */
ComponentDef.prototype.getFlavorOverrides = function() {};

/**
 * Returns true if this or a super component is dynamically flavorable.
 * Performs a faster check than #getDynamicallyFlavorable.
 * @returns {Boolean}
 * @private
 */
ComponentDef.prototype.isDynamicallyFlavorable = function() {};

/**
 * Returns a list of component defs from the inheritance hierarchy that are
 * marked dynamically flavorable, including this one if applicable.
 *
 * To simply perform a boolean check, use #isDynamicallyFlavorable instead.
 * @returns {Array}
 * @private
 */
ComponentDef.prototype.getDynamicallyFlavorable = function() {};

/**
 * Gets all the attribute definitions. Returns an AttributeDef object.
 *
 * @returns {AttributeDefSet}
 * @export
 */
ComponentDef.prototype.getAttributeDefs = function() {};

/**
 * Gets the component facets. A facet is any attribute of type Aura.Component[].
 *
 * @returns {Object}
 * @export
 */
ComponentDef.prototype.getFacets = function() {};


/**
 * Gets Facet by facet descriptor
 * @param  {String} facetName The name of the facet. If the attribute name was param1, then facetName would be "param1". Does not handle v., so v.param1 would not work.
 * @return {Object}           The facet descriptor. Is current an object with two properties, descriptor which is the facetName you specified, and value which is the contents of the facet.
 */
ComponentDef.prototype.getFacet = function(facetName) {};

/**
 * Gets the controller definition. Returns a ControllerDef object.
 *
 * @returns {ControllerDef}
 * @export
 */
ComponentDef.prototype.getControllerDef = function() {};

/**
 * Gets the model definition. Returns a ModelDef object.
 *
 * @returns {ModelDef}
 * @export
 */
ComponentDef.prototype.getModelDef = function() {};

/**
 * Value Event Enum
 *
 * @returns {ModelDef}
 */
ComponentDef.valueEvents = {
    "change" : "aura:valueChange",
    "destroy": "aura:valueDestroy",
    "init"   : "aura:valueInit",
    "render" : "aura:valueRender"
};

/**
 * Returns the event definitions.
 *
 * @param {String}
 *            The name of the event definition.
 * @param {Boolean}
 *            includeValueEvents Set to true to include the value events.
 * @returns{Object}
 * @export
 */
ComponentDef.prototype.getEventDef = function(name, includeValueEvents) {};

/**
 * Get an event name by descriptor qualified name.
 *
 * This is only used in the case of an action firing a component event. It is a
 * bit of a hack, but will give back the name of the event that corresponds to
 * the descriptor.
 *
 * @param {String}
 *            descriptor a descriptor qualified name.
 * @return {String} null, or the component fired event name.
 * @protected
 */
ComponentDef.prototype.getEventNameByDescriptor = function(descriptor) {};

/**
 * Gets all events associated with the Component.
 *
 * @returns {Object}
 * @export
 */
ComponentDef.prototype.getAllEvents = function() {};

/**
 * Gets the application handler definitions.
 *
 * @returns {Object}
 * @export
 */
ComponentDef.prototype.getAppHandlerDefs = function() {};

/**
 * Gets the component handler definitions.
 *
 * @returns {Object}
 * @export
 */
ComponentDef.prototype.getCmpHandlerDefs = function() {};

/**
 * Gets the value of the handler definitions.
 *
 * @returns {Object}
 */
ComponentDef.prototype.getValueHandlerDefs = function() {};

/**
 * Converts a ComponentDef object to type String.
 *
 * @returns {String}
 * @export
 */
ComponentDef.prototype.toString = function() {};

/**
 * Checks whether the Component is an instance of the given component name (or
 * interface name).
 *
 * @param {String}
 *            name The name of the component (or interface), with a format of
 *            <code>namespace:componentName</code> (e.g.,
 *            <code>ui:button</code>).
 * @returns {Boolean} True if the Component is an instance, or false otherwise.
 * @export
 */
ComponentDef.prototype.isInstanceOf = function(name) {};

/**
 * Primarily used by isInstanceOf().
 *
 * @private
 */
ComponentDef.prototype.implementsDirectly = function(type) {};

/**
 * Gets the location change event. Returns the qualified name of the event in
 * the format <code>markup://aura:locationChange</code>.
 * @export
 */
ComponentDef.prototype.getLocationChangeEvent = function() {};

/**
 * @export
 */
ComponentDef.prototype.getLayouts = function() {};

/**
 * Returns the component locator definition
 *
 * @returns {Object} LocatorDef Object
 * @export
 */
ComponentDef.prototype.getLocatorDefs = function() {};

/**
 * @private
 */
ComponentDef.prototype.initSuperDef = function(config) {};

/**
 * Setup the style defs details.
 *
 * Note that the style defs are in reverse order so that they get applied in
 * forward order.
 *
 * @private
 */
ComponentDef.prototype.initStyleDefs = function() {};

/**
 * Returns the minimum API version that a component should be at to use this component
 * @returns {undefined | String}
 * @private
 */
ComponentDef.prototype.getMinVersion = function() {};

/**
 * Returns the API Version of the ComponentDef
 *
 * @private
 * @return {String | undefined} The API version string.
 */
ComponentDef.prototype.getApiVersion = function() {};

Aura.Component.ComponentDef = ComponentDef;
