/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description Storage for component, library, and event definitions. If persistent storage
 * is not available then most operations are noops.
 * @constructor
 * @protected
 */
function ComponentDefStorage() {}

ComponentDefStorage.prototype.STORAGE_NAME = "ComponentDefStorage";

/**
 * Target size, as a percent of max size, for component def storage during eviction.
 */
ComponentDefStorage.prototype.EVICTION_TARGET_LOAD = 0.75;

/**
 * Minimum head room, as a percent of max size, to allocate after eviction and adding new definitions.
 */
ComponentDefStorage.prototype.EVICTION_HEADROOM = 0.1;

/**
 * Cookie used to track when persistent def storage is known to contain a broken graph.
 */
ComponentDefStorage.prototype.BROKEN_GRAPH_COOKIE = "auraBrokenDefGraph";

/**
 * Key to use of the MutexLocker to guarantee atomic execution across tabs.
 */
ComponentDefStorage.prototype.MUTEX_KEY = ComponentDefStorage.prototype.STORAGE_NAME;

/**
 * Function to release the mutex, set while the mutex is held.
 */
ComponentDefStorage.prototype.mutexUnlock = undefined;

/**
 * Queue of operations on the def store. Used to prevent concurrent DML on the underlying AuraStorage, or analysis
 * of the AuraStorage contents during DML, which often results in a broken def graph. This is set to an array
 * when an operation is in-flight.
 */
ComponentDefStorage.prototype.queue = undefined;

/**
 * Map of defs to store. To improve performance defs are batched while the mutex around def AuraStorage is acquired.
 */
ComponentDefStorage.prototype.defsToStore = undefined;

/**
 * Whether defs are backed up to persistent storage. Evaluated to a boolean
 * on first request to operate on defs (get, put, or remove).
 */
ComponentDefStorage.prototype.useDefStore = undefined;

/**
 * The AuraStorage instance.
 */
ComponentDefStorage.prototype.storage = undefined;


/**
 * Whether to use storage for component definitions.
 * @returns {Boolean} Whether to use storage for component definitions.
 */
ComponentDefStorage.prototype.useDefinitionStorage = function() {};

/**
 * Sets up persistent storage for definitions if the necessary prerequisites are satisfied.
 * If they are not then storage is not setup and definitions are not backed up.
 */
ComponentDefStorage.prototype.setupDefinitionStorage = function() {};

/**
 * Gets the storage for component definitions.
 * @return {AuraStorage|null} The component def storage or null if it's disabled.
 */
ComponentDefStorage.prototype.getStorage = function () {};

/**
 * Stores component and library definitions into storage.
 * @param {Array} cmpConfigs The component definitions to store.
 * @param {Array} libConfigs The library definitions to store.
 * @param {Array} evtConfigs The event definitions to store.
 * @param {Array} moduleConfigs The module definitions to store.
 * @param {AuraContext} context The Aura context.
 * @return {Promise} Promise that resolves when storing is complete.
 */
ComponentDefStorage.prototype.storeDefs = function(cmpConfigs, libConfigs, evtConfigs, moduleConfigs, context) {};

/**
 * Removes definitions from storage.
 *
 * Must be called from within ComponentDefStorage.enqueue() for mutex. The mutex
 * is not acquired within this function because the entire selective eviction flow
 * (load, analyze, evict, repeat) must be atomic.
 *
 * @param {String[]} descriptors The descriptors identifying the definitions to remove.
 * @return {Promise} A promise that resolves when the definitions are removed.
 */
ComponentDefStorage.prototype.removeDefs = function(descriptors) {};


/**
 * Gets all definitions from storage.
 * @return {Promise} A promise that resolves with a map of descriptor name to definitions. If
 *  the underlying storage fails or is disabled then the promise resolves to an empty map.
 */
ComponentDefStorage.prototype.getAll = function () {};

/**
 * Asynchronously retrieves all definitions from storage and adds to component service.
 * @return {Promise} A promise that resolves when definitions are restored.
 */
ComponentDefStorage.prototype.restoreAll = function(context) {};


/**
 * Enqueues a function that requires isolated access (including across tabs) to the underlying AuraStorage.
 * @param {Function} execute The function to execute.
 * @return {Promise} A promise that resolves when the provided function executes.
 */
ComponentDefStorage.prototype.enqueue = function(execute) {};


/**
 * Clears persisted definitions and all dependent stores and context.
 * @param {Object=} metricsPayload An optional payload to send to metrics service.
 * @return {Promise} A promise that resolves when all stores are cleared.
 */
ComponentDefStorage.prototype.clear = function(metricsPayload) {};


ComponentDefStorage.prototype.getBrokenGraphCookie = function() {};

ComponentDefStorage.prototype.setBrokenGraphCookie = function() {};

ComponentDefStorage.prototype.clearBrokenGraphCookie = function() {};




Aura.Component.ComponentDefStorage = ComponentDefStorage;
