/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Construct a new HtmlComponent.
 *
 * @public
 * @class
 * @constructor
 *
 * @param {Object}
 *            config - component configuration
 * @param {Boolean}
 *            [localCreation] - local creation
 * @export
 */
function HtmlComponent(config, localCreation) {}

HtmlComponent.prototype = Object.create(Component.prototype);

// Not returning anything in these since empty functions will not be called by the JS engine as an

/** The SuperRender calls are blank since we will never have a super, no need to ever do any logic to for them. */
HtmlComponent.prototype.superRender = function(){};
HtmlComponent.prototype.superAfterRender = function(){};
HtmlComponent.prototype.superRerender = function(){};
HtmlComponent.prototype.superUnrender = function(){};

/** No Super, so just return undefined */
HtmlComponent.prototype.getSuper = function(){};

/** Will always be Superest, so no need to check for a super */
HtmlComponent.prototype.getSuperest = function(){};

/** 
 * Component.js has logic that is specific to HtmlComponent. Great! So we can move that into here and out of Component.js
 * That logic is the LockerService part to assign trust to the owner.
 */
HtmlComponent.prototype.setupComponentDef = function() {};

/**
 * Not all the value providers are necessary for HTML. We don't need the component events (e.) or controller (c.)
 */
HtmlComponent.prototype.setupValueProviders = function(customValueProviders) {};

/**
 * Simple type checking. All simple components implement aura:rootComponent and cannot be extended, 
 * so the simple condition here is sufficient unless any of the individual components change.
 */
HtmlComponent.prototype.isInstanceOf = function(type) {};

/**
 * Copied from htmlRenderer.js
 * Now has access to internal APIs and advanced mode compilation.
 */
HtmlComponent.prototype["renderer"] = {
    "render" : function(component, helper) {},

    "rerender" : function(component, helper) {},

    "afterRender" : function(component, helper) {},

    "unrender" : function(component, helper) {}
};


/** Copied unchanged from htmlHelper.js */
HtmlComponent.prototype["helper"] = {
    SPECIAL_BOOLEANS: {
        "checked": true,
        "selected": true,
        "disabled": true,
        "readonly": true,
        "multiple": true,
        "ismap": true,
        "defer": true,
        "declare": true,
        "noresize": true,
        "nowrap": true,
        "noshade": true,
        "compact": true,
        "autocomplete": true,
        "required": true
    },

    SPECIAL_CASINGS: {
        "readonly": "readOnly",
        "colspan": "colSpan",
        "rowspan": "rowSpan",
        "bgcolor": "bgColor",
        "tabindex": "tabIndex",
        "usemap": "useMap",
        "accesskey": "accessKey",
        "maxlength": "maxLength",
        "for": "htmlFor",
        "class": "className",
        "frameborder": "frameBorder"
    },

    // "void elements" as per http://dev.w3.org/html5/markup/syntax.html#syntax-elements
    BODYLESS_TAGS: {
        "area": true,
        "base": true,
        "br": true,
        "col": true,
        "command": true,
        "embed": true,
        "hr": true,
        "img": true,
        "input": true,
        "keygen": true,
        "link": true,
        "meta": true,
        "param": true,
        "source": true,
        "track": true,
        "wbr": true
    },

    // List must be kept in sync with org.auraframework.def.HtmlTag enum
    ALLOWED_TAGS:{
        "a":true,
        "abbr":true,
        "acronym":true,
        "address":true,
        "area":true,
        "article":true,
        "aside":true,
        "audio":true,
        "b":true,
        "bdi":true,
        "bdo":true,
        "big":true,
        "blockquote":true,
        "body":true,
        "br":true,
        "button":true,
        "caption":true,
        "canvas":true,
        "center":true,
        "cite":true,
        "code":true,
        "col":true,
        "colgroup":true,
        "command":true,
        "datalist":true,
        "dd":true,
        "del":true,
        "details":true,
        "dfn":true,
        "dir":true,
        "div":true,
        "dl":true,
        "dt":true,
        "em":true,
        "fieldset":true,
        "figure":true,
        "figcaption":true,
        "footer":true,
        "form":true,
        "h1":true,
        "h2":true,
        "h3":true,
        "h4":true,
        "h5":true,
        "h6":true,
        "head":true,
        "header":true,
        "hgroup":true,
        "hr":true,
        "html":true,
        "i":true,
        "iframe":true,
        "img":true,
        "input":true,
        "ins":true,
        "keygen":true,
        "kbd":true,
        "label":true,
        "legend":true,
        "li":true,
        "link":true,
        "map":true,
        "mark":true,
        "menu":true,
        "meta":true,
        "meter":true,
        "nav":true,
        "ol":true,
        "optgroup":true,
        "option":true,
        "output":true,
        "p":true,
        "pre":true,
        "progress":true,
        "q":true,
        "rp":true,
        "rt":true,
        "ruby":true,
        "s":true,
        "samp":true,
        "script":true,
        "section":true,
        "select":true,
        "small":true,
        "source":true,
        "span":true,
        "strike":true,
        "strong":true,
        "style":true,
        "sub":true,
        "summary":true,
        "sup":true,
        "table":true,
        "tbody":true,
        "td":true,
        "textarea":true,
        "tfoot":true,
        "th":true,
        "thead":true,
        "time":true,
        "title":true,
        "tr":true,
        "track":true,
        "tt":true,
        "u":true,
        "ul":true,
        "var":true,
        "video":true,
        "wbr":true
    },

    // string constants used to save and remove click handlers
    NAMES: {
        "domHandler": "fcDomHandler",
        "hashHandler": "fcHashHandler"
    },

    validateTagName: function(tagName) {},
    
    caseAttribute: function (attribute) {},

    /**
     * Adds or replaces existing "onclick" handler for the given handlerName.
     *
     * Is used to add independent handlers eg. dom level and hash navigation handling on <a href/>
     */
    addNamedClickHandler: function (element, handler, handlerName) {},

    domEventHandler: function (event) {},

    // NOTE: Do not remove attributes from this method
    // Used by MetricsService plugin to collect information
    dispatchAction: function (action, event) {},

    canHaveBody: function (component) {},

    createHtmlAttribute: function (component, element, name, attribute) {},

    destroyHtmlAttribute: function (component, name, attribute) {},

    processJavascriptHref: function (element) {},

    inlineJavasciptCSPViolationPreventer: function(event) {}
};

Aura.Component.HtmlComponent = HtmlComponent;
