/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description Represents a Aura client-side context, created during HTTP requests for component definitions. A context
 *            can include a mode, such as "DEV" for development mode or "PROD" for production mode.
 * @constructor
 * @protected
 * @class AuraContext
 * @param {Object} config the 'founding' config for the context from the server.
 * @param {Function} initCallback an optional callback invoked after the config has finished its asynchronous initialization.
 * @export
 */
Aura.Context.AuraContext = function AuraContext(config, initCallback) {};

/**
 * Unique id for the current instance of Aura. In a multi-tab scenario
 * each tab will have a unique id.
 */
Aura.Context.AuraContext.CLIENT_SESSION_ID = [
    window.pageStartTime, // first byte sent
    Math.round(Aura.time() * 1000000), // current time (microseconds)
    Math.random().toString(16).substr(2)
].join('').substring(0, 32);

/**
 * Temporary shim, until W-2812858 is addressed to serialize GVPs as a map and fix $A GVPs.
 * Convert config GVPs from array to map, and merge $A GVPs, and create the context GVPs.
 * @export
 */
Aura.Context.AuraContext.prototype.initGlobalValueProviders = function(gvps, callback) {};


/**
 * Returns the mode for the current request. Defaults to "PROD" for production mode and "DEV" for development mode.
 * The HTTP request format is <code>http://<your server>/namespace/component?aura.mode=PROD</code>.
 *
 * @return {string} the mode from the server.
 * @export
 */
Aura.Context.AuraContext.prototype.getMode = function() {};

/**
 * Gets the application configuration tokens allowed to be used in component markup.
 * @private
 */
Aura.Context.AuraContext.prototype.getTokens=function(){};

/**
 * Sets the application configuration tokens allowed to be used in component markup.
 * @param tokens The object map containing name value pairs of tokens.
 * @private
 */
Aura.Context.AuraContext.prototype.setTokens=function(tokens){};

/**
 * Adds a new global value provider.
 * @param type The key to identify the valueProvider.
 * @param valueProvider The valueProvider to add.
 * @private
 */
Aura.Context.AuraContext.prototype.addGlobalValueProvider = function(type,valueProvider) {};

/**
 * Provides access to global value providers.
 * For example, <code>$A.get("$Label.Related_Lists.task_mode_today");</code> gets the label value.
 *
 * @return {GlobalValueProviders}
 * @private
 */
Aura.Context.AuraContext.prototype.getGlobalValueProvider = function(type) {};

/**
 * JSON representation of context for server requests.
 *
 * This must remain in sync with AuraTestingUtil so that we can accurately test.
 *
 * @return {String} json representation
 * @private
 */
Aura.Context.AuraContext.prototype.encodeForServer = function(includeDynamic, includeCacheKeyForCacheableXHR) {};

/**
 * @param {Object} otherContext the context from the server to join in to this one.
 * @export
 */
Aura.Context.AuraContext.prototype.merge = function(otherContext, allowMissmatch) {};

/**
 * @param $Label mapping of additional labels to add
 * @export
 */
Aura.Context.AuraContext.prototype.mergeLabels = function(labels) {};

/**
 * FIXME: this should return a string, and it should probably not even be here.
 *
 * @return {number} the 'num' for this context
 * @private
 * @export
 */
Aura.Context.AuraContext.prototype.getNum = function() {};

/**
 * @private
 */
Aura.Context.AuraContext.prototype.incrementNum = function() {};

/**
 * @private
 */
Aura.Context.AuraContext.prototype.incrementRender = function() {};

/**
 * @return {Number} incremented transaction number
 * @private
 * @export
 */
Aura.Context.AuraContext.prototype.incrementTransaction = function() {};

/**
 * @return {Number} gets the number of the current transaction
 * @private
 */
Aura.Context.AuraContext.prototype.getTransaction = function() {};

/**
 * @private
 */
Aura.Context.AuraContext.prototype.updateTransactionName = function(_transactionName) {};

/**
 * @return {String} gets the name of the transaction
 * @private
 */
Aura.Context.AuraContext.prototype.getTransactionName = function() {};

/**
 * @private
 */
Aura.Context.AuraContext.prototype.clearTransactionName = function() {};

/**
 * @return {Number} Next global ID
 * @private
 */
Aura.Context.AuraContext.prototype.getNextGlobalId = function() {};

/**
 * Returns components configs object
 * @param {String} creationPath creation path to check
 * @return {Boolean} Whether creation path is in component configs
 * @private
 */
Aura.Context.AuraContext.prototype.containsComponentConfig = function(creationPath) {};

/**
 * @param {string} creationPath the creation path to look up.
 * @private
 */
Aura.Context.AuraContext.prototype.getComponentConfig = function(creationPath) {};

/**
 * When we have consumed the component config from the context, its worth removing it to narrow down
 * the list of pending configs left to handle.
 * @param {String} creationPath is the components creationPath that we are operating on.
 * @private
 */
Aura.Context.AuraContext.prototype.removeComponentConfig = function(creationPath) {};

/**
 * Returns the app associated with the request.
 * @export
 */
Aura.Context.AuraContext.prototype.getApp = function() {};

/**
 * @param {Object}
 *      otherComponentConfigs the component configs from the server to join in.
 * @param {string}
 *      actionId the id of the action that we are joining in (used to amend the creationPath).
 * @private
 */
Aura.Context.AuraContext.prototype.joinComponentConfigs = function(otherComponentConfigs, actionId) {};

/**
 * Internal routine to clear out component configs to factor out common code.
 *
 * @param {string} actionId the action id that we should clear.
 * @param {boolean} logit should we log as we go? including errors.
 * @return {number} the count of component configs removed.
 * @private
 */
Aura.Context.AuraContext.prototype.internalClear = function(actionId, logit) {};

/**
 * finish off the component configs for an action.
 *
 * This routine looks through all of the pending component configs, and
 * flags any that are un-consumed at the end of the action. This is a
 * relatively strict enforcement, but since we have all of the partial
 * configs to create the components, it is not clear why we would leave
 * them lying around.
 *
 * The Rule: You must consume component configs in the action callback.
 * you may _not_ delay creating components.
 *
 * @param {string} actionId the action id that we should clear.
 * @private
 */
Aura.Context.AuraContext.prototype.finishComponentConfigs = function(actionId) {};

/**
 * Clear out pending component configs.
 *
 * This routine can be used in error conditions (or in tests) to clear out
 * configs left over by an action. In this case, we remove them, and drop
 * them on the floor to be garbage collected.
 *
 * @public
 * @param {string} actionId the action id that we should clear.
 * @return {number} the count of component configs removed.
 * @export
 */
Aura.Context.AuraContext.prototype.clearComponentConfigs = function(actionId) {};

/**
 * @private
 */
Aura.Context.AuraContext.prototype.joinLoaded = function(loaded, dontOverride) {};

/**
 * Add back a loaded pair that was extracted using 'findLoaded'.
 *
 * @param {Object} pair the object returned by findLoaded.
 * @private
 */
Aura.Context.AuraContext.prototype.addLoaded = function(pair) {};

/**
 * Find a 'loaded' pair for a descriptor.
 *
 * @param {String} descriptor the descriptor string.
 * @param {Object=} [loaded] Object to search
 * @returns {Object} an object that can be passed to addloaded.
 * @private
 */
Aura.Context.AuraContext.prototype.findLoaded = function(descriptor, loaded) {};

/**
 * This should be private but is needed for testing... ideas?
 *
 * ... should move to $A.test.
 * @export
 */
Aura.Context.AuraContext.prototype.getLoaded = function() {};

/**
 * Reset the loaded set to its original value at launch.
 */
Aura.Context.AuraContext.prototype.resetLoaded = function() {};

/**
 * DCHASMAN Will be private again soon as part of the second phase of W-1450251
 * @export
 */
Aura.Context.AuraContext.prototype.setCurrentAction = function(action) {};

/**
 * EBA - temporarily made public for helpers to obtain action - return to private when current visibility is determined
 * @public
 * @return {Action} the current action.
 * @export
 */
Aura.Context.AuraContext.prototype.getCurrentAction = function() {};

/**
 * Temporarily made public for Communities - return to private when prefixes are not needed
 * @public
 * @return {String} path prefix.
 * @export
 */
Aura.Context.AuraContext.prototype.getPathPrefix = function () {};

/**
 * Servlet container context path
 * @return {String} Servlet container context path
 * @private
 * @export
 */
Aura.Context.AuraContext.prototype.getContextPath = function() {};

/** @export */
Aura.Context.AuraContext.prototype.setContextPath = function(path) {};

/**
 * @return {boolean} if action public caching is enabled or not
 * @export
 */
Aura.Context.AuraContext.prototype.isActionPublicCachingEnabled = function() {};

/**
 * @return {String} The action public cache key
 * @export
 */
Aura.Context.AuraContext.prototype.getActionPublicCacheKey = function() {};

/**
 * @return {boolean} if URI Addressable Defs enabled or not
 */
Aura.Context.AuraContext.prototype.isURIAddressableDefsEnabled = function() {};

/**
 * @return {boolean} if CDN is enabled
 */
Aura.Context.AuraContext.prototype.isCDNEnabled = function() {};

/**
 * Whether compat source is needed.
 * @returns {Boolean} whether compat is enabled
 */
Aura.Context.AuraContext.prototype.isCompat = function() {};

/**
 * should only be used in AuraContext.
 * @private
 */
Aura.Context.AuraContext.prototype.__setDefaultURIDefsState = function(state) {};

/**
 * @private
 */
Aura.Context.AuraContext.prototype.getURIDefsStateFromQuery = function() {};

/**
 * @private
 */
Aura.Context.AuraContext.prototype.getURIDefsState = function() {};
