/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * A base class for an Aura Action to be passed to an associated component. An Action is created in a client-side or
 * server-side controller. Invoke an Action in a controller by declaring cmp.get("c.actionName"). Call a server-side
 * Action from a client-side controller.
 *
 * @constructor
 * @class
 * @param {Object}
 *            def The definition of the Action.
 * @param {string}
 *            suffix A suffix to distinguish components.
 * @param {function}
 *            method The method for the Action. For client-side Action only. A function to serialize the Action as a
 *            String in the JSON representation.
 * @param {Object}
 *            paramDefs The parameter definitions for the Action.
 * @param {boolean}
 *            background is the action defined as a background action?
 * @param {Component}
 *            cmp The component associated with the Action.
 * @param {boolean}
 *            caboose should this action wait for the next non boxcar action?
 * @platform
 * @export
 */
function Action(def, suffix, method, paramDefs, background, cmp, caboose) {}

// Static methods:

Action.getStorageKey = function(descriptor, params) {};

// Instance methods:

Action.prototype.nextActionId = 0;

/**
 * Gets the Action Id.
 *
 * @returns {string}
 * @private
 * @export
 */
Action.prototype.getId = function() {};

/**
 * Sets the action id
 *
 * @returns {string}
 * @private
 */

 Action.prototype.setId = function (id) {};

/**
 * Gets the next action scoped Id.
 *
 * @returns {string}
 * @private
 */
Action.prototype.getNextGlobalId = function() {};

/**
 *  If a component is replacing the same-named component at the same level (e.g. provider),
 *  this reactivates the path's error detection, so that is can request its location again without
 *  reporting an error.
 *
 *  @private
 */
Action.prototype.reactivatePath = function() {};

/**
 * Forces the creation path to match a given value.
 *
 * This checks to see if the path matches, otherwise, it forces the path
 * to the one supplied. A warning is emitted if the path mismatches but only
 * if it is not the top level.
 *
 * @param {string} path the path to force
 * @private
 */
Action.prototype.forceCreationPath = function(path) {};

/**
 * Mark an action as deferred if it is abortable.
 *
 * This is used to side-track actions that are queued when the primary display is refreshed. Since it
 * is a heuristic, it has a few problems in that it may defer actions that should really not be.
 *
 * @private
 */
Action.prototype.setDeferred = function() {};

/**
 * Check to see if an action has been deferred.
 *
 * @private
 * @returns {Boolean} true if the action was marked deferred.
 */
Action.prototype.isDeferred = function() {};

/**
 * Releases a creation path that was previously forced.
 *
 * This is the mirrored call to 'forceCreationPath' that releases the 'force'.
 * The path must match the call to forceCreationPath, and the path must have
 * been forced.
 *
 * @param {string} path the path to release.
 * @private
 */
Action.prototype.releaseCreationPath = function(path) {};

/**
 * Pushes a new part on the creation path.
 *
 * @param {string} pathPart the new path part to insert.
 * @private
 */
Action.prototype.pushCreationPath = function(pathPart) {};

/**
 * Pops off the path part that was previously pushed.
 *
 * @param {string} pathPart the path part previously pushed.
 * @private
 */
Action.prototype.popCreationPath = function(pathPart) {};

/**
 * Gets the path for the top entry of the path stack.
 *
 * @return {string} the top level path.
 * @private
 */
Action.prototype.topPath = function() {};

/**
 * Sets the path index.
 *
 * @param {number} the index to set.
 * @private
 */
Action.prototype.setCreationPathIndex = function(idx) {};

/**
 * Gets the current creatorPath from the top of the pathStack
 *
 * @returns {String}
 * @private
 */
Action.prototype.getCurrentPath = function() {};

/**
 * Gets the <code>ActionDef</code> object. Shorthand: <code>get("def")</code>
 * <p>
 * See Also: <a href="#reference?topic=api:ActionDef">ActionDef</a>
 * </p>
 *
 * @public
 * @returns {ActionDef} The action definition, including its name, origin, and descriptor.
 * @export
 */
Action.prototype.getDef = function() {};

/**
 * Gets name of the Action.
 *
 * @platform
 * @returns {String} Name of the Action.
 * @export
 */
Action.prototype.getName = function() {};

/**
 * Sets parameters for the Action.
 *
 * @public
 * @param {Object}
 *            config - The key/value pairs for action parameters.
 *             For example, <code>serverAction.setParams({ "record": id });</code> sets a parameter on <code>serverAction</code>.
 * @platform
 * @export
 */
Action.prototype.setParams = function(config) {};

/**
 * Sets a single parameter for the Action.
 *
 * @public
 * @param {string}
 *            key - The name of the parameter to set.
 * @param {Object}
 *            value - the value to set for the parameter.
 * @platform
 * @export
 */
Action.prototype.setParam = function(key, value) {};

/**
 * Gets an action parameter value for a parameter name.
 *
 * @public
 * @param {string}
 *            name - The name of the parameter.
 * @returns {Object} The parameter value
 * @platform
 * @export
 */
Action.prototype.getParam = function(name) {};

/**
 * Gets the collection of parameters for this Action.
 *
 * @public
 * @returns {Object} The key/value pairs that specify the action parameters.
 * @platform
 * @export
 */
Action.prototype.getParams = function() {};

/**
 * Gets the component for this Action.
 *
 * @returns {Component} the component, if any.
 * @private
 */
Action.prototype.getComponent = function() {};

/**
 * Sets the callback function that is executed after the server-side action returns. Call a server-side action from a
 * client-side controller using <code>callback</code>.
 *
 * Note that you can register a callback for an explicit state, or you can use 'ALL' which registers callbacks for
 * "SUCCESS", "ERROR", and "INCOMPLETE" (but not "ABORTED" for historical compatibility). It is recommended that you
 * use an explicit name, and not the default 'undefined' to signify 'ALL'.
 *
 * The valid names are:
 *  * SUCCESS: if the action successfully completes.
 *  * ERROR: if the action has an error (including javascript errors for client side actions)
 *  * INCOMPLETE: if a server side action failed to complete because there is no connection
 *  * ABORTED: if the action is aborted via abort()
 *  * REFRESH: for server side storable actions, this will be called instead of the SUCCESS action when the storage is refreshed.
 *
 * @example
 *
 * action.setCallback(this, function(response) {
 *     var state = response.getState();
 *     // This callback doesn’t reference cmp. If it did,
 *     // you should run an isValid() check
 *     //if (cmp.isValid() && state === "SUCCESS") {
 *     if (state === "SUCCESS") {
 *         // Alert the user with the value returned
 *         // from the server
 *         alert("From server: " + response.getReturnValue());
 *
 *         // do something
 *     }
 *     //else if (cmp.isValid() && state === "INCOMPLETE") {
 *     else if (state === "INCOMPLETE") {
 *         // do something
 *     }
 *     //else if (cmp.isValid() && state === "ERROR") {
 *     else if (state === "ERROR") {
 *         var errors = response.getError();
 *         if (errors) {
 *             if (errors[0] && errors[0].message) {
 *                 console.log("Error message: " +
 *                          errors[0].message);
 *             }
 *         } else {
 *             console.log("Unknown error");
 *         }
 *     }
 * });
 *
 * @public
 * @param {Object}
 *            scope - The scope in which the function is executed. You almost always want to set scope to the keyword this.
 * @param {function}
 *            callback - The callback function to run when the server-side action completes.
 * @param {String}
 *            name - The action state that the callback is associated with.
 * @platform
 * @export
 */
Action.prototype.setCallback = function(scope, callback, name) {};


/**
 * Gets current callback for a give action Type
 *
 * @private
 * @returns {Object} the callback scope and function that was set for this action.
 * @export
 */
 Action.prototype.getCallback = function (type) {};

/**
 * Set an 'all aboard' callback, called just before the action is sent.
 *
 * This can be used in conjunction with 'caboose' to implement a log+flush pattern.
 * Intended to be called as the 'train' leaves the 'station'. Note that setParam should
 * be used to set additional parameters at this point.
 *
 * @public
 * @param {Object}
 *      scope The scope for the callback function.
 * @param {Function}
 *      callback the function to call.
 *
 * @export
 */
Action.prototype.setAllAboardCallback = function(scope, callback) {};

/**
 * Call the 'all aboard' callback.
 *
 * This should only be called internally just before an action is sent to the server.
 *
 * @private
 * @return false if the callback failed.
 */
Action.prototype.callAllAboardCallback = function (context) {};

/**
 * Wrap the current action callbacks to ensure that they get called before a given function.
 *
 * This can be used to add additional functionality to the already existing callbacks, allowing the user to effectively
 * 'append' a function to the current one.
 *
 * @param {Object}
 *            scope the scope in which the new function should be called.
 * @param {Function}
 *            callback the callback to call after the current callback is executed.
 * @private
 */
Action.prototype.wrapCallback = function(scope, callback) {};

/**
 * Deprecated. Note: This method is deprecated and should not be used. Instead, use the <code>enqueueAction</code>
 * method on $A. For example, <code>$A.enqueueAction(action)</code>.
 *
 * The deprecated run method runs client-side actions. Do not use it for running server-side actions.
 *
 * If you must have synchronous execution, you can temporarily use runDeprecated.
 *
 * @deprecated
 * @public
 * @param {Event}
 *            evt The event that calls the Action.
 * @export
 */
Action.prototype.run = function(evt) {};

/**
 * Deprecated. Run an action immediately.
 *
 * This function should only be used for old code that requires inline execution of actions. Note that the code then
 * must know if the action is client side or server side, since server side actions cannot be executed inline.
 *
 * @deprecated
 * @public
 * @param {Event}
 *            evt The event that calls the Action.
 * @export
 */
Action.prototype.runDeprecated = function(evt) {};

/**
 * Gets the current state of the Action. You should check the state of the action
 * in the callback after the server-side action completes.
 *
 * @public
 * @returns {string} The possible action states are:
 *   "NEW": The action was created but is not in progress yet
 *   "RUNNING": The action is in progress
 *   "SUCCESS": The action executed successfully
 *   "FAILURE": Deprecated. ERROR is returned instead. The action failed. This state is only valid for client-side actions.
 *   "ERROR": The server returned an error
 *   "INCOMPLETE": The server didn't return a response. The server might be down or the client might be offline.
 *   "ABORTED": The action was aborted. You can register a callback for this explicitly in setCallback().
 * @platform
 * @export
 */
Action.prototype.getState = function() {};

/**
 * Gets the return value of the Action. A server-side Action can return any object containing serializable JSON data.
 *
 * @public
 * @platform
 * @export
 */
Action.prototype.getReturnValue = function() {};

/**
 * Returns an array of error objects only for server-side actions.
 * Each error object has a message field.
 * In any mode except PROD mode, each object also has a stack field, which is a list
 * describing the execution stack when the error occurred.
 *
 * @public
 * @returns {Object[]} An array of error objects. Each error object has a message field.
 * @platform
 * @export
 */
Action.prototype.getError = function() {};

/**
 * Returns true if the actions should be enqueued in the background, false if it should be run in the foreground.
 *
 * @public
 * @platform
 * @export
 */
Action.prototype.isBackground = function() {};

/**
 * Sets the action to run as a background action. This cannot be unset. Background actions are usually long running and
 * lower priority actions. A background action is useful when you want your app to remain responsive to a user while it
 * executes a low priority, long-running action. A rough guideline is to use a background action if it takes more than
 * five seconds for the response to return from the server.
 *
 * @public
 * @platform
 * @export
 */
Action.prototype.setBackground = function() {};

/**
 * Updates the fields from a response.
 *
 * @param {Object} response The response from the server.
 * @return {Boolean} Returns true if the response differs from the original response
 * @private
 */
Action.prototype.updateFromResponse = function(response) {};

/**
 * Gets a storable response from this action.
 *
 * WARNING: must use after finishAction() which updates <code>this.components</code>.
 *
 * @private
 */
Action.prototype.getStored = function() {};

/**
 * Returns the json representation of the action
 * If the action is publicly cacheable, the ID is stripped out
 * @private
 */
Action.prototype.prepareToSend = function() {};

/**
 * Calls callbacks and fires events upon completion of the action.
 *
 * @param {AuraContext} context the context for pushing and popping the current action.
 * @private
 */
Action.prototype.finishAction = function(context) {};

/**
 *
 * @param e - the exception to handle
 * @param err - additional error
 * @param raise - boolean flag to tell it to throw now
 * @private
 */
Action.prototype.processFinishActionException = function(e, message, err, raise) {};

/**
 * Abort an action if the component is not valid.
 *
 * @param {Boolean} beforeSend Have we sent the action to the server yet?
 * @return {Boolean} true if the action was aborted.
 * @private
 */
Action.prototype.abortIfComponentInvalid = function(beforeSend) {};

/**
 * Mark this action as aborted.
 *
 * @private
 */
Action.prototype.abort = function() {};

/**
 * Set the action as abortable. Abortable actions are not sent to the server if the component is not valid.
 * A component is automatically destroyed and marked invalid by the framework when it is unrendered.
 *
 * Actions not marked abortable are always sent to the server regardless of the validity of the component.
 * For example, a save/modify action should not be set abortable to ensure it's always sent to the server
 * even if the component is deleted.
 *
 * Setting an action as abortable cannot be undone
 *
 * @platform
 * @export
 */
Action.prototype.setAbortable = function() {};

/**
 * Checks if this action is a refresh.
 * @export
 */
Action.prototype.isRefreshAction = function() {};

/**
 * Returns the current state of the abortable flag.
 *
 * @public
 * @returns {Boolean} the abortable flag
 * @export
 */
Action.prototype.isAbortable = function() {};

/**
 * Marks the Action as storable. For server-side Actions only.
 * Mark an action as storable to have its response stored in the client-side cache by the framework. Caching can be useful
 * if you want your app to be functional for devices that temporarily don’t have a network connection.
 *
 * @public
 * @param {Object}
 *            config - Optional. A set of key/value pairs that specify the storage options to set. You can set the
 *            following option:
 *            <code>ignoreExisting</code>: Set to <code>true</code> to refresh the stored item with a newly retrieved value,
 *              regardless of whether the item has expired or not. The default value is <code>false</code>.
 * @platform
 * @export
 */
Action.prototype.setStorable = function(config) {};


/**
 * Returns true if the function is storable, or false otherwise. For server-side Actions only.
 *
 * @public
 * @returns {Boolean}
 * @export
 */
Action.prototype.isStorable = function() {};

/**
 * Sets this action as a 'caboose'.
 *
 * This is only relevant for server side actions.
 * This action will not be sent to the server until there is some other action
 * that would cause a server round-trip or after 60s since last send.
 *
 * @public
 * @export
 */
Action.prototype.setCaboose = function() {};


/**
 * Returns true if the function should not create an XHR request.
 *
 * @public
 * @returns {boolean}
 * @export
 */
Action.prototype.isCaboose = function() {};

/**
 * @private
 */
Action.prototype._isStorable = function() {};

/**
 * Gets the storage key in name-value pairs.
 *
 * @private
 * @export
 */
Action.prototype.getStorageKey = function() {};

/**
 * Returns true if a given function is from the current storage, or false otherwise.
 *
 * @public
 * @returns {Boolean}
 * @export
 */
Action.prototype.isFromStorage = function() {};

/**
 * Chains a function to run after the current Action. For server-side Actions only.
 *
 * @public
 * @export
 */
Action.prototype.setChained = function() {};

/**
 * Returns true if a given function is chained, or false otherwise. For server-side Actions only.
 *
 * @returns {Boolean}
 * @private
 */
Action.prototype.isChained = function() {};

/**
 * Returns the number of times this action has been retried
 * @returns {number}
 */
Action.prototype.getRetryCount = function() {};

/**
 * Increment the retry counter on this action
 */
Action.prototype.incrementRetryCount = function() {};

/**
 * Returns the key/value pairs of the Action id, descriptor, and parameters in JSON format.
 *
 * @public
 * @export
 */
Action.prototype.toJSON = function() {};

/**
 * Mark an action as having an error.
 *
 * @param e the exception with which we want to mark the action.
 */
Action.prototype.markException = function(e) {};

/**
 * Mark the current action as having an error and finish the Action.
 *
 * @param context The current context.
 * @param e The error with which we want to mark the action.
 * @private
 */
Action.prototype.markError = function(context, e) {};

/**
 * Mark the current action as incomplete.
 *
 * @private
 */
Action.prototype.incomplete = function(context) {};

/**
 * Internal routine to do the basic copy to a new refresh action.
 */
Action.prototype.copyToRefresh = function() {};

/**
 * Gets a new action instance that can be used to refresh this storable action.
 * @return {Action} a new instance of this action.
 * @private
 */
Action.prototype.getRefreshAction = function(originalResponse) {};

/**
 * Returns an action that retries this action from storage with the server or null if the action wasn't from storage
 *
 * @private
 * @returns {Action}
 */
Action.prototype.getRetryFromStorageAction = function() {};

/**
 * Gets the Action storage.
 *
 * @returns {Storage}
 * @deprecated
 * @export
 */
Action.prototype.getStorage = function() {};

/**
 * Uses the event object in the action's response and fires the event.
 *
 * @private
 */
Action.prototype.parseAndFireEvent = function(evtObj) {};

/**
 * Fire off a refresh event if there is a valid component listener.
 *
 * @private
 */
Action.prototype.fireRefreshEvent = function(event, responseUpdated) {};

/**
 * Returns true if public caching is enabled and the current action is publicly cacheable (based on the definition).
 *
 * For server-side Actions only.
 *
 * @returns {Boolean}
 * @private
 */
Action.prototype.isPubliclyCacheable = function() {};

Aura.Controller.Action = Action;
