/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description API to access action storage.
 *
 * This class is *the* API to access storable actions and action storage. All access to action storage
 * must use this API to ensure a consistent and reliable cache.
 *
 * By default a `filter` of storable action keys accessible to the current browser tab is built.
 * This is required because actions depend on defs, defs depend on labels, and the
 * label GVP is stored in the actions store. Labels and defs are loaded at framework boot so
 * the same must be done for actions. Failing to do this can result in storable action cache hits
 * that reference defs or labels the current tab does not have access to, which typically causes
 * a failure in the app. This commonly occurs in multi-tab browser scenarios because persistent
 * storage is shared across tabs but not synchronized into each tab's memory.
 *
 * Building the filter incurs a runtime cost at boot so is done only when persistent actions storage
 * is used. It may also selectively be disabled by the application (eg if the environment guarantees
 * a single tab).
 *
 * @constructor
 * @protected
 */
function ActionStorage() {}

ActionStorage.prototype.STORAGE_NAME = "actions";

ActionStorage.prototype.URI_DEFS_ENABLED_KEY = "_uri_defs_enabled";

/**
 * Enables or disables the persisted actions filter, required for multi-tab
 * environments that use storable actions.
 * @param {Boolean} enable - true to enable the filter, false to disable the filter.
 */
ActionStorage.prototype.enableActionsFilter = function(enable) {};

/**
 * Initialize the persisted actions filter.
 * @return {Boolean} true if actions filter is set up
 */
ActionStorage.prototype.setupActionsFilter = function() {};

/**
 * Populates the persisted actions filter if applicable.
 */
ActionStorage.prototype.populateActionsFilter = function() {};

/**
 * Reset the persisted actions filter.
 */
ActionStorage.prototype.clearActionsFilter = function() {};

ActionStorage.prototype.isActionsFilterEnabled = function() {};

/**
 * Check if actions filter has been set up.
 */
ActionStorage.prototype.isActionsFilterInitialized = function() {};

/**
 * Check if an action is absent in Action Storage (from in-memory cache).
 * This function only guarantees that an action storage key is absent in storage.
 * It doesn't guarantee that an action can be retrieved from storage if it returns false.
 *
 * @param {String} key - action storage key
 * @return {Boolean} true if the action storage key is guaranteed to not be accessible from storage; false if accessible or unknown.
 */
ActionStorage.prototype.isKeyAbsentFromCache = function(key) {};

/**
 * Returns the underlying AuraStorage
 * @private
 */
ActionStorage.prototype.getStorage = function() {};

/**
 * Check if the underlying AuraStorage is persistent
 * @return {Boolean} true if action storage is persistent
 */
ActionStorage.prototype.isStoragePersistent = function() {};

/**
 * Check if the underlying AuraStorage is initialized
 * @return {Boolean} true if action storage is enabled
 */
ActionStorage.prototype.isStorageEnabled = function() {};

/**
 * Asynchronously stores multiple actions to action storage. All or none of the values are stored.
 * Stored action keys are added into action filter if actions filter is enabled.
 *
 * @param {Object} values The actionKey-value pairs to store.
 * @returns {Promise} A promise that resolves when all of the actionKey-value pairs are stored.
 */
ActionStorage.prototype.setAll = function(values) {};

/**
 * Asynchronously stores the action in storage using the specified key.
 * Stored action key is added into action filter if actions filter is enabled.
 *
 * @param {String} actionKey - The action key to store.
 * @param {*} value The value of the action to store.
 * @returns {Promise} A promise that resolves when are stored.
 */
ActionStorage.prototype.set = function(actionKey, value) {};

/**
 * Asynchronously gets multiple items from storage.
 * Retrieved action keys are added into action filter if actions filter is enabled.
 *
 * @param {String[]} [actionKeys] The set of action keys to retrieve. Empty array or falsey to retrieve all items.
 * @returns {Promise} A promise that resolves to an object that contains key-value pairs. {key: storedItem}
 */
ActionStorage.prototype.getAll = function(actionKeys) {};

/**
 * Asynchronously gets a sigle action from storage corresponding to the action key.
 * Retrieved action keys is added into action filter if actions filter is enabled.
 *
 * @param {String} actionKey The key of the action to retrieve.
 * @returns {Promise} A promise that resolves to the stored item or undefined if the key is not found.
 */
ActionStorage.prototype.get = function(actionKey) {};

/**
 * Asynchronously removes multiple actions from storage. All or none of the values are removed.
 * Removed actions keys get deleted from action filter if action filter is enabled.
 *
 * @param {String[]} actionKeys The keys of the actions to remove.
 * @param {Boolean=} doNotFireModified Whether to fire the modified event on item removal.
 * @returns {Promise} A promise that resolves when all of the values are removed.
 */
ActionStorage.prototype.removeAll = function(actionKeys, doNotFireModified) {};

/**
 * Asynchronously removes a single action from storage corresponding to the specified key.
 * Removed action key gets deleted from action filter if action filter is enabled.
 *
 * @param {String} actionKey The key of the action to remove.
 * @param {Boolean} doNotFireModified Whether to fire the modified event on item removal.
 * @returns {Promise} A promise that will remove the value from storage.
 */
ActionStorage.prototype.remove = function(actionKey, doNotFireModified) {};

/**
 * Clear the action storge. The action filter get reset if action filter is enable.
 *
 * @returns {Promise} A promise that will clear storage.
 */
ActionStorage.prototype.clear = function() {};

Aura.Controller.ActionStorage = ActionStorage;
