/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description Global Value Provider. Holds global values: $Label, $Browser, $Locale, etc
 *
 * The interface required of a global value provider is:
 * <ul>
 *   <li>merge: merge a set of values from the server (if values come from the server)
 *   <li>get: get a single value from the GVP
 *   <li>getStorableValues[optional] get a storable version of the GVP values
 *   <li>getValues: get a set of values that can be exposed.
 *   <li>set[optional]: set a value on the provider
 *   <li>isStorable[optional]: should values be saved to storage
 * </ul>
 *
 * @param {Object} gvp an optional serialized GVP to load.
 * @param {Function} initCallback an optional callback invoked after the GVP has finished its
 *  asynchronous initialization.
 * @constructor
 * @export
 */
function GlobalValueProviders (gvp, initCallback) {
    this.valueProviders = {
        "$Browser" : new Aura.Provider.ObjectValueProvider(gvp["$Browser"]),
        "$Label": new Aura.Provider.LabelValueProvider(gvp["$Label"]),
        "$Locale": new Aura.Provider.ObjectValueProvider(gvp["$Locale"]),
        "$Global": new Aura.Provider.ContextValueProvider(gvp["$Global"])
    };

    for(var type in gvp){
        if (["$Browser", "$Label", "$Locale", "$Global"].indexOf(type) >= 0) {
            continue;
        }

        $A.assert(this.valueProviders[type]==null,"$A.globalValueProviders.ctor(): '"+type+"' has already been registered.");
        // work around the obfuscation logic to allow external GVPs
        var valueProvider = gvp[type];
        valueProvider.getValues = valueProvider.getValues || valueProvider["getValues"];
        valueProvider.get       = valueProvider.get       || valueProvider["get"];
        valueProvider.merge     = valueProvider.merge     || valueProvider["merge"];
        this.valueProviders[type] = valueProvider;
    }

    var that = this;
    this.loadFromStorage(function() {});
}

/**
 * Persistent storage key for GVPs.
 */
GlobalValueProviders.prototype.STORAGE_KEY = "globalValueProviders";


/**
 * Key to use of the MutexLocker to guarantee atomic execution across tabs.
 */
GlobalValueProviders.prototype.MUTEX_KEY = "GlobalValueProviders";

/**
 * Function to release the mutex, set while the mutex is held.
 */
GlobalValueProviders.prototype.mutexUnlock = undefined;

/**
 * Set to true while GVP persistence is async acquiring the lock, enabling
 * concurrent GVP updates to skip their own persistence calls.
 */
GlobalValueProviders.prototype.persistenceQueued = false;

/**
 * True if GVPs were loaded from persistent storage. */
GlobalValueProviders.prototype.LOADED_FROM_PERSISTENT_STORAGE = false;

/**
 * Cookie used to track when persistent storage is known to be missing GVP values.
 */
GlobalValueProviders.prototype.ABSENT_GVP_VALUES_COOKIE = "auraGvpValuesAbsence";


/**
 * Merges new GVPs with existing and saves to storage
 *
 * @param {Array} gvps The new GVPs to merge. Provided as an array of objects,
 *  where each object has two keys: "type" and "values".
 * @param {Boolean} doNotPersist
 * @protected
 */
GlobalValueProviders.prototype.merge = function(gvps, doNotPersist) {};


/**
 * Wrapper to get storage.
 *
 * @return {Object} storage - undefined if no storage exists
 * @private
 */
GlobalValueProviders.prototype.getStorage = function () {};

/**
 * load GVPs from storage if available
 * @private
 */
GlobalValueProviders.prototype.loadFromStorage = function(callback) {};

/**
 * Loads GVP config when from context
 *
 * @param {Object} gvp Global Value Providers
 * @private
 */
GlobalValueProviders.prototype.load = function(gvp) {};


/**
 * Adds a new global value provider.
 * @param type The key to identify the valueProvider.
 * @param valueProvider The valueProvider to add.
 * @private
 */
GlobalValueProviders.prototype.addValueProvider = function(type, valueProvider) {};

/**
 * Returns value provider or empty ObjectValueProvider
 *
 * @param {String} type the key to identify the valueProvider
 * @return {Object} ValueProvider
 * @private
 */
GlobalValueProviders.prototype.getValueProvider = function(type) {};

/**
 * Calls getValue for Value Object. Unwraps and calls callback if provided.
 *
 * @param {String} expression
 * @param {Component} component
 * @return {String} The value of expression
 * @export
 */
GlobalValueProviders.prototype.get = function(expression, callback) {};


GlobalValueProviders.prototype.getAbsentGvpValuesCookie = function() {};

GlobalValueProviders.prototype.setAbsentGvpValuesCookie = function() {};

GlobalValueProviders.prototype.clearAbsentGvpValuesCookie = function() {};

Aura.Provider.GlobalValueProviders = GlobalValueProviders;
