/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description The storage service implementation.
 * @constructor
 * @param {Object} config The configuration describing the characteristics of the storage to be created.
 * @export
 */
var AuraStorage = function AuraStorage(config) {};

/** Log levels */
AuraStorage.prototype.LOG_LEVEL = {
    INFO:    { id: 0, fn: "log" },
    WARNING: { id: 1, fn: "warning" }
};

/**
 * Anti-obfuscate to support adapters provided by non-framework.
 */
AuraStorage.prototype.adapterAntiObfuscation = function(adapter) {};

/**
 * Returns the name of the storage adapter. For example, "indexeddb" or "memory".
 * @returns {String} The storage adapter's name.
 * @export
 */
AuraStorage.prototype.getName = function() {};

/**
 * Gets the current storage size in KB.
 * @returns {Promise} A promise that resolves to the current storage size in KB.
 * @export
 */
AuraStorage.prototype.getSize = function() {};

AuraStorage.prototype.getSizeInternal = function(resolve, reject) {};


/**
 * Returns the maximum storage size in KB.
 * @returns {number} The maximum storage size in KB.
 * @export
 */
AuraStorage.prototype.getMaxSize = function() {};

/**
 * Enqueues a function to execute when the adapter is ready.
 * @param {Function} execute the function to execute.
 * @returns {Promise} a promise that resolves when the function is executed.
 * @private
 */
AuraStorage.prototype.enqueue = function(execute) {};

/**
 * Callback function provided to adapters to indicate initialization is complete.
 * @param {Boolean} readyState true if the adapter successfully completed initialization, false if initialization failed.
 * @param {Error} error details if initialization failed, undefined otherwise.
 * @private
 */
AuraStorage.prototype.adapterInitialize = function(readyState, error) {};


/**
 * Runs the pending queue of requests.
 * @private
 */
AuraStorage.prototype.executeQueue = function() {};


/**
 * Gets the error message when the adapter fails to initialize.
 * @private
 */
AuraStorage.prototype.getInitializationError = function() {};


/**
 * Returns a promise that clears the storage.
 * @returns {Promise} A promise that will clear storage.
 * @export
 */
AuraStorage.prototype.clear = function() {};

AuraStorage.prototype.clearInternal = function(resolve, reject) {};


/**
 * Asynchronously gets an item from storage corresponding to the specified key.
 * @param {String} key The key of the item to retrieve.
 * @param {Boolean=} includeExpired True to return expired items, false to not return expired items.
 * @returns {Promise} A promise that resolves to the stored item or undefined if the key is not found.
 * @export
 */
AuraStorage.prototype.get = function(key, includeExpired) {};

/**
 * Gets the count of in-flight operations. Note that it's possible the underlying
 * adapter is performing operations that are not triggered from this API.
 * @returns {Number} Number of operations currently waiting on being resolved.
 * @export
 */
AuraStorage.prototype.inFlightOperations = function() {};


/**
 * Asynchronously gets multiple items from storage.
 * @param {String[]} [keys] The set of keys to retrieve. Empty array or falsey to retrieve all items.
 * @param {Boolean} [includeExpired] True to return expired items, falsey to not return expired items.
 * @returns {Promise} A promise that resolves to an object that contains key-value pairs. {key: storedItem}
 * @export
 */
AuraStorage.prototype.getAll = function(keys, includeExpired) {};

AuraStorage.prototype.getAllInternal = function(keys, includeExpired, resolve, reject) {};

/**
 * Builds the payload to store in the adapter.
 * @param {String} key The key of the item to store.
 * @param {*} value The value of the item to store.
 * @param {Number} now The current time (milliseconds).
 * @returns {Array} A key-value-size tuple to pass to the adapter's setItems.
 * @private
 */
AuraStorage.prototype.buildPayload = function(key, value, now) {};

/**
 * Asynchronously stores the value in storage using the specified key.
 * @param {String} key The key of the item to store.
 * @param {*} value The value of the item to store.
 * @returns {Promise} A promise that resolves when are stored.
 * @export
 */
AuraStorage.prototype.set = function(key, value) {};

/**
 * Asynchronously stores multiple values in storage. All or none of the values are stored.
 * @param {Object} values The key-values to store. Eg <code>{key1: value1, key2: value2}</code>.
 * @returns {Promise} A promise that resolves when all of the key-values are stored.
 * @export
 */
AuraStorage.prototype.setAll = function(values) {};

AuraStorage.prototype.setAllInternal = function(values, resolve, reject) {};


/**
 * Asynchronously removes the value from storage corresponding to the specified key.
 * @param {String} key The key of the value to remove.
 * @param {Boolean} doNotFireModified Whether to fire the modified event on item removal.
 * @returns {Promise} A promise that will remove the value from storage.
 * @export
 */
AuraStorage.prototype.remove = function(key, doNotFireModified) {};

/**
 * Asynchronously removes multiple values from storage. All or none of the values are removed.
 * @param {String[]} keys The keys of the values to remove.
 * @param {Boolean=} doNotFireModified Whether to fire the modified event on item removal.
 * @returns {Promise} A promise that resolves when all of the values are removed.
 * @export
 */
AuraStorage.prototype.removeAll = function(keys, doNotFireModified) {};

AuraStorage.prototype.removeAllInternal = function(keys, doNotFireModified, resolve, reject) {};


/**
 * Asynchronously sweeps the store to remove expired items.
 * @param {Boolean} ignoreInterval True to ignore minimum sweep intervals.
 * @return {Promise} A promise that resolves when sweeping is completed.
 * @private
 */
AuraStorage.prototype.sweep = function(ignoreInterval) {};


/**
 * Suspends sweeping.
 *
 * Expired storage entries are proactively removed by sweeping. Sweeping is often suspended
 * when the connection goes offline so expired items remain accessible.
 * @export
 */
AuraStorage.prototype.suspendSweeping = function() {};


/**
 * Resumes sweeping to remove expired storage entries.
 * @export
 */
AuraStorage.prototype.resumeSweeping = function() {};


/**
 * Log a message.
 * @param {LOG_LEVEL} level The log level.
 * @param {String} msg The msg to log.
 * @param {Object} [obj] Optional object to log.
 * @private
 */
AuraStorage.prototype.log = function(level, msg, obj) {};


/**
 * Logs an error to the server.
 * @param {Object} payload The error payload object.
 * @param {String} payload.operation The operation which errored (eg get, set).
 * @param {Error=} payload.error Optional error object.
 * @private
 */
AuraStorage.prototype.logError = function(payload) {};


/**
 * Logs runtime statistics to the server.
 * @private
 */
AuraStorage.prototype.logStats = function() {};


/**
 * Whether the storage implementation is persistent.
 * @returns {boolean} True if persistent.
 * @export
 */
AuraStorage.prototype.isPersistent = function() {};


/**
 * Whether the storage implementation is secure.
 * @returns {boolean} True if secure.
 * @export
 */
AuraStorage.prototype.isSecure = function() {};


/**
 * Returns the storage version.
 * @returns {String} The storage version.
 * @export
 */
AuraStorage.prototype.getVersion  = function() {};


/**
 * Returns the expiration in seconds.
 * @returns {Number} The expiration in seconds.
 * @export
 */
AuraStorage.prototype.getExpiration = function() {};


/**
 * Returns the auto-refresh interval in seconds.
 * @returns {Number} The auto-refresh interval in seconds.
 */
AuraStorage.prototype.getDefaultAutoRefreshInterval = function() {};


/**
 * Asynchronously deletes this storage.
 * @private
 */
AuraStorage.prototype.deleteStorage = function() {};

AuraStorage.prototype.deleteStorageInternal = function(resolve, reject) {};

/**
 * Generates the key prefix for storage.
 * @param {String} isolationKey The isolation key.
 * @param {String} version The version.
 * @private
 */
AuraStorage.prototype.generateKeyPrefix = function(isolationKey, version) {};

/**
 * Fires an auraStorage:modified event for this storage.
 * @private
 */
AuraStorage.prototype.fireModified = function() {};


/**
 * Storage key delimiter, separating isolation and version key from
 * the user-provided key.
 * @private
 */
AuraStorage.KEY_DELIMITER = ":";

/**
 * Sweep intervals (milliseconds).
 */
AuraStorage["SWEEP_INTERVAL"] = {
        "MIN": 60000, // 1 min
        "MAX": 300000 // 5 min
};


Aura.Storage.AuraStorage = AuraStorage;
