/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description The Aura Storage Service, accessible using <code>$A.storageService</code>.
 * @constructor
 * @export
 */
function AuraStorageService(){}

/**
 * Returns an existing storage using the specified name.
 * For example, <code>$A.storageService.getStorage("MyStorage").getSize()</code> returns the size of MyStorage.
 * <p>See Also: <a href="#reference?topic=api:AuraStorage">AuraStorage</a></p>
 * @param {String} name The name of the requested storage.
 * @memberOf AuraStorageService
 * @returns {AuraStorage} Returns an AuraStorage object corresponding to an existing storage.
 * @export
 */
AuraStorageService.prototype.getStorage = function(name) {};


/**
 * Returns all existing storages.
 * <p>See Also: <a href="#reference?topic=api:AuraStorage">AuraStorage</a></p>
 * @memberOf AuraStorageService
 * @returns {Object} Returns a map of storage names to AuraStorage objects.
 * @export
 */
AuraStorageService.prototype.getStorages = function() {};


/**
 * Initializes and returns a new storage.
 *
 * @example
 * var storage = $A.storageService.initStorage({
 *     "name":                "MyStorage",
 *     "persistent":          true,
 *     "secure":              true,
 *     "maxSize":             524288, // (bytes) (512 * 1024)
 *     "expiration":          600,    // (seconds)
 *     "autoRefreshInterval": 600,    // (seconds)
 *     "debugLogging":        true,
 *     "clearOnInit":         false
 * });
 *
 * @param {Object} config The configuration for the new storage. The name property is required. All other properties are optional.
 * @param {String} config.name Required. The unique name of the storage to be initialized.
 * @param {Boolean=} config.persistent Set to true if storage can be persistent. Default is false.
 * @param {Boolean=} config.secure Set to true if storage must be secure. Default is false.
 * @param {Number=} config.maxSize Specifies the maximum storage size (bytes). Default is 1000KB.
 * @param {Number=} config.expiration Specifies the time (seconds) after which an item expires. When an item is requested that has gone past the expiration time, it will not be used. Default is 10s.
 * @param {Boolean=} config.debugLogging Set to true to enable debug logging in the JavaScript console. Default is false.
 * @param {Boolean=} config.clearOnInit Set to true to clear storage when storage is initialized. Default is true.
 * @param {String=} config.version The version of the storage. Only items matching the version are returned. This is useful to avoid retrieving items for an older version of your application. Default is "".
 * @param {Number=} config.autoRefreshInterval Specifies the interval (seconds) after which an item is to be refreshed. The caller must track and perform the refresh itself. Default is 30s.
 * @memberOf AuraStorageService
 * @returns {AuraStorage} Returns an AuraStorage object for the new storage.
 * @export
 */
AuraStorageService.prototype.initStorage = function(config) {};


/**
 * Registers a new Aura Storage Service adapter.
 *
 * @param {Object} config Adapter configuration object.
 * @param {String} config.name Name of the adapter. Must be unique among adapters.
 * @param {AuraStorageAdapter} config.adapterClass Constructor function of the adapter.
 * @param {Boolean} config.persistent True if the adapter is persistent.
 * @param {Boolean} config.secure True if the adapter is secure.
 * @memberOf AuraStorageService
 * @export
 */
AuraStorageService.prototype.registerAdapter = function(config) {};

/**
 * Whether an adapter is registered
 *
 * @param {String} name The adapter's name.
 * @returns {Boolean} Whether adapter is registered.
 * @memberOf AuraStorageService
 * @export
 */
AuraStorageService.prototype.isRegisteredAdapter = function(name) {};

/**
 * Returns an adapter's configuration.
 *
 * @param {String} adapter Name of the adapter.
 * @memberOf AuraStorageService
 * @export
 */
AuraStorageService.prototype.getAdapterConfig = function(adapter) {};

/**
 * Selects an adapter based on the given configuration. Some configuration is a hard requirement;
 * other is best effort.
 * @param {Boolean} persistent Set to true if the adapter should be persistent.
 * @param {Boolean} secure Set to true if the adapter must be secure.
 * @return {String} The name of the selected adapter.
 * @memberOf AuraStorageService
 //#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
	@export
 //#end
 */
AuraStorageService.prototype.selectAdapter = function(persistent, secure) {};

/**
 * Deletes a storage.
 * @param {String} name name of storage to delete.
 * @return {Promise} a promise that resolves when the specified storage is deleted.
 * @export
 */
AuraStorageService.prototype.deleteStorage = function(name) {};

/**
 * Sets the default version for all storages.
 * @param {String} version default version for storages.
 * @export
 */
AuraStorageService.prototype.setVersion = function(version) {};

/**
 * Gets the default version for all storages.
 * @return {String} the default version for storages.
 * @export
 */
AuraStorageService.prototype.getVersion = function() {};

/**
 * Sets a key from which isolation in the storage system is enforced.
 *
 * This mechanism is typically used to isolate multiple users' data by setting
 * the isolation key to the user id.
 *
 * It should only be called once during the application life cycle, since it
 * will be deleted after invocation in production mode.
 *
 *
 * @param {String} isolationKey the key defining isolation.
 * @export
 */
AuraStorageService.prototype.setIsolation = function(isolationKey) {};

/**
 * Sets a name for the table within the storage system.
 *
 * This mechanism is typically used to distinguish storage tables from one another
 * when no context has been set.
 *
 * It should only be called once during the application life cycle, since it
 * will be deleted after invocation in production mode.
 *
 * @param {String} partitionName the name to define a table.
 * @export
 */
AuraStorageService.prototype.setPartition = function(partitionName) {};

Aura.Services.AuraStorageService = AuraStorageService;
