/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @description The crypto adapter for Aura Storage Service.
 *
 * This adapter provides AES-CBC encryption, using the browser Web Cryptography API
 * (https://dvcs.w3.org/hg/webcrypto-api/raw-file/tip/spec/Overview.html#dfn-GlobalCrypto)
 * with a server-provided key, persisting into the IndexedDB adapter.
 *
 * Unlike other storage adapters this adapter does not automatically register itself. Use
 * &lt;auraStorage:crypto/&gt; in the &lt;auraPreInitBlock/&gt; section of the app's template
 * to register this adapter. Doing so guarantees that CryptoAdapter.setKey() will be invoked
 * with a server-provided cryptographic key.
 *
 * Please note:
 * 1. If the runtime environment doesn't support the Web Cryptography API the adapter is not
 *    registered.
 * 2. A cryptographic key must be provided by &lt;auraStorage:crypto/&gt;. Until a key is
 *    provided the adapter remains in an initialization state.
 * 3. If an invalid cryptographic key is provided, initialization completes in the error
 *    state, causing AuraStorage to fallback to the memory adapter.
 *
 * @constructor
 */
function CryptoAdapter(config) {}


/** Name of adapter. */
CryptoAdapter.NAME = "crypto";

/** Log levels */
CryptoAdapter.LOG_LEVEL = {
    INFO:    { id: 0, fn: "log" },
    WARNING: { id: 1, fn: "warning" }
};

/** Encryption algorithm. */
CryptoAdapter.ALGO = "AES-CBC";

/** Initialization vector length (bytes). */
CryptoAdapter.IV_LENGTH = 16;

/** A sentinel value to verify the key against pre-existing data. */
CryptoAdapter.SENTINEL = "cryptoadapter";

/** Initializes he Web Cryptography API */
CryptoAdapter.initializeEngine = function() {};

CryptoAdapter.initializeEngine();


/** Promise that resolves with the per-application encryption key. */
CryptoAdapter.key = new Promise(function(resolve, reject) {});


/**
 * Sets the per-application encryption key.
 * @param {ArrayBuffer} rawKey The raw bytes of the encryption key
 */
CryptoAdapter["setKey"] = function(rawKey) {};


/**
 * Registers crypto adapter.
 */
CryptoAdapter["register"] = function() {};


/**
 * Returns the name of the adapter.
 * @returns {String} name of adapter
 */
CryptoAdapter.prototype.getName = function() {};

/**
 * Starts the initialization process.
 * @return {Promise} a promise that resolves when initialization has completed, or rejects if initialization has failed.
 */
CryptoAdapter.prototype.initialize = function() {};


/**
 * Initializes the adapter by waiting for the app-wide crypto key to be set,
 * then validates the key works for items already in persistent storage. Several
 * error scenarios to detect:
 * - invalid key provided -> fallback to memory storage
 * - valid key is provided but can't fetch what's in storage -> clear then use crypto with new key
 * - valid key is provided but doesn't match what's in storage -> clear then use crypto with new key
 * @private
 */
CryptoAdapter.prototype.initializeInternal = function() {};


/**
 * Returns adapter size.
 * @returns {Promise} a promise that resolves with the size in bytes
 */
CryptoAdapter.prototype.getSize = function() {};


/**
 * Retrieves items from storage.
 * @param {String[]} [keys] The set of keys to retrieve. Undefined to retrieve all items.
 * @param {Boolean} [includeExpired] True to return expired items, false to not return expired items.
 * @returns {Promise} A promise that resolves with an object that contains key-value pairs.
 */
CryptoAdapter.prototype.getItems = function(keys, includeInternalKeys) {};


/**
 * Decrypts a stored cached entry.
 * @param {String} key The key of the value to decrypt.
 * @param {Object} value The cache entry to decrypt.
 * @returns {Promise} Promise that resolves with the decrypted item.
 * @private
 */
CryptoAdapter.prototype.decrypt = function(key, value) {};


/**
 * Converts an object to an ArrayBuffer.
 * @param {Object} o The object to convert.
 * @private
 */
CryptoAdapter.prototype.objectToArrayBuffer = function(o) {};

/**
 * Alias for objectToArrayBuffer, used by AuraStorage to avoid multiple calls to json.encode
 * @param {Object} o The object to convert
 */
CryptoAdapter.prototype.encodeValue = function(o) {};

/**
 * Converts an ArrayBuffer to object.
 * @param {ArrayBuffer} ab The ArrayBuffer to convert.
 * @private
 */
CryptoAdapter.prototype.arrayBufferToObject = function(ab) {};


/**
 * Stores entry used to determine whether encryption key provided can decrypt the store.
 * @returns {Promise} Promise that resolves when the item is stored.
 */
CryptoAdapter.prototype.setSentinelItem = function() {};


/**
 * Encrypts a tuple.
 * @param {Array} tuple An array of key-value-size.
 * @returns {Promise} Promise that resolves to a tuple of key-encrypted value-size.
 */
CryptoAdapter.prototype.encryptToTuple = function(tuple) {};


/**
 * Stores items in storage.
 * @param {Array} tuples An array of key-value-size pairs. Eg <code>[ [key1, value1, size1], [key2, value2, size2] ]</code>.
 * @returns {Promise} A promise that resolves when the items are stored.
 */
CryptoAdapter.prototype.setItems = function(tuples) {};


/**
 * Removes items from storage.
 * @param {String[]} keys The keys of the items to remove.
 * @returns {Promise} A promise that resolves when all items are removed.
 */
CryptoAdapter.prototype.removeItems = function(keys) {};


/**
 * Clears storage.
 * @returns {Promise} a promise that resolves when the store is cleared
 */
CryptoAdapter.prototype.clear = function() {};


/**
 * Sweeps over the store to evict expired items.
 * @returns {Promise} A promise that resolves when the sweep is complete.
 */
CryptoAdapter.prototype.sweep = function() {};


/**
 * Deletes this storage.
 * @returns {Promise} A promise that resolves when storage is deleted
 */
CryptoAdapter.prototype.deleteStorage = function() {};


/**
 * Suspends eviction.
 */
CryptoAdapter.prototype.suspendSweeping = function() {};


/**
 * Resumes eviction.
 */
CryptoAdapter.prototype.resumeSweeping = function() {};


/**
 * @returns {Boolean} whether the adapter is secure.
 */
CryptoAdapter.prototype.isSecure = function() {};


/**
 * @returns {Boolean} whether the adapter is persistent.
 */
CryptoAdapter.prototype.isPersistent = function() {};


/**
 * Logs a message.
 * @param {CryptoAdapter.LOG_LEVEL} level Log line level
 * @param {String} msg The log message
 * @param {Object} [obj] Optional log payload
 * @private
 */
CryptoAdapter.prototype.log = function (level, msg, obj) {};


/**
 * Anti-obfuscate to support adapters provided by non-framework. Eg tests.
 */
CryptoAdapter.prototype.adapterAntiObfuscation = function(adapter) {};


Aura.Storage.CryptoAdapter = CryptoAdapter;

// export crypto adapter as $A.storageService.CryptoAdapter exposing effectively
// only the non-mangled functions. not using @export because it exports the
// constructor function which is not desired.
AuraStorageService.prototype["CryptoAdapter"] = CryptoAdapter;
