/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description The IndexedDB adapter for Aura Storage Service.
 *
 * Implementation notes:
 *
 * Each store name gets its own DB. Each app gets its own ObjectStore (aka table). If the same
 * store name is used across apps (eg actions ) then a single DB contains multiple ObjectStores.
 * TODO - it'd be better scoped to have one DB per app with all its tables within.
 *
 * Sizing is approximate and updates to sizes are very approximate. We recalculate when our error bars get
 * too big or after a certain number of updates. This is locked to happen no more than once every 15 minutes
 * if our size is not over the limit.
 * TODO - revamp size calculations to be more understandable while still supporting multiple browser tabs
 *
 * Entire table scans are performed for:
 * (1) getAll(undefined), since we have to.
 * (2) size or error bar over the limit.
 * (3) getSize, with an old size guess.
 * (4) sweep, since we're already scanning the table.
 *
 * @constructor
 */
var IndexedDBAdapter = function IndexedDBAdapter(config) {};

/** Name of the adapter */
IndexedDBAdapter.NAME = "indexeddb";

/** Log levels */
IndexedDBAdapter.LOG_LEVEL = {
    INFO:    { id: 0, fn: "log" },
    WARNING: { id: 1, fn: "warning" }
};

/** Max time for initialize to complete */
IndexedDBAdapter.INITIALIZE_TIMEOUT = 30 * 1000;

/** Threshold of time elapsed getting items from object store before a metric is logged */
IndexedDBAdapter.OBJECTSTORE__TRANSACTION_THRESHOLD = 500;

/**
 * Returns the name of the adapter.
 * @returns {String} name of adapter
 */
IndexedDBAdapter.prototype.getName = function() {};


/**
 * Starts the initialization process.
 * @return {Promise} a promise that resolves when initialization has completed, or rejects if initialization has failed.
 */
IndexedDBAdapter.prototype.initialize = function() {};


/**
 * Initializes the adapter by setting up the DB and ObjectStore.
 * @param {Number=} version Optional version value for IndexedDB.open(). Set
 *  to a new value if schema needs to change (eg new table, new index).
 * @param {Object} [transactionTimer] timer measuring open / initialization of db.
 * @private
 */
IndexedDBAdapter.prototype.initializeInternal = function(version, transactionTimer) {};


/**
 * Returns adapter size.
 * @returns {Promise} a promise that resolves with the size in bytes
 */
IndexedDBAdapter.prototype.getSize = function() {};

/**
 * Retrieves items from storage.
 * @param {String[]} [keys] The set of keys to retrieve. Undefined to retrieve all items.
 * @param {Boolean} [includeExpired] True to return expired items, false to not return expired items.
 * @returns {Promise} A promise that resolves with an object that contains key-value pairs.
 */
IndexedDBAdapter.prototype.getItems = function(keys /*, includeExpired*/) {};

/**
 * Suspends eviction.
 */
IndexedDBAdapter.prototype.suspendSweeping = function() {};

/**
 * Resumes eviction.
 */
IndexedDBAdapter.prototype.resumeSweeping = function() {};

/**
 * @returns {Boolean} Whether the adapter is secure.
 */
IndexedDBAdapter.prototype.isSecure = function() {};

/**
 * @returns {Boolean} Whether the adapter is persistent.
 */
IndexedDBAdapter.prototype.isPersistent = function() {};

/**
 * Sweeps over the store to evict expired items.
 * @returns {Promise} A promise that resolves when the sweep is complete.
 */
IndexedDBAdapter.prototype.sweep = function() {};

/**
 * Initializes the structure with a new DB.
 * @param {Event} event IndexedDB event.
 * @param {Object} transactionTimer measuring time taken to open/initialize db.
 * @private
 */
IndexedDBAdapter.prototype.setupDB = function(event, transactionTimer) {};

/**
 * Creates tables in the DB.
 * @param {Event} event IndexedDB event
 * @private
 */
IndexedDBAdapter.prototype.createTables = function(event) {};

/**
 * Marks initialization of the adapter as completed successfully or not.
 * @param {Boolean} ready True if the adapter is ready; false if the adapter is in permanent error state.
 * @param {String} errorMessage If ready is false, the error message describing cause of the initialization failure.
 * @param {Object} transactionTimer measuring time taken to open/initialize db.
 * @private
 */
IndexedDBAdapter.prototype.initializeComplete = function(ready, errorMessage, transactionTimer) {};

/**
 * Remove customer information from keys (action params). Used prior to logging.
 * @param {String[]} keys The keys to be cleansed
 * @private
 */
IndexedDBAdapter.prototype.cleanseKeys = function(keys) {};

/**
 * Retrieves items from storage.
 * @param {String[]} keys The keys of the items to retrieve.
 * @param {Function} resolve Promise resolve function.
 * @param {Function} reject Promise resolve function.
 * @private
 */
IndexedDBAdapter.prototype.getItemsInternal = function(keys, resolve, reject) {};

/**
 * Walks everything in the DB (read only).
 * @param {Function} resolve Promise resolve function
 * @param {Function} reject Promise reject function
 * @param {Boolean} sendResult True to resolve the promise with the full set of results; false to resolve with the size.
 * @private
 */
IndexedDBAdapter.prototype.walkInternal = function(resolve, reject, sendResult) {};


/**
 * Stores items in storage.
 * @param {Array} tuples An array of key-value-size pairs.
 * @returns {Promise} A promise that resolves when the items are stored.
 */
IndexedDBAdapter.prototype.setItems = function(tuples) {};


/**
 * Removes items from storage.
 * @param {String[]} keys The keys of the items to remove.
 * @returns {Promise} A promise that resolves when all items are removed.
 */
IndexedDBAdapter.prototype.removeItems = function(keys) {};


/**
 * Clears storage.
 * @returns {Promise} A promise that resolves when the store is cleared.
 */
IndexedDBAdapter.prototype.clear = function() {};


/**
 * Evicts entries and updates the cached size of the store.
 *
 * Entries are evicted until requested size is freed. Algorithm evicts
 * items based on age and not sharing the key prefix (aka isolation key). An
 * LRU algorithm is not used which differentiates this adapter from others.
 *
 * The rest of the store is traversed to calculate the real size of the
 * persisted data.
 *
 * @param {Number} requestedSize the size to free in bytes
 * @param {Function} resolve promise resolve function
 * @param {Function} reject promise reject function
 * @private
 */
IndexedDBAdapter.prototype.expireCache = function(requestedSize, resolve, reject) {};


/**
 * Updates the guessed size of the store.
 * @param {Number} sizeChange The amount to change the size of the store.
 * @param {Number} error A really random guess of the size of the error.
 * @private
 */
IndexedDBAdapter.prototype.updateSize = function(sizeChange, error) {};


/**
 * Refreshes the cached size of the store from real data.
 * @param {Number} size The actual calculated size.
 * @param {Number} count The number of items in the store.
 * @private
 */
IndexedDBAdapter.prototype.refreshSize = function(size, count) {};


/**
 * Sets the cached size of the store. Callers must provide sizes based
 * on real data, not estimates.
 * @param {number} size The actual calculated size.
 * @param {number} count The number of items in the store.
 * @private
 */
IndexedDBAdapter.prototype.setSize = function(size, count) {};


/**
 * Converts a stored item to a shape appropriate for return to AuraStorage.
 * @param {Object} stored An item stored within IndexedDB.
 * @return {Object} A payload to return to AuraStorage.
 * @private
 */
IndexedDBAdapter.prototype.decodeStorable = function(stored) {};


/**
 * Converts a tuple to a storable item for IndexedDB.
 * @param {Object} tuple The item to store.
 * @return {Object} An item to store.
 * @private
 */
IndexedDBAdapter.prototype.encodeStorable = function(tuple) {};


/**
 * Logs a message.
 * @param {IndexedDBAdapter.LOG_LEVEL} level Log line level.
 * @param {String} msg The log message.
 * @param {Object=} obj Optional log payload.
 * @private
 */
IndexedDBAdapter.prototype.log = function (level, msg, obj) {};

/**
 * Returns a timer object to emit a metric.
 * @param name of the metricsService transaction.
 * @return object with 'end' function to be called with an object, any keys/values will be logged in the metricsService transaction.
 * @private
 */
IndexedDBAdapter.prototype.thresholdMetricTimer = function(name) {};


/**
 * Deletes the ENTIRE DB which may contain ObjectStores belonging to other app/cmp.
 * TODO W-2691320 - change db vs store name to avoid this issue.
 * @return {Promise} A promise that deletes the entire database
 */
IndexedDBAdapter.prototype.deleteStorage = function() {};


/**
 * Registers IndexedDB adapter.
 */
IndexedDBAdapter.register = function() {};

IndexedDBAdapter.register();

Aura.Storage.IndexedDBAdapter = IndexedDBAdapter;
