/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * @description The Javascript memory adapter for Aura Storage Service.
 * This adapter must always be registered to ensure at least one adapter is available.
 *
 * @constructor
 */
var MemoryAdapter = function MemoryAdapter(config) {};

MemoryAdapter.NAME = "memory";

/** Log levels */
MemoryAdapter.LOG_LEVEL = {
    INFO:    { id: 0, fn: "log" },
    WARNING: { id: 1, fn: "warning" }
};

/**
 * Resets memory objects.
 */
MemoryAdapter.prototype.reset = function() {};

/**
 * Gets the name of the adapter.
 * @returns {String} Name of adapter
 */
MemoryAdapter.prototype.getName = function() {};

/**
 * Starts the initialization process.
 * @return {Promise} a promise that resolves when initialization has completed, or rejects if initialization has failed.
 */
MemoryAdapter.prototype.initialize = function() {};

/**
 * Gets the adapter's size.
 * @returns {Promise} A promise that resolves with the size in bytes.
 */
MemoryAdapter.prototype.getSize = function() {};

/**
 * Retrieves items from storage.
 * @param {String[]} [keys] The set of keys to retrieve. Undefined to retrieve all items.
 * @param {Boolean} [includeExpired] True to return expired items, false to not return expired items.
 * @returns {Promise} A promise that resolves with an object that contains key-value pairs.
 */
MemoryAdapter.prototype.getItems = function(keys /* , includeExpired*/) {};

/**
 * Updates a key in the most recently used list.
 * @param {String} key the key to update
 */
MemoryAdapter.prototype.updateMRU = function(key) {};

/**
 * Stores items in storage.
 * @param {Array} tuples An array of key-value-size pairs. Eg <code>[ [key1, value1, size1], [key2, value2, size2] ]</code>.
 * @returns {Promise} A promise that resolves when the items are stored.
 */
MemoryAdapter.prototype.setItems = function(tuples) {};

/**
 * Removes items from storage.
 * @param {String[]} keys Keys of the values to remove.
 * @returns {Promise} A promise that resolves when the values are moved.
 */
MemoryAdapter.prototype.removeItems = function(keys) {};

/**
 * Removes an item from storage.
 * @param {String} key key for item to remove
 */
MemoryAdapter.prototype.removeItemInternal = function(key) {};

/**
 * Clears storage.
 * @returns {Promise} a promise that resolves when the store is cleared
 */
MemoryAdapter.prototype.clear = function() {};

/**
 * Evicts items. Expired items are evicted first. If additional space is required then
 * items are evicted based on LRU.
 * @param {Number} spaceNeeded The amount of space to free. Specify 0 to remove only expired items.
 * @returns {Promise} a promise that resolves when the requested space is freed.
 */
MemoryAdapter.prototype.expireCache = function(spaceNeeded) {};

/**
 * Gets the most-recently-used list.
 * @returns {Promise} a promise that results with the an array of keys representing the MRU.
 */
MemoryAdapter.prototype.getMRU = function() {};

/**
 * Logs a message.
 * @param {MemoryAdapter.LOG_LEVEL} level log line level
 * @param {String} msg the log message.
 * @param {Object} [obj] optional log payload.
 * @private
 */
MemoryAdapter.prototype.log = function (level, msg, obj) {};


/**
 * Removes expired items
 * @returns {Promise} when sweep completes
 */
MemoryAdapter.prototype.sweep = function() {};

/**
 * Deletes this storage.
 * @returns {Promise} a promise that resolves when storage is deleted
 */
MemoryAdapter.prototype.deleteStorage = function() {};

/**
 * @returns {Boolean} whether the adapter is secure.
 */
MemoryAdapter.prototype.isSecure = function() {};

/**
 * @returns {Boolean} whether the adapter is persistent.
 */
MemoryAdapter.prototype.isPersistent = function() {};


/**
 * @description A cache entry in the backing store of the MemoryAdapter.
 * Items which are not JSON serializable (eg cyclic graphs) will throw.
 * The caller must handle this.
 *
 * @constructor
 * @private
 */
MemoryAdapter.Entry = function Entry(item, size) {};

/**
 * @returns {Object} the stored item
 */
MemoryAdapter.Entry.prototype.getItem = function() {};

/**
 * @returns {Number} the size of the cache entry in bytes
 */
MemoryAdapter.Entry.prototype.getSize = function() {};


$A.storageService.registerAdapter({
    "name": MemoryAdapter.NAME,
    "adapterClass": MemoryAdapter,
    "secure": true
});

Aura.Storage.MemoryAdapter = MemoryAdapter;
