/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * @classdesc Utility functions for component testing, accessible using $A.test.
 * @constructor Test
 * @export
 */
function TestInstance() {}

/**
 * The set of errors accumulated.
 *
 * Note that this set of errors is 'static', since it can be accessed from window.onerror even before the $A.test
 * instance is initialized. We are careful to update it globally on the prototype instead of in the class.
 *
 * @private
 * @memberof Test
 */
TestInstance.prototype.errors = [];

// #include aura.test.Test_private

/**
 *
 * @description Asynchronously wait for a condition before continuing with the next stage of the test case. The wait
 *              condition is checked after the current test stage is completed but before the next stage is started.
 *
 * @example
 * $A.test.addWaitFor("I was updated.", function(){
 *   return element.textContent;
 *  }, function(){alert("the wait is over"});
 *
 * @param {Object}
 *            expected The value to compare against. If expected is a function, it will evaluate it before comparison.
 * @param {Object}
 *            testFunction A function to evaluate and compare against expected.
 * @param {Function}
 *            callback Invoked after the comparison evaluates to true
 * @export
 * @function Test#addWaitFor
 */
TestInstance.prototype.addWaitFor = function(expected, testFunction, callback) {};


/**
 *
 * @description Asynchronously wait for a condition before continuing with the next stage of the test case. The wait
 *              condition is checked after the current test stage is completed but before the next stage is started.
 *
 *  @example
 *  $A.test.addWaitForWithFailureMessage("i was updated", function(){<br/>
 *   return element.textContent;},"Failure Message", function(){alert("the wait is over"});
 *
 * @param {Object}
 *            expected The value to compare against. If expected is a function, it will evaluate it before comparison.
 * @param {Object}
 *            testFunction A function to evaluate and compare against expected.
 * @param {String}
 *            failureMessage The message that is returned if the condition is not true
 * @param {Function}
 *            callback Invoked after the comparison evaluates to true
 * @export
 * @function Test#addWaitForWithFailureMessage
 */
TestInstance.prototype.addWaitForWithFailureMessage = function(expected, testFunction, failureMessage, callback) {};

/**
 * Block requests from being sent to the server.
 *
 * This routine can be used to artificially force actions to be held on the client to be sent to the server at a later
 * date. It can be used to simulate delays in processing (or rapid action queueing on the client).
 *
 * @export
 * @function Test#blockRequests
 */
TestInstance.prototype.blockRequests = function() {};

/**
 * Block only foreground actions from being sent to the server.
 *
 * @export
 * @function Test#blockForegroundRequests
 */
TestInstance.prototype.blockForegroundRequests = function() {};

/**
 * Block only background actions from being sent to the server.
 *
 * @export
 * @function Test#blockBackgroundRequests
 */
TestInstance.prototype.blockBackgroundRequests = function() {};

/**
 * Release requests to be sent to the server.
 *
 * This must be called after blockRequests, otherwise it may result in unknown consequences.
 *
 * @export
 * @function Test#releaseRequests
 */
TestInstance.prototype.releaseRequests = function() {};

/**
 * Release only foreground requests from being sent to the server.
 *
 * Callers must be aware of what requests are currently blocked. Releasing requests that are not blocked will result in
 * unknown consequences.
 *
 * @export
 * @function Test#releaseForegroundRequests
 */
TestInstance.prototype.releaseForegroundRequests = function() {};

/**
 * Release only background actions from being sent to the server.
 *
 * Callers must be aware of what requests are currently blocked. Releasing requests that are not blocked will result in
 * unknown consequences.
 *
 * @export
 * @function Test#releaseBackgroundRequests
 */
TestInstance.prototype.releaseBackgroundRequests = function() {};

/**
 * Get total count of foreground and background requests sent to the server.
 *
 * This routine can be used to get a before and after count on server requests to attempt to verify we are only sending
 * the necessary amount of requests.
 *
 * @export
 * @function Test#getSentRequestCount
 */
TestInstance.prototype.getSentRequestCount = function() {};

/**
 * Get the total number of uri-addressable definitions that have been requested
 * @export
 */
TestInstance.prototype.getRequestedComponentDefCount = function() {};


/**
 * Check to see if an array of actions have all completed.
 *
 * @export
 * @function Test#areActionsComplete
 */
TestInstance.prototype.areActionsComplete = function(actions) {};

/**
 * Add a cleanup function that is run on teardown.
 *
 * @param {Function}
 *            cleanupFunction the function to run on teardown.
 * @export
 * @function Test#addCleanup
 */
TestInstance.prototype.addCleanup = function(cleanupFunction) {};

/**
 * Get an instance of an action based on the specified parameters and callback function.
 *
 * @param {Component}
 *            component The Component on which to search for the action
 * @param {String}
 *            name The name of the action from the component's perspective (e.g. "c.doSomething")
 * @param {Object}
 *            params The parameters to pass to the action
 * @param {Function}
 *            callback The callback function to execute for the action, or if not a function a name for the action
 * @returns {Action} An instance of the action
 * @export
 * @function Test#getAction
 */
TestInstance.prototype.getAction = function(component, name, params, callback) {};

/**
 * Enqueue an action, ensuring that it is safely inside an aura call.
 *
 * @param {Action}
 *            action The action to enqueue.
 * @param {Boolean}
 *            background Set to true to run the action in the background, otherwise the value of action.isBackground()
 *            is used.
 * @export
 * @function Test#enqueueAction
 */
TestInstance.prototype.enqueueAction = function(action, background) {};

/**
 *
 * @description Get an instance of a server action that is not available to the component.
 *
 * @example
 * $A.test.getExternalAction(cmp, "aura://ComponentController/ACTION$getComponent",
 *          {name:"aura:text", attributes:{value:"valuable"}},
 *          "java://org.auraframework.instance.component",
 *          function(action){alert(action.getReturnValue().attributes.values.value)})
 *
 * @param {Component} component
 *            The scope to run the action with, even if the action is not visible to it
 * @param {String} descriptor
 *            The descriptor for the action - e.g. java://my.own.Controller/ACTION$doIt
 * @param {Object} params
 *            The parameters to pass to the action, as a Map (name:value)
 * @param {Object} returnType
 *            The return type descriptor for the action, e.g. java://java.lang.String
 * @param {Function} callback
 *            An optional callback to execute with the component as the scope
 * @returns {Action} an instance of the action
 * @export
 * @function Test#getExternalAction
 */
TestInstance.prototype.getExternalAction = function(component, descriptor, params, returnType, callback) {};

/**
 * Clear out component configs returned by an action.
 *
 * This must be called within the action callback. It fails if no components are cleared.
 *
 * @param {Action}
 *            action The action to clear.
 * @export
 * @function Test#clearAndAssertComponentConfigs
 */
TestInstance.prototype.clearAndAssertComponentConfigs = function(a) {};

/**
 * Peek if there are any pending server actions.
 *
 * NOTE: this is used as a predicate and does not have access to 'this'. If this function changes to require 'this',
 * either the uses will need to be refactored, or isActionPending will need to be auto-bound.
 *
 * @returns {Boolean} Returns true if there are pending server actions, or false otherwise.
 * @export
 * @function Test#isActionPending
 */
TestInstance.prototype.isActionPending = function() {};

/**
 * Peek if there are any queued server actions.
 *
 * NOTE: this is used as a predicate and does not have access to 'this'. If this function changes to require 'this',
 * either the uses will need to be refactored, or isActionQueued will need to be auto-bound.
 *
 * @returns {Boolean} Returns true if there are pending server actions, or false otherwise.
 * @export
 * @function Test#isActionQueued
 */
TestInstance.prototype.isActionQueued = function() {};

/**
 * Invoke a server action. At the end of current test case stage, the test will wait for any actions to complete before
 * continuing to the next stage of the test case.
 *
 * @param {Action}
 *            action The action to invoke
 * @param {Boolean}
 *            doImmediate If set to true, the request will be sent immediately, otherwise the action will be handled as
 *            any other Action and may be queued behind prior requests.
 * @export
 * @function Test#callServerAction
 */
TestInstance.prototype.callServerAction = function(action, doImmediate) {};

/**
 * Set whether the server is reachable, to mimick being offline.
 *
 * Note that this will not work with IE < 10 (see W-2537764).
 *
 * @param {Boolean}
 *            reachable True or absent to make the server reachable; otherwise the server is made unreachable.
 * @export
 * @function Test#setServerReachable
 */
TestInstance.prototype.setServerReachable = function(reachable) {};

/**
 * @param xhrExclusivity boolean for xhr's to be run exclusively (one at a time)
 * @export
 */
TestInstance.prototype.setXHRExclusivity = function(xhrExclusivity) {};

/**
 * Invoke a callback after the provided condition evaluates to truthy, checking on the condition every specified
 * interval. Truthy values can refer to a non-empty String, a non-zero number, a non-empty array, an object, or an
 * expression evaluating to true.
 *
 * @param {Function}
 *            conditionFunction The function to evaluate
 * @param {Function}
 *            callback The callback function to run if conditionFunction evaluates to truthy
 * @param {Number}
 *            intervalInMs The number of milliseconds between each evaluation of conditionFunction
 * @export
 * @function Test#runAfterIf
 */
TestInstance.prototype.runAfterIf = function(conditionFunction, callback, intervalInMs) {};

/**
 * Set test to timeout in a period of milliseconds from now, clearing the existing timeout.
 *
 * @param {Number}
 *            timeoutMs The number of milliseconds from now in which the test should timeout
 * @export
 * @function Test#setTestTimeout
 */
TestInstance.prototype.setTestTimeout = function(timeoutMs) {};

/**
 * Return whether the test is finished.
 *
 * @returns {Boolean} Returns true if the test has completed, or false otherwise.
 * @export
 * @function Test#isComplete
 */
TestInstance.prototype.isComplete = function() {};

/**
 * Get the list of errors seen by the test, not including any errors handled explicitly by the framework.
 *
 * @returns {string} Returns an empty string if no errors are seen, else a json encoded list of errors
 * @export
 * @function Test#getErrors
 */
TestInstance.prototype.getErrors = function() {};

/**
 * Essentially a toString method, except strings are enclosed with double quotations. Returns a string even for
 * undefined/null value.
 *
 * @param {Object}
 *            value The value that will be converted to a String
 * @returns {String} The value that is returned as a String type
 * @private
 * @function Test#print
 */
TestInstance.prototype.print = function(value) {};

/**
 * Internally used error function to log an error for a given test.
 *
 * @param level
 *            ERROR
 * @param msg
 *            message to display this is being called by Logger.prototype.notify
 * @private
 * @function Test#auraError
 */
TestInstance.prototype.auraError = function(level, msg/* , error */) {};

/**
 * Warning- use this function with care. Tell the test that we expect an $A.auraError that occurs in a separate thread
 * than the main test thread. Any errors occurring in the main test thread should be caught and verified in the test
 * itself.
 *
 * Test will fail if expected error is not received.
 *
 * @param {string}
 *            e The error message that we expect.
 * @export
 * @function Test#expectAuraError
 */
TestInstance.prototype.expectAuraError = function(e) {};

/**
 * Internally used warning function to log a warning for a given test.
 *
 * @param level
 *            WARNING
 * @param msg
 *            message to display this is being called by Logger.prototype.notify
 * @private
 * @function Test#auraWarning
 */
TestInstance.prototype.auraWarning = function(level, msg) {};

/**
 * Tell the test that we expect a warning. If this function is called and the test does not receive the expected
 * warning, the test will fail.
 *
 * @param {String}
 *            w the warning message that we expect.
 * @export
 * @function Test#expectAuraWarning
 */
TestInstance.prototype.expectAuraWarning = function(w) {};

/**
 * Assert that the current component HTML is Accessibility compliant.
 *
 * @description Calls the checkAccessibilty method to verify certain tags are accessible.
 *
 * @param {String}
 *            errorMessage The message that is returned if the condition is not false
 * @throws {Error}
 *             Throws Error containing concatenated string representation of all accessibility errors found
 * @export
 * @function Test#assertAccessible
 */
TestInstance.prototype.assertAccessible = function() {};

/**
 *
 * @description Assert that if(condition) check evaluates to true.
 * A truthy value refers to an Object, a string, a non-zero number, a non-empty array, or true.
 *
 * @example
 * assertTruthy("helloWorld"); // Positive
 * assertTruthy(null); // Negative
 *
 * @param {Object} condition The condition to evaluate
 * @param {String} assertMessage The message that is returned if the condition is not true
 * @export
 */
TestInstance.prototype.assertTruthy = function(condition, assertMessage) {};

/**
 * @description A falsey value refers to zero, an empty string, null, undefined, or false.
 * Assert that the if(condition) check evaluates to false.
 *
 * @param {Object} condition The condition to evaluate
 * @param {String} assertMessage The message that is returned if the condition is not false
 *
 * @example
 * assertFalsy("helloWorld"); // Negative
 * assertFalsy(null); // Positive
 *
 * @export
 * @function Test#assertFalsy
 */
TestInstance.prototype.assertFalsy = function(condition, assertMessage) {};

/**
 * @description Assert that if(condition) check evaluates to true.
 *
 * @param {Object} condition The condition to evaluate
 * @param {String} assertMessage The message that is returned if the condition is not true
 *
 *
 * @example
 * assert("helloWorld"); // Positive
 * assert(null); // Negative
 * @export
 */
TestInstance.prototype.assert = function(condition, assertMessage) {};

/**
 * Assert that the two values provided are equal.
 *
 * @param {Object}
 *            arg1 The argument to evaluate against arg2
 * @param {Object}
 *            arg2 The argument to evaluate against arg1
 * @param {String}
 *            assertMessage The message that is returned if the two values are not equal
 * @export
 * @function Test#assertEquals
 */
TestInstance.prototype.assertEquals = function(arg1, arg2, assertMessage) {};

/**
 * Assert that the two string values provided are equal ignoring whitespace.
 *
 * This is important when checking constructed strings, as browsers may handle them differently.
 *
 * @param {string}
 *            arg1 The argument to evaluate against arg2
 * @param {string}
 *            arg2 The argument to evaluate against arg1
 * @param {String}
 *            assertMessage The message that is returned if the two values are not equal
 * @export
 * @function Test#assertEqualsIgnoreWhitespace
 */
TestInstance.prototype.assertEqualsIgnoreWhitespace = function(arg1, arg2, assertMessage) {};

/**
 * Assert that a string starts with another.
 *
 * @param {Object}
 *            start The start string.
 * @param {Object}
 *            full The string that is expected to start with the start string
 * @param {String}
 *            assertMessage The message that is returned if the two values are not equal
 * @export
 * @function Test#assertStartsWith
 */
TestInstance.prototype.assertStartsWith = function(start, full, assertMessage) {};

/**
 * Complement of assertEquals, throws Error if arg1===arg2.
 *
 * @param {Object}
 *            arg1 The argument to evaluate against arg2
 * @param {Object}
 *            arg2 The argument to evaluate against arg1
 * @param {String}
 *            assertMessage The message that is returned if the two values are equal
 * @export
 * @function Test#assertNotEquals
 */
TestInstance.prototype.assertNotEquals = function(arg1, arg2, assertMessage) {};

/**
 * Assert that the value is not undefined.
 *
 * @param {Object}
 *            condition The argument to evaluate
 * @param {String}
 *            assertMessage The message that is returned if arg1 is undefined
 * @export
 * @function Test#assertDefined
 */
TestInstance.prototype.assertDefined = function(condition, assertMessage) {};

/**
 * Assert that the condition === true.
 *
 * @param {Boolean}
 *            condition The condition to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the condition !==true
 * @export
 * @function Test#assertTrue
 */
TestInstance.prototype.assertTrue = function(condition, assertMessage) {};

/**
 * Assert that the condition === false.
 *
 * @param {Boolean}
 *            condition The condition to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the condition !==false
 * @export
 * @function Test#assertFalse
 */
TestInstance.prototype.assertFalse = function(condition, assertMessage) {};

/**
 * Assert that the value passed in is undefined.
 *
 * @param {Object}
 *            condition The argument to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the argument is not undefined
 * @export
 * @function Test#assertUndefined
 */
TestInstance.prototype.assertUndefined = function(condition, assertMessage) {};

/**
 * Assert that the value passed in is not undefined or null.
 *
 * @param {Object}
 *            condition The argument to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the argument is not undefined or null
 * @export
 * @function Test#assertNotUndefinedOrNull
 */
TestInstance.prototype.assertNotUndefinedOrNull = function(condition, assertMessage) {};

/**
 * Assert that the value passed in is either undefined or null.
 *
 * @param {Object}
 *            condition The argument to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the argument is not undefined or null
 * @export
 * @function Test#assertUndefinedOrNull
 */
TestInstance.prototype.assertUndefinedOrNull = function(condition, assertMessage) {};

/**
 * Assert that value === null.
 *
 * @param {Object}
 *            condition The argument to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the value !==null
 * @export
 * @function Test#assertNull
 */
TestInstance.prototype.assertNull = function(condition, assertMessage) {};

/**
 * Assert that value !== null.
 *
 * @param {Object}
 *            condition The argument to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the value is null
 * @export
 * @function Test#assertNotNull
 */
TestInstance.prototype.assertNotNull = function(condition, assertMessage) {};

/**
 * Assert that the value is an instance of the expected type.
 *
 * @param {String}
 *            type The type expected
 * @param {Object}
 *            condition The argument to evaluate
 * @param {String}
 *            assertMessage The message that is returned if the value is null
 * @export
 * @function Test#assertAuraType
 */
TestInstance.prototype.assertAuraType = function(type, condition, assertMessage) {};


/**
 * Throw an Error, making a test fail with the specified message.
 *
 * @param {String}
 *            assertMessage Defaults to "Assertion failure", if assertMessage is not provided
 * @param {String}
 *            extraInfoMessage
 * @throws {Error}
 *             Throws error with a message
 * @export
 * @function Test#fail
 */
TestInstance.prototype.fail = function(assertMessage, extraInfoMessage) {};

/**
 * Get an object's prototype.
 *
 * @param {Object}
 *            instance The instance of the object
 * @returns {Object} The prototype of the specified object
 * @export
 * @function Test#getPrototype
 */
TestInstance.prototype.getPrototype = function(instance) {};

/**
 * Replace a function on an object with a restorable override.
 *
 * @param {Object}
 *            instance The instance of the object
 * @param {String}
 *            name The name of the function to be replaced
 * @param {Function}
 *            newFunction The new function that replaces originalFunction
 * @returns {Function} The override (newFunction) with an added "restore" function that, when invoked, will restore
 *          originalFunction on instance
 * @throws {Error}
 *             Throws an error if instance does not have originalFunction as a property
 * @export
 * @function Test#overrideFunction
 */
TestInstance.prototype.overrideFunction = function(instance, name, newFunction) {};

/**
 * Add a handler function to an existing object's function. The handler may be attached before or after the target
 * function. If attached after (postProcess === true), the handler will be invoked with the original function's return
 * value followed by the original arguments. If attached before (postProcess !== true), the handler will be invoked with
 * just the original arguments.
 *
 * @param {Object}
 *            instance The instance of the object
 * @param {String}
 *            name The name of the function whose arguments are applied to the handler
 * @param {Function}
 *            newFunction The target function to attach the handler to
 * @param {Boolean}
 *            postProcess Set to true if the handler will be called after the target function or false if the handler
 *            will be called before originalFunction
 * @returns {Function} The override of originalFunction, which has a "restore" function that, when invoked, will restore
 *          originalFunction on instance
 * @export
 * @function Test#addFunctionHandler
 */
TestInstance.prototype.addFunctionHandler = function(instance, name, newFunction, postProcess) {};

/**
 * Get a DOM node's outerHTML.
 *
 * @param {Node}
 *            node The node to get outer HTML from
 * @returns {String} The outer HTML
 * @export
 * @function Test#getOuterHtml
 */
TestInstance.prototype.getOuterHtml = function(node) {};

/**
 * Get the text content of a DOM node. Tries <code>textContent</code> followed by <code>innerText</code>, followed
 * by <code>nodeValue</code> to take browser differences into account.
 *
 * @param {Node} node The node to get the text content from
 * @returns {String} The text content of the specified DOM node or empty string if unable to extract text
 * @export
 * @function Test#getText
 */
TestInstance.prototype.getText = function(node) {};

/**
 * Get the textContent of all elements rendered by this component.
 *
 * @param {Component}
 *            component The component to get the text content from
 * @returns {String} The text content of the specified component
 * @export
 * @function Test#getTextByComponent
 */
TestInstance.prototype.getTextByComponent = function(component) {};

/**
 * Get the current value for a style for a DOMElement.
 *
 * @param {DOMElement}
 *            elem The element to get the CSS property value from
 * @param {String}
 *            Style The property name to retrieve
 * @returns {String} The CSS property value of the specified DOMElement
 * @export
 * @function Test#getStyle
 */
TestInstance.prototype.getStyle = function(elem, style) {};

/**
 * Filter out comment nodes from a list of nodes.
 *
 * @param {Array|Object}
 *            nodes The list of nodes to filter
 * @returns {Array} The list of nodes without comment nodes
 * @export
 * @function Test#getNonCommentNodes
 */
TestInstance.prototype.getNonCommentNodes = function(nodes) {};

/**
 * Check if a node has been "deleted" by Aura.
 *
 * @param {Node}
 *            node The node to check
 * @returns {Boolean} Returns true if the specified node has been deleted, or false otherwise
 * @export
 * @function Test#isNodeDeleted
 */
TestInstance.prototype.isNodeDeleted = function(node) {};

/**
 * Return a node list and pass each argument as a separate parameter.
 *
 * @returns {Array} The list of nodes contained in the document node
 * @export
 * @function Test#select
 */
TestInstance.prototype.select = function() {};

/**
 * Check if a string contains another string.
 *
 * @param {String}
 *            testString The string to check
 * @param {String}
 *            targetString The string to look for within testString
 * @returns {Boolean} Return true if testString contains targetString, or false otherwise
 * @export
 * @function Test#contains
 */
TestInstance.prototype.contains = function(testString, targetString) {};

/**
 * Compares values. In the case of an Array or Object, compares first level references only. In the case of a literal,
 * directly compares value and type equality.
 *
 * @param {Object}
 *            expected The source value to compare.
 * @param {Object}
 *            actual The target value to compare.
 * @returns {Object} The result of the comparison, with reasons.
 * @export
 * @function Test#compareValues
 */
TestInstance.prototype.compareValues = function(expected, actual) {};

/**
 * Returns a reference to the object that is currently designated as the active element in the document.
 *
 * @returns {DOMElement} The current active element.
 * @export
 * @function Test#getActiveElement
 */
TestInstance.prototype.getActiveElement = function() {};

/**
 * Returns the inner text of the current active element in the document.
 *
 * @returns {String} The text of the current active DOM element.
 * @export
 * @function Test#getActiveElementText
 */
TestInstance.prototype.getActiveElementText = function() {};

/**
 * Used by getElementsByClassNameCustom for IE7
 *
 * @private
 * @function Test#walkTheDOM
 */
TestInstance.prototype.walkTheDOM = function(node, func) {};

/**
 * custom util to get element by class name for IE7
 *
 * @private
 * @function Test#getElementsByClassNameCustom
 */
TestInstance.prototype.getElementsByClassNameCustom = function(className, parentElement) {};

/**
 * Gets the first element on the page starting from parentElement, that has the specified class name.
 *
 * @param {Object}
 *            parentElement DOM element that we want to start at.
 * @param {String}
 *            classname The CSS class name.
 * @returns {Object} The first element denoting the class, or null if none is found.
 * @export
 * @function Test#findChildWithClassName
 */
TestInstance.prototype.findChildWithClassName = function(parentElement, className) {};

/**
 * Gets the first element on the page that have the specified class name.
 *
 * @param {String}
 *            classname The CSS class name.
 * @returns {Object} The element denoting the class, or null if none is found.
 * @export
 * @function Test#getElementByClass
 */
TestInstance.prototype.getElementByClass = function(classname) {};

/**
 * Given an HTML element and an eventName, fire the corresponding DOM event. Code adapted from a stack overflow
 * question's answer.
 *
 * @param {Object}
 *            element The HTML element whose corresponding DOM event is to be fired.
 * @param {String}
 *            eventName Initializes the given event that bubbles up through the event chain.
 * @param {Boolean}
 *            canBubble Optional. True if the event can be bubbled, defaults to true.
 * @param {Boolean}
 *            cancelable Optional. Indicates whether the event is cancelable or not, defaults to true.
 * @export
 * @function Test#fireDomEvent
 */
TestInstance.prototype.fireDomEvent = function(element, eventName, canBubble, cancelable) {};

/**
 * Issue a click on the element.
 *
 * @param {HTMLElement}
 *            element The element to click on.
 * @param {Boolean}
 *            canBubble true to allow bubbling of the click.
 * @param {Boolean}
 *            cancelable Indicates whether the event is cancelable or not.
 * @export
 * @function Test#clickOrTouch
 */
TestInstance.prototype.clickOrTouch = function(element, canBubble, cancelable) {};

/**
 * Checks if the specified node is a text node.
 *
 * @param {Node}
 *            node The node to check
 * @returns {Boolean} true if node is text node.
 * @export
 * @function Test#isInstanceOfText
 */
TestInstance.prototype.isInstanceOfText = function(node) {};

/**
 * Checks if the specified element is an anchor element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is an anchor element.
 * @export
 * @function Test#isInstanceOfAnchorElement
 */
TestInstance.prototype.isInstanceOfAnchorElement = function(element) {};

/**
 * Checks if the specified element is an input element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is an input element.
 * @export
 * @function Test#isInstanceOfInputElement
 */
TestInstance.prototype.isInstanceOfInputElement = function(element) {};

/**
 * Checks if the specified element is a list element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is a list element.
 * @export
 * @function Test#isInstanceOfLiElement
 */
TestInstance.prototype.isInstanceOfLiElement = function(element) {};

/**
 * Checks if the specified element is a paragraph element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is a paragraph element.
 * @export
 * @function Test#isInstanceOfParagraphElement
 */
TestInstance.prototype.isInstanceOfParagraphElement = function(element) {};

/**
 * Checks if the specified element is a button element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is a button element.
 * @export
 * @function Test#isInstanceOfButtonElement
 */
TestInstance.prototype.isInstanceOfButtonElement = function(element) {};

/**
 * Checks if the specified element is an image element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is an image element.
 * @export
 * @function Test#isInstanceOfImageElement
 */
TestInstance.prototype.isInstanceOfImageElement = function(element) {};

/**
 * Checks if the specified element is a div element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is a div element.
 * @export
 * @function Test#isInstanceOfDivElement
 */
TestInstance.prototype.isInstanceOfDivElement = function(element) {};

/**
 * Checks if the specified element is a span element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @returns {Boolean} true if element is a span element.
 * @export
 * @function Test#isInstanceOfSpanElement
 */
TestInstance.prototype.isInstanceOfSpanElement = function(element) {};

/**
 * Checks if the specified element is an instance of another element.
 *
 * @param {HTMLElement}
 *            element The element to check
 * @param {HTMLElement}
 *            elementType Checks element against elementType
 * @param {String}
 *            tag Check element.tagName against tag
 * @returns {Boolean} true if element is of type elementType. Or if elementType is undefined, check element is of type
 *          ELEMENT_NODE and it's tagName is equal to tag
 * @export
 * @function Test#isInstanceOf
 */
TestInstance.prototype.isInstanceOf = function(element, elementType, tag) {};

/**
 * Return attributeValue of an element
 *
 * @param {HTMLElement}
 *            element The element from which to retrieve data.
 * @param {String}
 *            attributeName The name of attribute to look up on element.
 * @export
 * @function Test#getElementAttributeValue
 */
TestInstance.prototype.getElementAttributeValue = function(element, attributeName) {};

/**
 * Add an event handler. If component is specified, the handler will be applied to component events. If component is not
 * specified, the handler will be applied to application events.
 *
 * @param {String}
 *            eventName The registered name, for component events; the descriptor name for application events.
 * @param {Function}
 *            handler The function handler, which should expect the event as input.
 * @param {Component}
 *            component The component to add the handler on.
 * @param {Boolean}
 *            insert For component events only, insert the handler at the front of the list if true, otherwise at the
 *            end
 * @export
 * @function Test#addEventHandler
 */
TestInstance.prototype.addEventHandler = function(eventName, handler, component, insert) {};

// Used by tests to modify framework source to trigger JS last mod update
/**
 * @export
 * @ignore
 * @function Test#dummyFunction
 */
TestInstance.prototype.dummyFunction = function() {};

/**
 * Extract the error message from Aura error div(the grey error message on the page)
 *
 * @returns {String} The text of the Aura error
 * @export
 * @function Test#getAuraErrorMessage
 */
TestInstance.prototype.getAuraErrorMessage = function() {};

/**
 * Assert that the Access check failure message is as expected
 *
 * @param {String}
 *      errorMessage An Aura Error Message
 * @param {String}
 *      delimiter split input error message
 * @param {String}
 *      targetCmp A string containing component being accessed
 * @param {String}
 *      accessingCmp A string containing accessing component details
 *
 * @export
 * @function Test#getPopOverErrorMessage
 */
TestInstance.prototype.getPopOverErrorMessage = function(errorMessage, delimiter, targetCmp, accessingCmp) {};

/**
 * Override function for client service get XHR.
 *
 * @private
 * @function Test#getAvailableXHROverride
 */
TestInstance.prototype.getAvailableXHROverride = function(config, isBackground) {};

/**
 * Override send so that we can account for packets.
 *
 * @private
 * @function Test#sendOverride
 */
TestInstance.prototype.sendOverride = function(config, auraXHR, actions, method, options) {};

/**
 * Override decode.
 * The callback before Decode take response in, you can make a copy of it, made some modification, then return your response.
 * The callback after Decode take the result of decode (see AuraClientService.decode for what's inside), at this point, we
 * don't modify response.
 *
 * @private
 * @function Test#decodeOverride
 */
TestInstance.prototype.decodeOverride = function(config, response, noStrip, timeOut) {};

/**
 * A simple structure to hold action and callbacks.
 *
 * @struct
 * @private
 */
TestInstance.prototype.PrePostConfig = function (action, preSendCallback, postSendCallback, preDecodeCallback, postDecodeCallback) {};

/**
 * Add a pre/post send callback.
 *
 * This function allows a test to insert a hook either pre or post send of XHR.
 *
 * Note that for the post XHR callback the XHR has actually not been 'sent', but actions are serialized
 * and put in the actual request, so changing actions will have no effect at that point.
 *
 * @param action the action to watch for (undefined/null means any action)
 * @param preSendCallback the hook function for before send.
 * @param postSendCallback the hook function for after send.
 * one of preSendCallback and postSendCallback can be null, but not both of them
 * @return a handle(a PrePostConfig object) to remove the callback later (only needed if the first parameter:action is empty).
 *
 */
TestInstance.prototype.addPrePostSendCallback = function (action, preSendCallback, postSendCallback) {};

/**
 * Add a pre send callback.
 *
 * This function allows a test to insert a hook pre send of XHR.
 * @param action the action to watch for (undefined/null means any action)
 * @param preSendCallback the hook function for before send.
 * @return a handle to remove the callback (only needed if the first parameter:action is empty).
 *
 * @export
 * @function Test#addPreSendCallback
 */
TestInstance.prototype.addPreSendCallback = function (action, preSendCallback) {};

/**
 * Add a post send callback.
 *
 * This function allows a test to insert a hook post send of XHR.
 *
 * Note that for the post XHR callback the XHR has actually not been 'sent', but actions are serialized
 * and put in the actual request, so changing actions will have no effect at that point.
 *
 * @param action the action to watch for (undefined/null means any action)
 * @param postSendCallback the hook function for after send.
 * @return a handle to remove the callback (only needed if the first parameter:action is empty).
 *
 * @export
 * @function Test#addPostSendCallback
 */
TestInstance.prototype.addPostSendCallback = function (action, postSendCallback) {};

/**
 * Remove a previously added callback.
 *
 *TODO: Lin to remove public access to this api
 * @export
 * @function Test#removePrePostSendCallback
 */
TestInstance.prototype.removePrePostSendCallback = function (handle) {};

/**
 * Remove a previously added callback.
 *
 * @export
 * @function Test#removePreSendCallback
 */
TestInstance.prototype.removePreSendCallback = function (handle) {};

/**
 * Remove a previously added callback.
 *
 * @export
 * @function Test#removePostSendCallback
 */
TestInstance.prototype.removePostSendCallback = function (handle) {};


/**
 * Add a callback right before we decode response
 * @export
 * @function Test#addPreDecodeCallback
 */
TestInstance.prototype.addPreDecodeCallback = function (preDecodeCallback) {};

/**
 * Add a callback right after we decode response
 * @export
 * @function Test#addPostDecodeCallback
 */
TestInstance.prototype.addPostDecodeCallback = function (postDecodeCallback) {};


/**
 * Add a callback before/after decode response
 */
TestInstance.prototype.addPrePostDecodeCallback = function (preDecodeCallback, postDecodeCallback) {};

/**
 * Remove a previously added callback
 */
TestInstance.prototype.removePrePostDecodeCallback = function (handle) {};

/**
 * Remove a previously added callback
 * @export
 * @function Test#removePostDecodeCallback
 */
TestInstance.prototype.removePostDecodeCallback = function (handle) {};
/**
 * Remove a previously added callback
 * @export
 * @function Test#removePreDecodeCallback
 */
TestInstance.prototype.removePreDecodeCallback = function (handle) {};

/**
 * Install all of the overrides needed.
 *
 * @private
 * @function Test#install
 */
TestInstance.prototype.installOverride = function() {};


/**
 * Run the test
 *
 * @param {String}
 *            name The name of the test in the suite to run
 * @param {String}
 *            code The full test suite code
 * @param {Integer}
 *            timeoutOverride Optional. Use to increase the test timeout by specified time in seconds. If not set the
 *            test will use a default timeout of 10 seconds.
 *
 * @export
 * @function Test#run
 */
TestInstance.prototype.run = function(name, code, timeoutOverride, quickFixException) {};

/**
 * Get the test name.
 *
 * @export
 */
TestInstance.prototype.getTestName = function () {};

/**
 * @private
 */
TestInstance.prototype.waitForRoot = function(testName, callback) {};

/**
 * @private
 * @function Test#runInternal
 */
TestInstance.prototype.runInternal = function(name) {};

/**
 * Calls user-defined setUp method. Accepts a Promise as a return value and waits for promise to resolve or reject
 * before continuing.
 *
 * @private
 * @function Test#callSetUp
 */
TestInstance.prototype.callSetUp = function(callback) {};

/**
 * @description Asynchronously wait for CKEditor instance in inputRichText component to be ready before continuing to
 *              enter test data.
 *
 * @example <code>$A.test..executeAfterCkEditorIsReady(inputRichTextComponent, function(){<br/>
 *   inputRichTextComponent.set('v.value', 'tab1 content'); });</code>
 *
 * @param {Component}
 *            ui:inputRichText component, or a component that extends it, that you are entering data in.
 * @param {Function}
 *            callback Invoked after the CKEditor is ready for user input
 * @export
 * @function Test#executeAfterCkEditorIsReady
 */
TestInstance.prototype.executeAfterCkEditorIsReady = function(inputRichTextComponent, callback) {};

/**
 * Gets the Global Value Providers based on type.
 *
 * @param {String}
 *            gvp type to get Global Value Provider
 * @export
 * @function Test#getGlobalValueProviders
 */
TestInstance.prototype.getGlobalValueProvider = function(type) {};

/**
 * Sweeps expired items from an AuraStorage instance. Ignores minimum sweep intervals
 * but does not circumvent other restrictions (eg no concurrent sweeps).
 *
 * @param {AuraStorage} storage the storage to sweep.
 * @export
 * @function Test#storageSweep
 */
TestInstance.prototype.storageSweep = function(storage) {};

/**
 * Store items to storage through the storage's adapter, bypassing the key prefix and
 * size validation logic AuraStorage performs.
 *
 * @param {StorageAdapter} adapter
 * @param {Array} tuples An array of key-value-size pairs
 * @export
 * @function Test#storageAdapterSetItems
 */
TestInstance.prototype.storageAdapterSetItems = function(storage, tuples) {};

/**
 * Gets all definitions from ComponentDefStorage.
 *
 * @export
 * @function Test#getAllComponentDefFromStorage
 */
TestInstance.prototype.getAllComponentDefsFromStorage = function() {};

/**
 * @export
 * @function Test#getCreationPath
 */
TestInstance.prototype.getCreationPath = function(cmp) {};

/**
 * @export
 * @function Test#createHttpRequest
 */
TestInstance.prototype.createHttpRequest = function() {};

/**
 * Performs a check if global namespace is polluted with new
 * variables apart from whitelisted ones
 *
 * @export
 * @function Test#checkGlobalNamespacePollution
 */
TestInstance.prototype.checkGlobalNamespacePollution = function(whitelistedPollutants) {};

/**
 * sets the uri addressable definitions state
 * @param newState
 * @return previous state
 * @export
 */
TestInstance.prototype.setURIDefsState = function(newState) {};

/**
 * Json instance for test. Used to export Json methods for testing.
 *
 * @export
 * @memberof Test
 */
JsonTestInstance = function() {};

/**
 * Serializes object in alphabetical asc order. Sorts object keys during serialization.
 *
 * @param {Object}
 *            obj Object to be serialized
 * @returns {String} serialized order object
 * @export
 * @function Test#json.orderedEncode
 */
JsonTestInstance.prototype.orderedEncode = function(obj) {};

/**
 * Passthrough to JSON decode utility for tests.
 * @export
 */
JsonTestInstance.prototype.decode = function(obj, refSupport) {};

// -- Aura Bootstrap ------------------------------------------------------------

$A["test"] = new TestInstance();
$A["test"]["json"] = new JsonTestInstance();

$A.logger.subscribe("WARNING", $A["test"].auraWarning.bind($A["test"]));
$A.logger.subscribe("ERROR", $A["test"].auraError.bind($A["test"]));

/**
 * Register a global error handler to catch uncaught javascript errors.
 * @export
 * @ignore
 */
window.onerror = (function() {})();
